@@ -6,27 +6,34 @@
  * Open Phone Abstraction Library (OPAL)
  * Formally known as the Open H323 project.
  *
- * Copyright (C) 2004-2006 Post Increment
+ * Copyright (C) 2004-2011 Post Increment
  *
- * The contents of this file are subject to the Mozilla Public License
- * Version 1.0 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Software distributed under the License is distributed on an "AS IS"
- * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
- * the License for the specific language governing rights and limitations
- * under the License.
+ * - Redistributions of source code must retain the above copyright
+ *   notice, this list of conditions and the following disclaimer.
+
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
  *
- * The Original Code is Open Phone Abstraction Library.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * The Initial Developer of the Original Code is Post Increment
- *
- * Contributor(s): ______________________________________.
- *
- * $Revision: 25159 $
+ * $Revision: 26796 $
  * $Author: rjongbloed $
- * $Date: 2011-02-21 23:55:45 -0600 (Mon, 21 Feb 2011) $
+ * $Date: 2011-12-13 03:26:25 -0600 (Tue, 13 Dec 2011) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGIN_H
@@ -181,6 +188,9 @@ enum PluginCodec_Flags {
   PluginCodec_ChannelsMask           = 0x003f0000
 };
 
+#define PluginCodec_SetChannels(n) (((n-1)<<PluginCodec_ChannelsPos)&PluginCodec_ChannelsMask)
+
+
 enum PluginCodec_CoderFlags {
   PluginCodec_CoderSilenceFrame      = 1,    // request audio codec to create silence frame
   PluginCodec_CoderForceIFrame       = 2     // request video codec to force I frame
@@ -289,6 +299,7 @@ struct PluginCodec_Option {
 #define PLUGINCODEC_OPTION_CHANNELS                   "Channels"
 #define PLUGINCODEC_OPTION_FRAME_TIME                 "Frame Time"
 #define PLUGINCODEC_OPTION_MAX_FRAME_SIZE             "Max Frame Size"
+#define PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE         "Max Tx Packet Size"
 #define PLUGINCODEC_OPTION_MAX_BIT_RATE               "Max Bit Rate"
 #define PLUGINCODEC_OPTION_TARGET_BIT_RATE            "Target Bit Rate"
 #define PLUGINCODEC_OPTION_RX_FRAMES_PER_PACKET       "Rx Frames Per Packet"
@@ -368,7 +379,211 @@ struct PluginCodec_Definition {
 typedef struct PluginCodec_Definition * (* PluginCodec_GetCodecFunction)(unsigned int *, unsigned int);
 typedef unsigned (* PluginCodec_GetAPIVersionFunction)();
 
+
 ///////////////////////////////////////////////////////////////////
+
+#define PLUGINCODEC_RAW_AUDIO "L16"
+#define PLUGINCODEC_RAW_VIDEO "YUV420P"
+
+/// Declare a pair of plug in definition entries for a codec
+#define PLUGINCODEC_CODEC_PAIR(MediaFormat,    /**< Media Format */ \
+                               PayloadName,    /**< IANA RTP payload code */ \
+                               Description,    /**< Description text */ \
+                               SampleRate,     /**< Sample rate */ \
+                               BitsPerSecond,  /**< Maximum bits per second */ \
+                               FrameTime,      /**< Microseconds per frame */ \
+                               p1,p2,p3,p4, \
+                               PayloadType,    /**< IANA RTP payload type code */ \
+                               H323type,       /**< h323CapabilityType enumeration */ \
+                               H323data,       /**< Data to go with h323CapabilityType */ \
+                               CreateEncoder,  /**< Create encoder function */ \
+                               DestroyEncoder, /**< Destroy encoder function */ \
+                               EncodeMedia,    /**< Encode media function */ \
+                               CreateDecoder,  /**< Create decoder function */ \
+                               DestroyDecoder, /**< Destroy decoder function */ \
+                               DecodeMedia,    /**< Decode media function */ \
+                               ControlsTable,  /**< Codec controls tables */ \
+                               Flags,          /**< Flags */ \
+                               RawFormat       /**< Raw format */ \
+                               ) \
+  { \
+    PLUGIN_CODEC_VERSION, &MyLicenseInfo, Flags, Description, RawFormat, MediaFormat, NULL, \
+    SampleRate, BitsPerSecond, FrameTime, {{ p1,p2,p3,p4 }}, PayloadType, PayloadName, \
+    CreateEncoder, DestroyEncoder, EncodeMedia, ControlsTable, H323type, H323data \
+  }, \
+  { \
+    PLUGIN_CODEC_VERSION, &MyLicenseInfo, Flags, Description, MediaFormat, RawFormat, NULL, \
+    SampleRate, BitsPerSecond, FrameTime, {{ p1,p2,p3,p4 }}, PayloadType, PayloadName, \
+    CreateDecoder, DestroyDecoder, DecodeMedia, ControlsTable, H323type, H323data \
+  }
+
+#define PLUGINCODEC_AUDIO_CODEC(MediaFormat,     /**< Media Format */ \
+                                PayloadName,     /**< IANA RTP payload code */ \
+                                Description,     /**< Description text */ \
+                                SampleRate,      /**< Sample rate */ \
+                                BitsPerSecond,   /**< Maximum bits per second */ \
+                                FrameTime,       /**< Microseconds per audio frame */ \
+                                SamplesPerFrame, /**< Samples per audio frame */ \
+                                BytesPerFrame,   /**< Samples per audio frame */ \
+                                RecFramesPerPacket, /**< Recommended frames per packet */ \
+                                MaxFramesPerPacket, /**< Maximum frames per packet */ \
+                                RtpFlags,         /**< Extra flags typically if RTP payload type is fixed */ \
+                                PayloadType,      /**< IANA RTP payload type code */ \
+                                H323type,         /**< h323CapabilityType enumeration */ \
+                                H323data,         /**< Data to go with h323CapabilityType */ \
+                                CreateEncoder,    /**< Create encoder function */ \
+                                DestroyEncoder,   /**< Destroy encoder function */ \
+                                EncodeAudio,      /**< Encode media function */ \
+                                CreateDecoder,    /**< Create decoder function */ \
+                                DestroyDecoder,   /**< Destroy decoder function */ \
+                                DecodeAudio,      /**< Decode media function */ \
+                                ControlsTable     /**< Codec controls tables */ \
+                                ) \
+         PLUGINCODEC_CODEC_PAIR(MediaFormat, \
+                                PayloadName, \
+                                Description, \
+                                SampleRate, \
+                                BitsPerSecond, \
+                                FrameTime, \
+                                SamplesPerFrame, \
+                                BytesPerFrame, \
+                                RecFramesPerPacket, \
+                                MaxFramesPerPacket, \
+                                PayloadType, \
+                                H323type, \
+                                H323data, \
+                                CreateEncoder, \
+                                DestroyEncoder, \
+                                EncodeAudio, \
+                                CreateDecoder, \
+                                DestroyDecoder, \
+                                DecodeAudio, \
+                                ControlsTable, \
+                                PluginCodec_MediaTypeAudio | /* audio codec */ \
+                                PluginCodec_InputTypeRaw |   /* raw input data */ \
+                                PluginCodec_OutputTypeRaw |  /* raw output data */ \
+                                (RtpFlags), \
+                                PLUGINCODEC_RAW_AUDIO)
+
+#define PLUGINCODEC_ONE_AUDIO_CODEC(MediaFormat,     /**< Media Format */ \
+                                    PayloadName,     /**< IANA RTP payload code */ \
+                                    Description,     /**< Description text */ \
+                                    SampleRate,      /**< Sample rate */ \
+                                    BitsPerSecond,   /**< Maximum bits per second */ \
+                                    FrameTime,       /**< Microseconds per audio frame */ \
+                                    SamplesPerFrame, /**< Samples per audio frame */ \
+                                    BytesPerFrame,   /**< Samples per audio frame */ \
+                                    RecFramesPerPacket, /**< Recommended frames per packet */ \
+                                    MaxFramesPerPacket, /**< Maximum frames per packet */ \
+                                    RtpFlags,         /**< Extra flags typically if RTP payload type is fixed */ \
+                                    PayloadType,      /**< IANA RTP payload type code */ \
+                                    H323type,         /**< h323CapabilityType enumeration */ \
+                                    H323data          /**< Data to go with h323CapabilityType */ \
+                                ) \
+    static struct PluginCodec_Definition CodecDefinitionTable[] = { \
+            PLUGINCODEC_AUDIO_CODEC(MediaFormat, \
+                                    PayloadName, \
+                                    Description, \
+                                    SampleRate, \
+                                    BitsPerSecond, \
+                                    FrameTime, \
+                                    SamplesPerFrame, \
+                                    BytesPerFrame, \
+                                    RecFramesPerPacket, \
+                                    MaxFramesPerPacket, \
+                                    RtpFlags, \
+                                    PayloadType, \
+                                    H323type, \
+                                    H323data, \
+                                    MyCreateEncoder, \
+                                    MyDestroyEncoder, \
+                                    MyEncodeAudio, \
+                                    MyCreateDecoder, \
+                                    MyDestroyDecoder, \
+                                    MyDecodeAudio, \
+                                    MyControlsTable \
+                                    ) \
+    }
+
+#define PLUGINCODEC_VIDEO_CODEC(MediaFormat,     /**< Media Format */ \
+                                PayloadName,     /**< IANA RTP payload code */ \
+                                Description,     /**< Description text */ \
+                                BitsPerSecond,   /**< Maximum bits per second */ \
+                                MaxWidth,        /**< Max resolution (width) */ \
+                                MaxHeight,       /**< Max resolution (height) */ \
+                                RtpFlags,        /**< Extra flags typically if RTP payload type is fixed */ \
+                                PayloadType,     /**< IANA RTP payload type code */ \
+                                H323type,        /**< h323CapabilityType enumeration */ \
+                                H323data,        /**< Data to go with h323CapabilityType */ \
+                                CreateEncoder,   /**< Create encoder function */ \
+                                DestroyEncoder,  /**< Destroy encoder function */ \
+                                EncodeVideo,     /**< Encode media function */ \
+                                CreateDecoder,   /**< Create decoder function */ \
+                                DestroyDecoder,  /**< Destroy decoder function */ \
+                                DecodeVideo,     /**< Decode media function */ \
+                                ControlsTable    /**< Codec controls tables */ \
+                                ) \
+         PLUGINCODEC_CODEC_PAIR(MediaFormat, \
+                                PayloadName, \
+                                Description, \
+                                SampleRate, \
+                                BitsPerSecond, \
+                                90000, \
+                                BitsPerSecond, \
+                                100000, \
+                                MaxWidth, \
+                                MaxHeight, \
+                                0,30, \
+                                PayloadType, \
+                                H323type, \
+                                H323data, \
+                                CreateEncoder, \
+                                DestroyEncoder, \
+                                EncodeVideo, \
+                                CreateDecoder, \
+                                DestroyDecoder, \
+                                DecodeVideo, \
+                                ControlsTable, \
+                                PluginCodec_MediaTypeVideo | /* video codec */ \
+                                PluginCodec_InputTypeRRP |   /* RTP input data */ \
+                                PluginCodec_OutputTypeRRP |  /* RTP output data */ \
+                                (RtpFlags), \
+                                PLUGINCODEC_RAW_VIDEO)
+
+#define PLUGINCODEC_ONE_VIDEO_CODEC(MediaFormat,     /**< Media Format */ \
+                                    PayloadName,     /**< IANA RTP payload code */ \
+                                    Description,     /**< Description text */ \
+                                    BitsPerSecond,   /**< Maximum bits per second */ \
+                                    MaxWidth,        /**< Max resolution (width) */ \
+                                    MaxHeight,       /**< Max resolution (height) */ \
+                                    RtpFlags,        /**< Extra flags typically if RTP payload type is fixed */ \
+                                    PayloadType,     /**< IANA RTP payload type code */ \
+                                    H323type,        /**< h323CapabilityType enumeration */ \
+                                    H323data         /**< Data to go with h323CapabilityType */ \
+                                ) \
+    static struct PluginCodec_Definition CodecDefinitionTable[] = { \
+            PLUGINCODEC_VIDEO_CODEC(MediaFormat, \
+                                    PayloadName, \
+                                    Description, \
+                                    BitsPerSecond, \
+                                    MaxWidth, \
+                                    MaxHeight, \
+                                    RtpFlags, \
+                                    PayloadType, \
+                                    H323type, \
+                                    H323data, \
+                                    CreateEncoder, \
+                                    DestroyEncoder, \
+                                    EncodeAudio, \
+                                    CreateDecoder, \
+                                    DestroyDecoder, \
+                                    DecodeAudio, \
+                                    ControlsTable \
+                                    ) \
+    }
+
+
+//////////////////////////////////////////////////////////////////
 //
 //  H.323 specific values
 //
@@ -594,16 +809,40 @@ enum {
 #define PluginCodec_RTP_MinHeaderSize  (12)
 #define PluginCodec_RTP_MaxPayloadSize (PluginCodec_RTP_MaxPacketSize - PluginCodec_RTP_MinHeaderSize)
 
-#define PluginCodec_RTP_GetHeaderLength(ptr)      ((((unsigned char*)(ptr))[0] & 0x0f)*4 + PluginCodec_RTP_MinHeaderSize)
+#define PluginCodec_RTP_GetWORD(ptr, off)       ((((unsigned char*)(ptr))[off] << 8) | ((unsigned char*)(ptr))[off+1])
+
+#define PluginCodec_RTP_GetDWORD(ptr, off)      ((((unsigned char*)(ptr))[off  ] << 24)|\
+                                                 (((unsigned char*)(ptr))[off+1] << 16)|\
+                                                 (((unsigned char*)(ptr))[off+2] << 8 )|\
+                                                  ((unsigned char*)(ptr))[off+3])
+
+#define PluginCodec_RTP_SetWORD(ptr, off, val)  ((((unsigned char*)(ptr))[off  ] = (unsigned char)((val) >> 8 )),\
+                                                 (((unsigned char*)(ptr))[off+1] = (unsigned char) (val)      ))
+
+#define PluginCodec_RTP_SetDWORD(ptr, off, val) ((((unsigned char*)(ptr))[off  ] = (unsigned char)((val) >> 24)),\
+                                                 (((unsigned char*)(ptr))[off+1] = (unsigned char)((val) >> 16)),\
+                                                 (((unsigned char*)(ptr))[off+2] = (unsigned char)((val) >> 8 )),\
+                                                 (((unsigned char*)(ptr))[off+3] = (unsigned char) (val)      ))
+
+#define PluginCodec_RTP_GetCSRCHdrLength(ptr)      ((((unsigned char*)(ptr))[0] & 0x0f)*4 + PluginCodec_RTP_MinHeaderSize)
+#define PluginCodec_RTP_GetExtHdrLength(ptr)       ((((unsigned char*)(ptr))[0] & 0x10) ? (PluginCodec_RTP_GetWORD(ptr, PluginCodec_RTP_GetCSRCHdrLength(ptr)+2)*4+4) : 0)
+
+#define PluginCodec_RTP_GetHeaderLength(ptr)       (PluginCodec_RTP_GetCSRCHdrLength(ptr) + PluginCodec_RTP_GetExtHdrLength(ptr))
 #define PluginCodec_RTP_GetPayloadPtr(ptr)          ((unsigned char*)(ptr) + PluginCodec_RTP_GetHeaderLength(ptr))
 #define PluginCodec_RTP_GetPayloadType(ptr)        (((unsigned char*)(ptr))[1] & 0x7f)
-#define PluginCodec_RTP_SetPayloadType(ptr, type)  (((unsigned char*)(ptr))[1] = (((unsigned char*)(ptr))[1] & 0x80) | (type & 0x7f))
+#define PluginCodec_RTP_SetPayloadType(ptr, type)   (((unsigned char*)(ptr))[1] = (unsigned char)((((unsigned char*)(ptr))[1] & 0x80) | (type & 0x7f)))
 #define PluginCodec_RTP_GetMarker(ptr)            ((((unsigned char*)(ptr))[1] & 0x80) != 0)
-#define PluginCodec_RTP_SetMarker(ptr, mark)       (((unsigned char*)(ptr))[1] = (((unsigned char*)(ptr))[1] & 0x7f) | (mark != 0 ? 0x80 : 0))
-#define PluginCodec_RTP_GetTimestamp(ptr)         ((((unsigned char*)(ptr))[4] << 24) | (((unsigned char*)(ptr))[5] << 16) | (((unsigned char*)(ptr))[6] << 8) | ((unsigned char*)(ptr))[7])
-#define PluginCodec_RTP_SetTimestamp(ptr, ts)     ((((unsigned char*)(ptr))[4] = ((ts) >> 24)),(((unsigned char*)(ptr))[5] = ((ts) >> 16)),(((unsigned char*)(ptr))[6] = ((ts) >> 8)),(((unsigned char*)(ptr))[7] = (ts)))
-#define PluginCodec_RTP_GetSequenceNumber(ptr)    ((((unsigned char*)(ptr))[2] << 8) | ((unsigned char*)(ptr))[3])
-#define PluginCodec_RTP_SetSequenceNumber(ptr, sn)((((unsigned char*)(ptr))[2] = ((sn) >> 8)),(((unsigned char*)(ptr))[3] = (sn)))
+#define PluginCodec_RTP_SetMarker(ptr, mark)        (((unsigned char*)(ptr))[1] = (unsigned char)((((unsigned char*)(ptr))[1] & 0x7f) | (mark != 0 ? 0x80 : 0)))
+#define PluginCodec_RTP_GetTimestamp(ptr)          PluginCodec_RTP_GetDWORD(ptr, 4)
+#define PluginCodec_RTP_SetTimestamp(ptr, ts)      PluginCodec_RTP_SetDWORD(ptr, 4, ts)
+#define PluginCodec_RTP_GetSequenceNumber(ptr)     PluginCodec_RTP_GetWORD(ptr, 2)
+#define PluginCodec_RTP_SetSequenceNumber(ptr, sn) PluginCodec_RTP_SetWORD(ptr, 2, sn)
+#define PluginCodec_RTP_GetSSRC(ptr)               PluginCodec_RTP_GetDWORD(ptr, 8)
+#define PluginCodec_RTP_SetSSRC(ptr, ssrc)         PluginCodec_RTP_SetDWORD(ptr, 8, ssrc)
+
+#define PluginCodec_RTP_SetExtended(ptr, type, sz) ((((unsigned char*)(ptr))[0] |= 0x10), \
+                                                    PluginCodec_RTP_SetWORD(ptr, PluginCodec_RTP_GetCSRCHdrLength(ptr), type), \
+                                                    PluginCodec_RTP_SetWORD(ptr, PluginCodec_RTP_GetCSRCHdrLength(ptr)+2, sz))
 
 
 /////////////////
