@@ -23,9 +23,9 @@
  *
  * Contributor(s): 
  *
- * $Revision: 24807 $
+ * $Revision: 26728 $
  * $Author: rjongbloed $
- * $Date: 2010-10-19 23:33:34 -0500 (Tue, 19 Oct 2010) $
+ * $Date: 2011-12-01 22:59:14 -0600 (Thu, 01 Dec 2011) $
  */
 
 #ifndef OPAL_CODEC_VIDCODEC_H
@@ -121,6 +121,8 @@ class OpalVideoTranscoder : public OpalTranscoder
 
        The default behaviour checks for a OpalVideoUpdatePicture and sets the
        updatePicture member variable if that is the command.
+
+       @returns true if command is handled.
       */
     virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
@@ -147,15 +149,11 @@ class OpalVideoTranscoder : public OpalTranscoder
 
     bool WasLastFrameIFrame() const { return lastFrameWasIFrame; }
     void ForceIFrame()              { forceIFrame = true; }
-
-    PINDEX GetMaxEncoderOutputSize() const { return videoEncoderMaxOutputSize; }
-
   //@}
 
   protected:
     PINDEX inDataSize;
     PINDEX outDataSize;
-    PINDEX videoEncoderMaxOutputSize;
     bool   forceIFrame;
     bool   lastFrameWasIFrame;
 
