@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25271 $
+ * $Revision: 26728 $
  * $Author: rjongbloed $
- * $Date: 2011-03-05 03:56:37 -0600 (Sat, 05 Mar 2011) $
+ * $Date: 2011-12-01 22:59:14 -0600 (Thu, 01 Dec 2011) $
  */
 
 #ifndef OPAL_H323_H323CON_H
@@ -2039,7 +2039,15 @@ class H323Connection : public OpalRTPConnection
     H4507Handler&  getH4507handler(){return *h4507handler;}
 #endif
 
-    virtual void OnMediaCommand(OpalMediaCommand & note, INT extra);
+    /** Callback for media commands.
+        Calls the SendIntraFrameRequest on the rtp session
+
+       @returns true if command is handled.
+      */
+    virtual bool OnMediaCommand(
+      OpalMediaStream & stream,         ///< Stream command executed on
+      const OpalMediaCommand & command  ///< Media command being executed
+    );
     
   protected:
     /**Internal function to check if call established.
