@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21283 $
- * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Revision: 24606 $
+ * $Author: dereksmithies $
+ * $Date: 2010-07-28 22:51:05 -0500 (Wed, 28 Jul 2010) $
  */
 
 #ifndef OPAL_IAX2_IAX2CON_H
@@ -41,9 +41,9 @@
 #if OPAL_IAX2
 
 #include <opal/connection.h>
+#include <rtp/jitter.h>
 
 #include <iax2/frame.h>
-#include <iax2/iax2jitter.h>
 #include <iax2/iedata.h>
 #include <iax2/processor.h>
 #include <iax2/callprocessor.h>
@@ -85,6 +85,11 @@ class IAX2Connection : public OpalConnection
   ~IAX2Connection();
   //@}
 
+  /**this method starts the callprocessor of a connection, and maves
+     the phase to setup. Essentially, we start the entire call
+     handling process here. */
+  void StartOperation();
+
     /**Get indication of connection being to a "network".
        This indicates the if the connection may be regarded as a "network"
        connection. The distinction is about if there is a concept of a "remote"
@@ -143,8 +148,6 @@ class IAX2Connection : public OpalConnection
       CallEndReason reason = EndedByLocalUser /// Reason for call clearing
       );
 
-  OpalConnection::SendUserInputModes GetRealSendUserInputMode() const;
-
   /**Provided as a link between the iax endpoint and the iax processor */
   void SendDtmf(const PString & dtmf);
 
@@ -241,16 +244,16 @@ class IAX2Connection : public OpalConnection
 			  ); 
      
   /**Open a new media stream.  This will create a media stream of 
-     subclass OpalIAXMediaStream.
+     class OpalIAX2MediaStream.
      
      Note that media streams may be created internally to the
      underlying protocol. This function is not the only way a stream
      can come into existance.
   */
-  OpalMediaStream * CreateMediaStream(
-				      const OpalMediaFormat & mediaFormat, ///<Media format for stream
-				      unsigned sessionID,///<Session number for stream
-				      PBoolean isSource  ///<Is a source stream
+  virtual OpalMediaStream * CreateMediaStream(
+				      const OpalMediaFormat & mediaFormat, ///< Media format for stream
+				      unsigned sessionID,///< Session number for stream
+				      PBoolean isSource  ///< Is a source stream
 				      );
 
   /**Give the call token a value. The call token is the ipaddress of
@@ -302,11 +305,6 @@ class IAX2Connection : public OpalConnection
      This method is called after OnIncomingConnection().*/
   void OnSetUp();
 
-  PBoolean OnIncomingCall(
-    unsigned int options, 
-    OpalConnection::StringOptions * stringOptions
-  );
-
 
   /**Start an outgoing connection.
      This function will initiate the connection to the remote entity, for
@@ -342,14 +340,25 @@ class IAX2Connection : public OpalConnection
       value defined by FullFrameVoice::AudioSc */
   unsigned int ChooseCodec();
   
-  /**Return PTrue if the current connection is on hold.*/
-  virtual PBoolean IsConnectionOnHold();
-  
-  /**Take the current connection off hold*/
-  virtual bool RetrieveConnection();
-  
-  /**Put the current connection on hold, suspending all media streams.*/
-  virtual bool HoldConnection();
+    /**Put the current connection on hold, suspending all media streams.
+       The \p fromRemote parameter indicates if we a putting the remote on
+       hold (false) or it is a request for the remote to put us on hold (true).
+
+       The /p placeOnHold parameter indicates of teh command/request is for
+       going on hold or retrieving from hold.
+     */
+    virtual bool Hold(
+      bool fromRemote,  ///< Flag for if remote has us on hold, or we have them
+      bool placeOnHold  ///< Flag for setting on or off hold
+    );
+
+    /**Return true if the current connection is on hold.
+       The \p fromRemote parameter indicates if we are testing if the remote
+       system has us on hold, or we have them on hold.
+     */
+    virtual bool IsOnHold(
+      bool fromRemote  ///< Flag for if remote has us on hold, or we have them
+    );
   
   /**Signal that the remote side has put the connection on hold*/
   void RemoteHoldConnection();
@@ -385,7 +394,7 @@ class IAX2Connection : public OpalConnection
      function when an application wishes to redirct an unwanted incoming
      call.
      
-     The return value is PTrue if the call is to be forwarded, PFalse
+     The return value is true if the call is to be forwarded, false
      otherwise. Note that if the call is forwarded the current connection is
      cleared with teh ended call code of EndedByCallForwarded.
   */
@@ -400,7 +409,7 @@ class IAX2Connection : public OpalConnection
   
   /**Test to see if it is a status query type iax frame (eg lagrq) and
      handle it. If the frame is a status query, and it is handled,
-     return PTrue */
+     return true */
   //static PBoolean IsStatusQueryEthernetFrame(IAX2Frame *frame);
     
   /**Return reference to the endpoint class */
@@ -447,8 +456,8 @@ class IAX2Connection : public OpalConnection
 
   /**This jitter buffer smooths out the delivery times from the network, so
      that packets arrive in schedule at the far end. */
-  IAX2JitterBuffer jitterBuffer;
-  
+  OpalJitterBuffer jitterBuffer;
+
   /**The payload type, which we put on all RTP_DataFrame packets. This
      variable is placed on all RTP_DataFrame instances, prior to placing these
      frames into the jitter buffer.
@@ -469,10 +478,9 @@ class IAX2Connection : public OpalConnection
 
 #endif // OPAL_IAX2_IAX2CON_H
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 2 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file. 
+ * With the comment below, the tab key does auto indent to 2 spaces.     
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
