@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23984 $
+ * $Revision: 25336 $
  * $Author: rjongbloed $
- * $Date: 2010-01-25 18:06:02 -0600 (Mon, 25 Jan 2010) $
+ * $Date: 2011-03-16 03:51:39 -0500 (Wed, 16 Mar 2011) $
  */
 
 #ifndef OPAL_OPAL_CALL_H
@@ -108,9 +108,9 @@ class OpalCall : public PSafeObject
 
     /**Get the call clearand reason for this connection shutting down.
        Note that this function is only generally useful in the
-       H323EndPoint::OnConnectionCleared() function. This is due to the
+       OpalEndPoint::OnClearedCall() function. This is due to the
        connection not being cleared before that, and the object not even
-       exiting after that.
+       existing after that.
 
        If the call is still active then this will return OpalConnection::NumCallEndReasons.
       */
@@ -192,7 +192,7 @@ class OpalCall : public PSafeObject
        remote endpoint is "ringing". Generally some time after the
        MakeConnection() function was called, this is function is called.
 
-       If PFalse is returned the connection is aborted.
+       If false is returned the connection is aborted.
 
        If an application overrides this function, it should generally call the
        ancestor version for correct operation. An application would typically
@@ -319,14 +319,20 @@ class OpalCall : public PSafeObject
     }
 
     /**Put call on hold.
+       This function places the remote user of the network connection in the
+       call on hold.
       */
     bool Hold();
 
     /**Retrieve call from hold.
+       This function retrieves the remote user of the network connection in
+       the call from hold.
       */
     bool Retrieve();
 
     /**Indicate if call is in hold.
+       This function determines if the remote user of the network connection
+       in the call is currently on hold.
       */
     bool IsOnHold() const;
 
@@ -371,18 +377,31 @@ class OpalCall : public PSafeObject
   /**@name Media management */
   //@{
     /**Get the media formats of the connections in call.
-       This returns the intersection of all the media formats that all
-       connections in the call, optionally excepting the one provided as a
-       parameter, are capable of.
+       This returns the intersection of all the media formats available in
+       connections in the call, not including the parameter. It is in
+       esence the "local" media formats available to the connection.
 
        This will also add to the list all media formats for which there are
        transcoders registered.
       */
     virtual OpalMediaFormatList GetMediaFormats(
-      const OpalConnection & connection,  ///<  Connection requesting formats
-      PBoolean includeSpecifiedConnection     ///<  Include parameters media
+      const OpalConnection & connection  ///<  Connection requesting formats
     );
 
+    /**Adjust media formats available on a connection.
+       This is called by a connection after it has called
+       OpalCall::GetMediaFormats() to get all media formats that it can use so
+       that an application may remove or reorder the media formats before they
+       are used to open media streams.
+
+       The default behaviour calls the OpalManager function of the same name.
+      */
+    virtual void AdjustMediaFormats(
+      bool local,                         ///<  Media formats a local ones to be presented to remote
+      const OpalConnection & connection,  ///<  Connection that is about to use formats
+      OpalMediaFormatList & mediaFormats  ///<  Media formats to use
+    ) const;
+
     /**Open source media streams for the specified connection.
        A source media stream is opened for the connection, if successful then
        sink media streams are created for every other connection in the call.
@@ -427,10 +446,14 @@ class OpalCall : public PSafeObject
       const RTP_Session & session         ///<  Session with statistics
     );
 
+    /**Start the media streams on the connections.
+     */
+    virtual void StartMediaStreams();
+
     /**Close the media streams on the connections.
      */
     virtual void CloseMediaStreams();
-    
+
     /**See if the media can bypass the local host.
      */
     virtual PBoolean IsMediaBypassPossible(
