@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 26937 $
+ * $Author: rjongbloed $
+ * $Date: 2012-02-06 19:54:19 -0600 (Mon, 06 Feb 2012) $
  */
 
 #ifndef OPAL_OPAL_ENDPOINT_H
@@ -70,7 +70,7 @@ class OpalEndPoint : public PObject
   public:
     enum Attributes {
       CanTerminateCall = 1,
-      HasLineInterface = 2
+      SupportsE164 = 2
     };
 
   /**@name Construction */
@@ -110,6 +110,12 @@ class OpalEndPoint : public PObject
     /**Add a set of listeners to the endoint.
        This allows for the automatic creating of incoming call connections. 
        If the list is empty then GetDefaultListeners() is used.
+
+       Note: while the \p interfaces parameter is a string array, each element
+       of the array should be compatible with OpalTransportAddress.
+
+       See OpalTransportAddress for more details on the syntax of an interface
+       definition.
       */
     PBoolean StartListeners(
       const PStringArray & interfaces ///<  Address of interface to listen on.
@@ -118,6 +124,9 @@ class OpalEndPoint : public PObject
     /**Add a listener to the endoint.
        This allows for the automatic creating of incoming call connections. /
        If the address is empty then the first entry of GetDefaultListeners() is used.
+
+       See OpalTransportAddress for more details on the syntax of an interface
+       definition.
       */
     PBoolean StartListener(
       const OpalTransportAddress & iface ///<  Address of interface to listen on.
@@ -138,14 +147,26 @@ class OpalEndPoint : public PObject
       */
     virtual PStringArray GetDefaultListeners() const;
 
+    /**Get comma separated list of transport protocols to create if
+       no explicit listeners started.
+      */
+    virtual PString GetDefaultTransport() const;
+
     /**Find a listener given the transport address.
       */
     OpalListener * FindListener(
         const OpalTransportAddress & iface ///<  Address of interface we may be listening on.
     );
 
+    /** Find a listener that is compatible with the specified protocol
+     */
+    bool FindListenerForProtocol(
+      const char * proto,         ///< Protocol to findlistener, e.g "tcp" or "udp"
+      OpalTransportAddress & addr ///< Address of listner interface
+    );
+
     /**Stop a listener given the transport address.
-       Returns PTrue if a listener was on that interface, and was stopped.
+       Returns true if a listener was on that interface, and was stopped.
       */
     PBoolean StopListener(
         const OpalTransportAddress & iface ///<  Address of interface we may be listening on.
@@ -161,7 +182,7 @@ class OpalEndPoint : public PObject
     /**Return a list of the transport addresses for all listeners on this endpoint
       */
     OpalTransportAddressArray GetInterfaceAddresses(
-      PBoolean excludeLocalHost = PTrue,       ///<  Flag to exclude 127.0.0.1
+      PBoolean excludeLocalHost = true,       ///<  Flag to exclude 127.0.0.1
       const OpalTransport * associatedTransport = NULL
                           ///<  Associated transport for precedence and translation
     );
@@ -174,15 +195,25 @@ class OpalEndPoint : public PObject
 
     /**Handle new incoming connection from listener.
 
-       A return value of PTrue indicates that the transport object should be
-       deleted by the caller. PFalse indicates that something else (eg the
+       A return value of true indicates that the transport object should be
+       deleted by the caller. false indicates that something else (eg the
        connection) has taken over responsibility for deleting the transport.
 
-       The default behaviour just returns PTrue.
+       The default behaviour just returns true.
       */
     virtual PBoolean NewIncomingConnection(
       OpalTransport * transport  ///<  Transport connection came in on
     );
+
+    /**Call back for a new connection has been constructed.
+       This is called after CreateConnection has returned a new connection.
+       It allows an application to make any custom adjustments to the
+       connection before it begins to process the protocol. behind it.
+      */
+    virtual void OnNewConnection(
+      OpalCall & call,              ///< Call that owns the newly created connection.
+      OpalConnection & connection   ///< New connection just created
+    );
   //@}
 
   /**@name Connection management */
@@ -203,12 +234,12 @@ class OpalEndPoint : public PObject
 
        The proto field is optional when passed to a specific endpoint. If it
        is present, however, it must agree with the endpoints protocol name or
-       PFalse is returned.
+       false is returned.
 
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the assiciated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -216,12 +247,12 @@ class OpalEndPoint : public PObject
 
        The default behaviour is pure.
      */
-    virtual PBoolean MakeConnection(
+    virtual PSafePtr<OpalConnection> MakeConnection(
       OpalCall & call,          ///<  Owner of connection
       const PString & party,    ///<  Remote party to call
       void * userData = NULL,          ///<  Arbitrary data to pass to connection
-      unsigned int options = 0,     ///<  options to pass to conneciton
-      OpalConnection::StringOptions * stringOptions = NULL
+      unsigned int options = 0,     ///<  Options bit mask to pass to conneciton
+      OpalConnection::StringOptions * stringOptions = NULL ///< Options to pass to connection
     ) = 0;
 
     /**Callback for outgoing connection, it is invoked after OpalLineConnection::SetUpConnection
@@ -233,7 +264,7 @@ class OpalEndPoint : public PObject
        This function is used for an application to control the answering of
        incoming calls.
 
-       If PTrue is returned then the connection continues. If PFalse then the
+       If true is returned then the connection continues. If false then the
        connection is aborted.
 
        Note this function should not block for any length of time. If the
@@ -253,14 +284,25 @@ class OpalEndPoint : public PObject
     virtual PBoolean OnIncomingConnection(
       OpalConnection & connection,  ///<  Connection that is calling
       unsigned options,             ///<  options for new connection (can't use default value as overrides will fail)
-      OpalConnection::StringOptions * stringOptions
+      OpalConnection::StringOptions * stringOptions  ///< Options to pass to connection
     );
-    virtual PBoolean OnIncomingConnection(
-      OpalConnection & connection,  ///<  Connection that is calling
-      unsigned options              ///<  options for new connection (can't use default value as overrides will fail)
-    );
-    virtual PBoolean OnIncomingConnection(
-      OpalConnection & connection   ///<  Connection that is calling
+
+    /**Call back for remote party is now responsible for completing the call.
+       This function is called when the remote system has been contacted and it
+       has accepted responsibility for completing, or failing, the call. This
+       is distinct from OnAlerting() in that it is not known at this time if
+       anything is ringing. This indication may be used to distinguish between
+       "transport" level error, in which case another host may be tried, and
+       that finalising the call has moved "upstream" and the local system has
+       no more to do but await a result.
+
+       If an application overrides this function, it should generally call the
+       ancestor version for correct operation.
+
+       The default behaviour calls the OpalManager function of the same name.
+     */
+    virtual void OnProceeding(
+      OpalConnection & connection   ///<  Connection that is proceeeding
     );
 
     /**Call back for remote party being alerted.
@@ -268,7 +310,7 @@ class OpalEndPoint : public PObject
        remote endpoint is "ringing". Generally some time after the
        MakeConnection() function was called, this is function is called.
 
-       If PFalse is returned the connection is aborted.
+       If false is returned the connection is aborted.
 
        If an application overrides this function, it should generally call the
        ancestor version for correct operation. An application would typically
@@ -279,7 +321,7 @@ class OpalEndPoint : public PObject
        The default behaviour calls the OpalManager function of the same name.
      */
     virtual void OnAlerting(
-      OpalConnection & connection   ///<  Connection that was established
+      OpalConnection & connection   ///<  Connection that is alerting
     );
 
     /**Call back for answering an incoming call.
@@ -376,6 +418,58 @@ class OpalEndPoint : public PObject
       const PString & remoteParty         ///<  The new remote party
     );
 
+    /**A call back function to monitor the progress of a transfer.
+       When a transfer operation is initiated, the Transfer() function will
+       generally return immediately and the transfer may take some time. This
+       call back can give an indication to the application of the progress of
+       the transfer.
+       the transfer.
+
+       For example in SIP, the OpalCall::Transfer() function will have sent a
+       REFER request to the remote party. The remote party sends us NOTIFY
+       requests about the progress of the REFER request.
+
+       An application can now make a decision during the transfer operation
+       to short circuit the sequence, or let it continue. It can also
+       determine if the transfer did not go through, and it should "take back"
+       the call. Note no action is required to "take back" the call other than
+       indicate to the user that they are back on.
+
+       A return value of false will immediately disconnect the current call.
+
+       The exact format of the \p info parameter is dependent on the protocol
+       being used. As a minimum, it will always have a values info["result"]
+       and info["party"].
+
+       The info["party"] indicates the part the \p connection is playing in
+       the transfer. This will be:
+          "A"   party being transferred
+          "B"   party initiating the transfer of "A"
+          "C"   party "A" is being transferred to
+
+       The info["result"] will be at least one of the following:
+          "success"     Transfer completed successfully (party A or B)
+          "incoming"    New call was from a transfer (party C)
+          "started"     Transfer operation has started (party A)
+          "progress"    Transfer is in progress (party B)
+          "blind"       Transfer is blind, no further notification (party B)
+          "error"       Transfer could not begin (party B)
+          "failed"      Transfer started but did not complete (party A or B)
+
+       For SIP, there may be an additional info["state"] containing the NOTIFY
+       subscription state, an info["code"] entry containing the 3 digit
+       code returned in the NOTIFY body and info["Referred-By"] indicating the
+       URI of party B. Other fields may also be present.
+
+       The default behaviour calls the OpalManager function of the same name.
+       The default action of that function is to return false, thereby
+       releasing the connection if the info["result"] == "success".
+      */
+    virtual bool OnTransferNotify(
+      OpalConnection & connection,  ///< Connection being transferred.
+      const PStringToString & info  ///< Information on the transfer
+    );
+
     /**Clear a call.
        This finds the call by using the token then calls the OpalCall::Clear()
        function on it. All connections are released, and the conenctions and
@@ -407,7 +501,7 @@ class OpalEndPoint : public PObject
       */
     virtual void ClearAllCalls(
       OpalConnection::CallEndReason reason = OpalConnection::EndedByLocalUser, ///<  Reason for call clearing
-      PBoolean wait = PTrue   ///<  Flag for wait for calls to e cleared.
+      PBoolean wait = true   ///<  Flag for wait for calls to e cleared.
     );
 
     /**Find a connection that uses the specified token.
@@ -416,7 +510,7 @@ class OpalEndPoint : public PObject
       */
     PSafePtr<OpalConnection> GetConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
+      PSafetyMode mode = PSafeReadWrite ///< Locking mode
     ) { return connectionsActive.FindWithLock(token, mode); }
 
     /**Find a connection that uses the specified token.
@@ -428,7 +522,7 @@ class OpalEndPoint : public PObject
     template <class ConnClass>
     PSafePtr<ConnClass> GetConnectionWithLockAs(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
+      PSafetyMode mode = PSafeReadWrite ///< Locking mode
     )
     {
       PSafePtr<ConnClass> connection = PSafePtrCast<OpalConnection, ConnClass>(GetConnectionWithLock(token, mode));
@@ -473,10 +567,9 @@ class OpalEndPoint : public PObject
        Note that a specific connection may not actually support all of the
        media formats returned here, but should return no more.
 
-       The default behaviour returns the most basic media formats, PCM audio
-       and YUV420P video.
+       The default behaviour is pure.
       */
-    virtual OpalMediaFormatList GetMediaFormats() const;
+    virtual OpalMediaFormatList GetMediaFormats() const = 0;
 
     /**Adjust media formats available on a connection.
        This is called by a connection after it has called
@@ -487,6 +580,7 @@ class OpalEndPoint : public PObject
        The default behaviour calls the OpalManager function of the same name.
       */
     virtual void AdjustMediaFormats(
+      bool local,                         ///<  Media formats a local ones to be presented to remote
       const OpalConnection & connection,  ///<  Connection that is about to use formats
       OpalMediaFormatList & mediaFormats  ///<  Media formats to use
     ) const;
@@ -516,15 +610,6 @@ class OpalEndPoint : public PObject
     );
 
 #if OPAL_VIDEO
-    /**Add video media formats available on a connection.
-
-       The default behaviour calls the OpalEndPoint function of the same name.
-      */
-    virtual void AddVideoMediaFormats(
-      OpalMediaFormatList & mediaFormats, ///<  Media formats to use
-      const OpalConnection * connection = NULL  ///<  Optional connection that is using formats
-    ) const;
-
     /**Create an PVideoInputDevice for a source media stream.
       */
     virtual PBoolean CreateVideoInputDevice(
@@ -580,9 +665,43 @@ class OpalEndPoint : public PObject
     );
   //@}
 
+  /**@name Instant Messaging */
+  //@{
+    /**Send text message
+     */
+    virtual PBoolean Message(
+      const PString & to, 
+      const PString & body
+    );
+    virtual PBoolean Message(
+      const PURL & to, 
+      const PString & type,
+      const PString & body,
+      PURL & from, 
+      PString & conversationId
+    );
+    virtual PBoolean Message(
+      OpalIM & Message
+    );
+
+    /**Called when text message received
+     */
+    virtual void OnMessageReceived(
+      const OpalIM & message
+    );
+  //@}
+
   /**@name Other services */
   //@{
-    /**Callback called when Message Waiting Indication (MWI) is received
+    /**Callback called when Message Waiting Indication (MWI) is received.
+       Multiple callbacks may occur with each MessageWaitingType. A \p type
+       of NumMessageWaitingTypes indicates the server is unable to distinguish
+       the message type.
+
+       The \p extraInfo parameter is generally of the form "a/b" where a and b
+       unsigned integers representing new and old message count. However, it
+       may be a simple "yes" or "no" if the remote cannot provide a message
+       count.
      */
     virtual void OnMWIReceived (
       const PString & party,                ///< Name of party MWI is for
@@ -591,7 +710,7 @@ class OpalEndPoint : public PObject
     );
 
     /** Execute garbage collection for endpoint.
-        Returns PTrue if all garbage has been collected.
+        Returns true if all garbage has been collected.
         Default behaviour deletes the objects in the connectionsActive list.
       */
     virtual PBoolean GarbageCollection();
@@ -655,6 +774,18 @@ class OpalEndPoint : public PObject
      */
     const OpalListenerList & GetListeners() const { return listeners; }
 
+    /**Get the default options for created connections.
+      */
+    const OpalConnection::StringOptions & GetDefaultStringOptions() const { return m_defaultStringOptions; }
+
+    /**Set the default options for created connections.
+      */
+    void SetDefaultStringOptions(const OpalConnection::StringOptions & opts) { m_defaultStringOptions = opts; }
+
+    /**Set the default option for created connections.
+      */
+    void SetDefaultStringOption(const PCaselessString & key, const PString & data) { m_defaultStringOptions.SetAt(key, data); }
+
     /**Get the default mode for sending User Input Indications.
       */
     OpalConnection::SendUserInputModes GetSendUserInputMode() const { return defaultSendUserInputMode; }
@@ -664,19 +795,9 @@ class OpalEndPoint : public PObject
     void SetSendUserInputMode(OpalConnection::SendUserInputModes mode) { defaultSendUserInputMode = mode; }
   //@}
 
-    virtual PString GetDefaultTransport() const;
-
-    /**Call back for a new connection has been constructed.
-       This is called after CreateConnection has returned a new connection.
-       It allows an application to make any custom adjustments to the
-       connection before it begins to process the protocol. behind it.
-      */
-    virtual void OnNewConnection(
-      OpalCall & call,              ///< Call that owns the newly created connection.
-      OpalConnection & connection   ///< New connection just created
-    );
-
 #if OPAL_PTLIB_SSL
+    /** Get the name of the file to use as SSL certificate for SSL based calls, e.g. sips or h323s
+      */
     PString GetSSLCertificate() const;
 #endif
 
@@ -685,11 +806,13 @@ class OpalEndPoint : public PObject
     PCaselessString prefixName;
     unsigned        attributeBits;
     WORD            defaultSignalPort;
+    PINDEX          m_maxSizeUDP;
     OpalProductInfo productInfo;
     PString         defaultLocalPartyName;
     PString         defaultDisplayName;
 
     unsigned initialBandwidth;  // in 100s of bits/sev
+    OpalConnection::StringOptions      m_defaultStringOptions;
     OpalConnection::SendUserInputModes defaultSendUserInputMode;
 
     OpalListenerList   listeners;
@@ -698,15 +821,28 @@ class OpalEndPoint : public PObject
     {
         virtual void DeleteObject(PObject * object) const;
     } connectionsActive;
-    PBoolean AddConnection(OpalConnection * connection);
+    OpalConnection * AddConnection(OpalConnection * connection);
 
     PMutex inUseFlag;
 
     friend void OpalManager::GarbageCollection();
     friend void OpalConnection::Release(CallEndReason reason);
+
+  private:
+    P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &, unsigned), false);
+    P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &), false);
+    P_REMOVE_VIRTUAL_VOID(AdjustMediaFormats(const OpalConnection &, OpalMediaFormatList &) const);
+    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const PURL&,const PString&,const PURL&,const PString&,const PString&,const PString&));
 };
 
 
+/// Test for if string is a valid E.164 number
+bool OpalIsE164(
+  const PString & number,   ///< Number to inspect
+  bool strict = false     ///< Strict interpretation, or allow leading '+'
+);
+
+
 #endif // OPAL_OPAL_ENDPOINT_H
 
 
