@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26069 $
+ * $Revision: 26939 $
  * $Author: rjongbloed $
- * $Date: 2011-06-23 19:56:50 -0500 (Thu, 23 Jun 2011) $
+ * $Date: 2012-02-07 01:17:49 -0600 (Tue, 07 Feb 2012) $
  */
 
 #ifndef OPAL_OPAL_LOCALEP_H
@@ -166,7 +166,8 @@ class OpalLocalEndPoint : public OpalEndPoint
        connection.
       */
     virtual bool AlertingIncomingCall(
-      const PString & token ///<  Token of connection to accept call
+      const PString & token, ///<  Token of connection to indicate alerting
+      OpalConnection::StringOptions * options = NULL  ///< Optional string optiosn to apply
     );
 
     /**Accept the incoming connection.
@@ -174,7 +175,8 @@ class OpalLocalEndPoint : public OpalEndPoint
        connection.
       */
     virtual bool AcceptIncomingCall(
-      const PString & token ///<  Token of connection to accept call
+      const PString & token, ///<  Token of connection to accept call
+      OpalConnection::StringOptions * options = NULL  ///< Optional string optiosn to apply
     );
 
     /**Reject the incoming connection.
@@ -323,6 +325,9 @@ class OpalLocalConnection : public OpalConnection
       */
     virtual PBoolean IsNetworkConnection() const { return false; }
 
+    /// Call back for connection to act on changed string options
+    virtual void OnApplyStringOptions();
+
     /**Start an outgoing connection.
        This function will initiate the connection to the remote entity, for
        example in H.323 it sends a SETUP, in SIP it sends an INVITE etc.
