@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24038 $
+ * $Revision: 26937 $
  * $Author: rjongbloed $
- * $Date: 2010-02-09 22:21:59 -0600 (Tue, 09 Feb 2010) $
+ * $Date: 2012-02-06 19:54:19 -0600 (Mon, 06 Feb 2012) $
  */
 
 #ifndef OPAL_OPAL_MANAGER_H
@@ -153,7 +153,7 @@ class OpalManager : public PObject
       const PString & partyB,       ///<  The B party of call
       void * userData = NULL,       ///<  user data passed to Call and Connection
       unsigned options = 0,         ///<  options passed to connection
-      OpalConnection::StringOptions * stringOptions = NULL   ///<  complex string options passed to call
+      OpalConnection::StringOptions * stringOptions = NULL ///< Options to pass to connection
     );
     virtual PBoolean SetUpCall(
       const PString & partyA,       ///<  The A party of call
@@ -161,9 +161,39 @@ class OpalManager : public PObject
       PString & token,              ///<  Token for call
       void * userData = NULL,       ///<  user data passed to Call and Connection
       unsigned options = 0,         ///<  options passed to connection
-      OpalConnection::StringOptions * stringOptions = NULL   ///<  complex string options passed to call
+      OpalConnection::StringOptions * stringOptions = NULL ///< Options to pass to connection
     );
 
+    /**Determine if a call is active.
+       Return true if there is an active call with the specified token. Note
+       that the call could clear any time (even milliseconds) after this
+       function returns true.
+      */
+    virtual PBoolean HasCall(
+      const PString & token  ///<  Token for identifying call
+    ) { return activeCalls.FindWithLock(token, PSafeReference) != NULL; }
+
+    /**Return the number of active calls.
+      */
+    PINDEX GetCallCount() const { return activeCalls.GetSize(); }
+
+    /**Get all tokens for active calls.
+      */
+     PArray<PString> GetAllCalls() const { return activeCalls.GetKeys(); }
+
+    /**Find a call with the specified token.
+       This searches the manager database for the call that contains the token
+       as provided by functions such as SetUpCall().
+
+       Note the caller of this function MUST call the OpalCall::Unlock()
+       function if this function returns a non-NULL pointer. If it does not
+       then a deadlock can occur.
+      */
+    PSafePtr<OpalCall> FindCallWithLock(
+      const PString & token,  ///<  Token to identify connection
+      PSafetyMode mode = PSafeReadWrite ///< Lock mode
+    ) { return activeCalls.FindWithLock(token, mode); }
+
     /**A call back function whenever a call is completed.
        In telephony terminology a completed call is one where there is an
        established link between two parties.
@@ -176,43 +206,16 @@ class OpalManager : public PObject
       OpalCall & call   ///<  Call that was completed
     );
 
-    /**Determine if a call is active.
-       Return PTrue if there is an active call with the specified token. Note
-       that the call could clear any time (even milliseconds) after this
-       function returns PTrue.
-      */
-    virtual PBoolean HasCall(
-      const PString & token  ///<  Token for identifying call
-    ) { return activeCalls.FindWithLock(token, PSafeReference) != NULL; }
-
-    /**Return the number of active calls.
-      */
-    PINDEX GetCallCount() const { return activeCalls.GetSize(); }
-
-
     /**Determine if a call is established.
-       Return PTrue if there is an active call with the specified token and
+       Return true if there is an active call with the specified token and
        that call has at least two parties with media flowing between them.
        Note that the call could clear any time (even milliseconds) after this
-       function returns PTrue.
+       function returns true.
       */
     virtual PBoolean IsCallEstablished(
       const PString & token  ///<  Token for identifying call
     );
 
-    /**Find a call with the specified token.
-       This searches the manager database for the call that contains the token
-       as provided by functions such as SetUpCall().
-
-       Note the caller of this function MUST call the OpalCall::Unlock()
-       function if this function returns a non-NULL pointer. If it does not
-       then a deadlock can occur.
-      */
-    PSafePtr<OpalCall> FindCallWithLock(
-      const PString & token,  ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
-    ) { return activeCalls.FindWithLock(token, mode); }
-
     /**Clear a call.
        This finds the call by using the token then calls the OpalCall::Clear()
        function on it. All connections are released, and the connections and
@@ -245,7 +248,7 @@ class OpalManager : public PObject
       */
     virtual void ClearAllCalls(
       OpalConnection::CallEndReason reason = OpalConnection::EndedByLocalUser, ///<  Reason for call clearing
-      PBoolean wait = PTrue   ///<  Flag to wait for calls to e cleared.
+      PBoolean wait = true   ///<  Flag to wait for calls to e cleared.
     );
 
     /**A call back function whenever a call is cleared.
@@ -297,9 +300,12 @@ class OpalManager : public PObject
     virtual PString GetNextToken(char prefix);
   //@}
 
-  /**@name Presence & Instant Messaging management */
+  /**@name Presence management */
   //@{
     /**Add a presentity.
+       If the presentity is already present, a new one is not added, and the
+       existing instance is returned.
+
        Returns a Read/Write locked pointer to presentity.
       */
     virtual PSafePtr<OpalPresentity> AddPresentity(
@@ -322,8 +328,11 @@ class OpalManager : public PObject
     virtual bool RemovePresentity(
       const PString & presentity  ///< Presentity URI
     );
+  //@}
 
-    /**Send text message
+  /**@name Instant Messaging management */
+  //@{
+    /**Send an IM to a remote party.
      */
     virtual PBoolean Message(
       const PString & to, 
@@ -336,27 +345,16 @@ class OpalManager : public PObject
       PURL & from, 
       PString & conversationId
     );
+    virtual PBoolean Message(
+      OpalIM & message
+    );
 
     /**Called when text message received
      */
     virtual void OnMessageReceived(
-      const PURL & from, 
-      const PString & fromName,
-      const PURL & to, 
-      const PString & type,
-      const PString & body,
-      const PString & conversationId
+      const OpalIM & message
     );
 
-#if OPAL_HAS_IM
-    /** Called when text message to be sent to remote end of a connection
-      */
-    virtual bool TransmitExternalIM(
-      OpalConnection & conn, 
-      const OpalMediaFormat & format, 
-      RTP_IMFrame & frame
-    );
-#endif
   //@}
 
   /**@name Connection management */
@@ -387,7 +385,7 @@ class OpalManager : public PObject
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the associated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -400,7 +398,7 @@ class OpalManager : public PObject
       const PString & party,             ///<  Party to call
       void * userData = NULL,            ///<  user data to pass to connections
       unsigned int options = 0,          ///<  options to pass to conneciton
-      OpalConnection::StringOptions * stringOptions = NULL
+      OpalConnection::StringOptions * stringOptions = NULL ///< Options to pass to connection
     );
 
     /**Call back for a new connection has been constructed.
@@ -416,7 +414,7 @@ class OpalManager : public PObject
        This function is used for an application to control the answering of
        incoming calls.
 
-       If PTrue is returned then the connection continues. If PFalse then the
+       If true is returned then the connection continues. If false then the
        connection is aborted.
 
        Note this function should not block for any length of time. If the
@@ -441,7 +439,7 @@ class OpalManager : public PObject
     virtual PBoolean OnIncomingConnection(
       OpalConnection & connection,   ///<  Connection that is calling
       unsigned options,              ///<  options for new connection (can't use default as overrides will fail)
-      OpalConnection::StringOptions * stringOptions
+      OpalConnection::StringOptions * stringOptions ///< Options to pass to connection
     );
 
     /**Route a connection to another connection from an endpoint.
@@ -456,7 +454,7 @@ class OpalManager : public PObject
       const PString & b_party,      ///< Destination indicated by source
       OpalCall & call,              ///< Call for new connection
       unsigned options,             ///< Options for new connection (can't use default as overrides will fail)
-      OpalConnection::StringOptions * stringOptions
+      OpalConnection::StringOptions * stringOptions ///< Options to pass to connection
     );
 
     /**Call back for remote party is now responsible for completing the call.
@@ -483,7 +481,7 @@ class OpalManager : public PObject
        remote endpoint is "ringing". This function is generally called
        some time after the MakeConnection() function was called.
 
-       If PFalse is returned the connection is aborted.
+       If false is returned the connection is aborted.
 
        If an application overrides this function, it should generally call the
        ancestor version for correct operation. An application would typically
@@ -564,7 +562,7 @@ class OpalManager : public PObject
        The default behaviour calls OnReleased() on the connection's
        associated OpalCall object. This indicates to the call that the
        connection has been released so it can release the last remaining
-       connection and then returns PTrue.
+       connection and then returns true.
       */
     virtual void OnReleased(
       OpalConnection & connection   ///<  Connection that was established
@@ -591,8 +589,59 @@ class OpalManager : public PObject
       OpalConnection & connection,  ///<  Connection that was held
       const PString & remoteParty         ///<  The new remote party
     );
+
+    /**A call back function to monitor the progress of a transfer.
+       When a transfer operation is initiated, the Transfer() function will
+       generally return immediately and the transfer may take some time. This
+       call back can give an indication to the application of the progress of
+       the transfer.
+       the transfer.
+
+       For example in SIP, the OpalCall::Transfer() function will have sent a
+       REFER request to the remote party. The remote party sends us NOTIFY
+       requests about the progress of the REFER request.
+
+       An application can now make a decision during the transfer operation
+       to short circuit the sequence, or let it continue. It can also
+       determine if the transfer did not go through, and it should "take back"
+       the call. Note no action is required to "take back" the call other than
+       indicate to the user that they are back on.
+
+       A return value of false will immediately disconnect the current call.
+
+       The exact format of the \p info parameter is dependent on the protocol
+       being used. As a minimum, it will always have a values info["result"]
+       and info["party"].
+
+       The info["party"] indicates the part the \p connection is playing in
+       the transfer. This will be:
+          "A"   party being transferred
+          "B"   party initiating the transfer of "A"
+          "C"   party "A" is being transferred to
+
+       The info["result"] will be at least one of the following:
+          "success"     Transfer completed successfully (party A or B)
+          "incoming"    New call was from a transfer (party C)
+          "started"     Transfer operation has started (party A)
+          "progress"    Transfer is in progress (party B)
+          "blind"       Transfer is blind, no further notification (party B)
+          "error"       Transfer could not begin (party B)
+          "failed"      Transfer started but did not complete (party A or B)
+
+       For SIP, there may be an additional info["state"] containing the NOTIFY
+       subscription state, an info["code"] entry containing the 3 digit
+       code returned in the NOTIFY body and info["Referred-By"] indicating the
+       URI of party B. Other fields may also be present.
+
+       The default behaviour returns false if info["result"] == "success".
+      */
+    virtual bool OnTransferNotify(
+      OpalConnection & connection,  ///< Connection being transferred.
+      const PStringToString & info  ///< Information on the transfer
+    );
   //@}
 
+
   /**@name Media Streams management */
   //@{
     /**Get common media formats.
@@ -679,8 +728,8 @@ class OpalManager : public PObject
        the transfer in some way, rather than the full media path of codecs and
        sockets which might not be necessary.
 
-       Note this is the complement to SetMediaBypass() as this function stops
-       RTP data from being sent/received, while SetMediaBypass() transfers
+       Note this is the complement to SetMediaPassThrough() as this function stops
+       RTP data from being sent/received, while SetMediaPassThrough() transfers
        RTP data between the two endpoints.
 
        The default behaviour returns false.
@@ -695,7 +744,7 @@ class OpalManager : public PObject
       bool opened                   ///< Media streams are opened/closed
     ) const;
 
-    /**Set bypass mode for media.
+    /**Set pass though mode for media.
 
        Bypass the internal media handling, passing RTP data directly from
        one call/connection to another.
@@ -710,24 +759,24 @@ class OpalManager : public PObject
        calls in the node, it is a waste to decode the audio, add to mixer and
        re-encode it again. In practice this is identical to just bypassing the
        mixer node completely, until a third party is added, then we need to
-       switch back to normal (non-bypassed) operation.
+       switch back to normal (non-pass-through) operation.
 
        Note this is the complement to OnLocalRTP() as this function transfers
        RTP data directly between the two endpoints, while OnLocalRTP() stops
        the RTP data from being sent/received.
 
-       @return true if bypass is started/stopped, false if there was no such
-               call/connection/stream, the streams are incompatible formats
+       @return true if pass through is started/stopped, false if there was no
+               such call/connection/stream, the streams are incompatible formats
                or a conflicting bypass is already in place.
       */
-    bool SetMediaBypass(
+    bool SetMediaPassThrough(
       const PString & token1, ///< First calls token
       const PString & token2, ///< Second calls token
       bool bypass,            ///< Bypass the media
       unsigned sessionID = 0, ///< Session ID of media stream, 0 indicates all
-      bool network  = true    ///< Bypass the network connections of the calls
+      bool network  = true    ///< Pass through the network connections of the calls only
     );
-    static bool SetMediaBypass(
+    static bool SetMediaPassThrough(
       OpalConnection & connection1, ///< First connection
       OpalConnection & connection2, ///< Second connection
       bool bypass,                  ///< Bypass the media
@@ -773,7 +822,7 @@ class OpalManager : public PObject
       */
     virtual OpalMediaPatch * CreateMediaPatch(
       OpalMediaStream & source,         ///<  Source media stream
-      PBoolean requiresPatchThread = PTrue
+      PBoolean requiresPatchThread = true  ///< The patch requires a thread
     );
 
     /**Destroy a OpalMediaPatch instance.
@@ -839,6 +888,7 @@ class OpalManager : public PObject
 
   /**@name Other services */
   //@{
+    /// Message waiting sub-types
     enum MessageWaitingType { 
       NoMessageWaiting,
       VoiceMessageWaiting, 
@@ -849,7 +899,15 @@ class OpalManager : public PObject
       NumMessageWaitingTypes
     };
 
-    /**Callback called when Message Waiting Indication (MWI) is received
+    /**Callback called when Message Waiting Indication (MWI) is received.
+       Multiple callbacks may occur with each MessageWaitingType. A \p type
+       of NumMessageWaitingTypes indicates the server is unable to distinguish
+       the message type.
+
+       The \p extraInfo parameter is generally of the form "a/b" where a and b
+       unsigned integers representing new and old message count. However, it
+       may be a simple "yes" or "no" if the remote cannot provide a message
+       count.
      */
     virtual void OnMWIReceived(
       const PString & party,    ///< Name of party MWI is for
@@ -915,7 +973,7 @@ class OpalManager : public PObject
 
        will produce the same result as the single entry "pots:26=h323:10.0.1.1".
 
-       If the "destination" parameter is of the form @filename, then the file
+       If the "destination" parameter is of the form \@filename, then the file
        is read with each line consisting of a pattern=destination route
        specification. 
        
@@ -965,7 +1023,7 @@ class OpalManager : public PObject
                  then the substitution is made so a legal URL can result.
 
          <dn>    Copy all valid consecutive E.164 digits from the "b_party" so
-                 pots:0061298765@vpb:1/2 becomes sip:0061298765@carrier.com
+                 pots:0061298765\@vpb:1/2 becomes sip:0061298765@carrier.com
 
          <dnX>   As above but skip X digits, eg <dn2> skips 2 digits, so
                  pots:00612198765 becomes sip:61298765@carrier.com
@@ -974,8 +1032,8 @@ class OpalManager : public PObject
 
          <dn2ip> Translate digits separated by '*' characters to an IP
                  address. e.g. 10*0*1*1 becomes 10.0.1.1, also
-                 1234*10*0*1*1 becomes 1234@10.0.1.1 and
-                 1234*10*0*1*1*1722 becomes 1234@10.0.1.1:1722.
+                 1234*10*0*1*1 becomes 1234\@10.0.1.1 and
+                 1234*10*0*1*1*1722 becomes 1234\@10.0.1.1:1722.
 
 
        Returns true if an entry was added.
@@ -988,7 +1046,7 @@ class OpalManager : public PObject
        This removes the current routeTable and calls AddRouteEntry for every
        string in the array.
 
-       Returns PTrue if at least one entry was added.
+       Returns true if at least one entry was added.
       */
     PBoolean SetRouteTable(
       const PStringArray & specs  ///<  Array of specification strings.
@@ -1008,15 +1066,15 @@ class OpalManager : public PObject
 
     /**Route the source address to a destination using the route table.
        The source parameter may be something like pots:vpb:1/2 or
-       sip:fred@nurk.com.
+       sip:fred\@nurk.com.
 
        The destination parameter is a partial URL, it does not include the
-       protocol, but may be of the form user@host, or simply digits.
+       protocol, but may be of the form user\@host, or simply digits.
       */
     virtual PString ApplyRouteTable(
-      const PString & source,      /// Source address, including endpoint protocol
-      const PString & destination, /// Destination address read from source protocol
-      PINDEX & entry
+      const PString & source,      ///< Source address, including endpoint protocol
+      const PString & destination, ///< Destination address read from source protocol
+      PINDEX & entry               ///< Index into table to start search
     );
   //@}
 
@@ -1256,11 +1314,23 @@ class OpalManager : public PObject
 
     /**Get the IP Type Of Service byte for media (eg RTP) channels.
      */
-    BYTE GetRtpIpTypeofService() const { return rtpIpTypeofService; }
+    BYTE GetMediaTypeOfService() const { return m_defaultMediaTypeOfService; }
 
     /**Set the IP Type Of Service byte for media (eg RTP) channels.
      */
-    void SetRtpIpTypeofService(unsigned tos) { rtpIpTypeofService = (BYTE)tos; }
+    void SetMediaTypeOfService(unsigned tos) { m_defaultMediaTypeOfService = (BYTE)tos; }
+
+    // For backward compatibility
+    BYTE P_DEPRECATED GetRtpIpTypeofService() const { return m_defaultMediaTypeOfService; }
+    void P_DEPRECATED SetRtpIpTypeofService(unsigned tos) { m_defaultMediaTypeOfService = (BYTE)tos; }
+
+    /**Get the IP Type Of Service byte for media (eg RTP) channels.
+     */
+    BYTE GetMediaTypeOfService(const OpalMediaType & type) const;
+
+    /**Set the IP Type Of Service byte for media (eg RTP) channels.
+     */
+    void SetMediaTypeOfService(const OpalMediaType & type, unsigned tos);
 
     /**Get the maximum transmitted RTP payload size.
        Defaults to maximum safe MTU size (576 bytes as per RFC879) minus the
@@ -1365,7 +1435,7 @@ class OpalManager : public PObject
 
     /**Set the parameters for the video device to be used for input.
        If the name is not suitable for use with the PVideoInputDevice class
-       then the function will return PFalse and not change the device.
+       then the function will return false and not change the device.
 
        This defaults to the value of the PVideoInputDevice::GetInputDeviceNames()
        function.
@@ -1381,7 +1451,7 @@ class OpalManager : public PObject
 
     /**Set the parameters for the video device to be used to preview input.
        If the name is not suitable for use with the PVideoOutputDevice class
-       then the function will return PFalse and not change the device.
+       then the function will return false and not change the device.
 
        This defaults to the value of the PVideoInputDevice::GetOutputDeviceNames()
        function.
@@ -1397,7 +1467,7 @@ class OpalManager : public PObject
 
     /**Set the parameters for the video device to be used for output.
        If the name is not suitable for use with the PVideoOutputDevice class
-       then the function will return PFalse and not change the device.
+       then the function will return false and not change the device.
 
        This defaults to the value of the PVideoInputDevice::GetOutputDeviceNames()
        function.
@@ -1462,7 +1532,9 @@ class OpalManager : public PObject
     PString       defaultUserName;
     PString       defaultDisplayName;
 
-    BYTE          rtpIpTypeofService;
+    BYTE                     m_defaultMediaTypeOfService;
+    map<OpalMediaType, BYTE> m_mediaTypeOfService;
+
     PINDEX        rtpPayloadSizeMax;
     PINDEX        rtpPacketSizeMax;
     unsigned      minAudioJitterDelay;
@@ -1551,6 +1623,7 @@ class OpalManager : public PObject
 
     PThread    * garbageCollector;
     PSyncPoint   garbageCollectExit;
+    bool         garbageCollectSkip;
     PDECLARE_NOTIFIER(PThread, OpalManager, GarbageMain);
 
 #ifdef OPAL_ZRTP
@@ -1566,6 +1639,16 @@ class OpalManager : public PObject
     P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &), false);
     P_REMOVE_VIRTUAL(PBoolean, OnStartMediaPatch(const OpalMediaPatch &), false);
     P_REMOVE_VIRTUAL_VOID(AdjustMediaFormats(const OpalConnection &, OpalMediaFormatList &) const);
+    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const PURL&,const PString&,const PURL&,const PString&,const PString&,const PString&));
+
+
+#ifdef OPAL_HAS_IM
+  public:
+    OpalIMManager & GetIMManager() { return *m_imManager; }
+
+  protected:
+    OpalIMManager * m_imManager;
+#endif
 };
 
 
