@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26320 $
+ * $Revision: 26920 $
  * $Author: rjongbloed $
- * $Date: 2011-08-17 06:58:09 -0500 (Wed, 17 Aug 2011) $
+ * $Date: 2012-01-31 23:02:10 -0600 (Tue, 31 Jan 2012) $
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -711,9 +711,9 @@ class OpalMediaFormat : public PContainer
 {
     PCLASSINFO(OpalMediaFormat, PContainer)
   public:
-    OpalMediaFormat(const OpalMediaFormat & c) : PContainer(c), m_info(c.m_info) { }
+    OpalMediaFormat(const OpalMediaFormat & c);
+    virtual ~OpalMediaFormat();
     OpalMediaFormat & operator=(const OpalMediaFormat & c)     { AssignContents(c); return *this; }
-    virtual ~OpalMediaFormat()                                 { Destruct(); }
     virtual PBoolean MakeUnique();
   protected:
     virtual void DestroyContents();
@@ -1130,6 +1130,13 @@ class OpalMediaFormat : public PContainer
       const OpalMediaFormat & mediaFormat  ///<  Media format to copy to master list
     );
 
+    /**Remove the media format from master format list entry.
+       The media format must already be registered. Returns false if not.
+      */
+    static bool RemoveRegisteredMediaFormat(
+      const OpalMediaFormat & mediaFormat  ///<  Media format to copy to master list
+    );
+
     /**
       * Add a new option to this media format
       */
@@ -1377,6 +1384,7 @@ class OpalVideoFormat : public OpalMediaFormat
 #define OPAL_iLBC           "iLBC"
 #define OPAL_H261           "H.261"
 #define OPAL_H263           "H.263"
+#define OPAL_H264           "H.264"
 #define OPAL_H264_MODE0     "H.264-0"
 #define OPAL_H264_MODE1     "H.264-1"
 #define OPAL_MPEG4          "MPEG4"
