@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25806 $
+ * $Revision: 26728 $
  * $Author: rjongbloed $
- * $Date: 2011-05-19 00:14:31 -0500 (Thu, 19 May 2011) $
+ * $Date: 2011-12-01 22:59:14 -0600 (Thu, 01 Dec 2011) $
  */
 
 #ifndef OPAL_OPAL_MEDIASTRM_H
@@ -162,9 +162,11 @@ class OpalMediaStream : public PSafeObject
        The default behaviour updates the mediaFormat member variable and
        pases the value on to the OpalMediaPatch.
       */
-    virtual bool UpdateMediaFormat(
-      const OpalMediaFormat & mediaFormat,  ///<  New media format
-      bool fromPatch = false                ///<  Is being called from OpalMediaPatch
+    bool UpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat   ///<  New media format
+    );
+    virtual bool InternalUpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat   ///<  New media format
     );
 
     /**Execute the command specified to the transcoder. The commands are
@@ -172,22 +174,13 @@ class OpalMediaStream : public PSafeObject
        apply to a video transcoder.
 
        The default behaviour passes the command on to the OpalMediaPatch.
+
+       @returns true if command is handled.
       */
     virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
     );
 
-    /**Set a notifier to receive commands generated by the transcoder. The
-       commands are highly context sensitive, for example OpalVideoUpdatePicture
-       would only apply to a video transcoder.
-
-       The default behaviour passes the command on to the OpalMediaPatch and
-       sets the member variable commandNotifier.
-      */
-    virtual void SetCommandNotifier(
-      const PNotifier & notifier    ///<  Command to execute.
-    );
-
     /**Open the media stream using the media format.
 
        The default behaviour simply sets the isOpen variable to true.
@@ -425,7 +418,6 @@ class OpalMediaStream : public PSafeObject
     unsigned         mismatchedPayloadTypes;
 
     OpalMediaPatch * mediaPatch;
-    PNotifier        commandNotifier;
 
     RTP_DataFrame::PayloadTypes m_payloadType;
     unsigned                    m_frameTime;
@@ -839,15 +831,6 @@ class OpalAudioMediaStream : public OpalRawMediaStream
       PINDEX frameTime  ///< Individual frame time (if applicable)
     );
 
-    /**Write raw media data to the sink media stream.
-       The default behaviour writes to the PChannel object.
-      */
-    virtual PBoolean WriteData(
-      const BYTE * data,   ///<  Data to write
-      PINDEX length,       ///<  Length of data to read.
-      PINDEX & written     ///<  Length of data actually written
-    );
-
     /**Indicate if the media stream is synchronous.
        Returns true for LID streams.
       */
@@ -857,9 +840,6 @@ class OpalAudioMediaStream : public OpalRawMediaStream
   protected:
     PINDEX   m_soundChannelBuffers;
     unsigned m_soundChannelBufferTime;
-
-    unsigned       m_failSafeRate;
-    PAdaptiveDelay m_failSafeDelay;
 };
 
 #endif // OPAL_PTLIB_AUDIO
@@ -897,6 +877,16 @@ class OpalVideoMediaStream : public OpalMediaStream
 
   /**@name Overrides of PChannel class */
   //@{
+    /**Update the media format. This can be used to adjust the
+       parameters of a codec at run time. Note you cannot change the basic
+       media format, eg change GSM0610 to G.711, only options for that
+       format, eg 6k3 mode to 5k3 mode in G.723.1. If the formats are
+       different then a OpalMediaFormat::Merge() is performed.
+      */
+    virtual bool InternalUpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat   ///<  New media format
+    );
+
     /**Open the media stream.
 
        The default behaviour sets the OpalLineInterfaceDevice format and
