@@ -25,9 +25,9 @@
  * Contributor(s): Craig Southeren (craigs@postincrement.com)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 25300 $
+ * $Revision: 26969 $
  * $Author: rjongbloed $
- * $Date: 2011-03-09 13:22:53 -0600 (Wed, 09 Mar 2011) $
+ * $Date: 2012-02-10 18:45:16 -0600 (Fri, 10 Feb 2012) $
  */
 
 
@@ -1153,7 +1153,7 @@ class OpalMixerNode : public PSafeObject
       std::map<PString, CachedAudio> m_cache;
 
       void PushOne(
-        OpalMixerMediaStream & stream,
+        PSafePtr<OpalMixerMediaStream> & stream,
         CachedAudio & cache,
         const short * audioToSubtract
       );
