@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22858 $
- * $Author: csoutheren $
- * $Date: 2009-06-12 22:50:19 +1000 (Fri, 12 Jun 2009) $
+ * $Revision: 26283 $
+ * $Author: rjongbloed $
+ * $Date: 2011-08-08 02:41:14 -0500 (Mon, 08 Aug 2011) $
  */
 
 #ifndef OPAL_IM_PRES_ENT_H
@@ -38,6 +38,9 @@
 #include <ptlib/safecoll.h>
 #include <ptclib/url.h>
 #include <ptclib/guid.h>
+#include <ptclib/vcard.h>
+
+#include <im/im.h>
 
 #include <list>
 #include <queue>
@@ -50,7 +53,7 @@ class OpalPresentityCommand;
 
 /**Presencu state information
   */
-class OpalPresenceInfo
+class OpalPresenceInfo : public PObject
 {
   public:
     /// Presence states.
@@ -100,19 +103,26 @@ class OpalPresenceInfo
     PString m_note;     ///< Additional information about state change
     PURL    m_entity;   ///< The presentity whose state had changed
     PURL    m_target;   ///< The presentity that is being informed about the state change
+    PTime   m_when;     ///< Time/date of state change
 
     OpalPresenceInfo(State state = Unchanged) : m_state(state) { }
 
     static PString AsString(State state);
     static State FromString(const PString & str);
     PString AsString() const;
+
+    Comparison Compare(const PObject & other) const;
 };
 
 ostream & operator<<(ostream & strm, OpalPresenceInfo::State state);
 
-
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
+class OpalSetLocalPresenceCommand;
+class OpalSubscribeToPresenceCommand;
+class OpalAuthorisationRequestCommand;
+class OpalSendMessageToCommand;
+
 /**Representation of a presence identity.
    This class contains an abstraction of the functionality for "presence"
    using a URL string as the "identity". The concrete class depends on the
@@ -131,8 +141,11 @@ class OpalPresentity : public PSafeObject
   protected:
     /// Construct the presentity class
     OpalPresentity();
+    OpalPresentity(const OpalPresentity & other);
 
   public:
+    ~OpalPresentity();
+
     /**Create a concrete class based on the scheme of the URL provided.
       */
     static OpalPresentity * Create(
@@ -166,32 +179,18 @@ class OpalPresentity : public PSafeObject
 
   /**@name Attributes */
   //@{
-    /**Dictionary of attributes associated with this presentity.
-      */
-    class Attributes :  public PStringToString
-    {
-      public:
-        /// Determine of the attribute exists.
-        virtual bool Has(const PString &   key   ) const { return Contains(key  ); }
-        virtual bool Has(const PString & (*key)()) const { return Contains(key()); }
-
-        /// Get the attribute value.
-        virtual PString Get(const PString &   key   , const PString & deflt = PString::Empty()) const { return (*this)(key  , deflt); }
-                PString Get(const PString & (*key)(), const PString & deflt = PString::Empty()) const { return (*this)(key(), deflt); }
-
-        /// Set the attribute value.
-        virtual void Set(const PString &   key   , const PString & value) { SetAt(key  , value); }
-                void Set(const PString & (*key)(), const PString & value) { SetAt(key(), value); }
-    };
-
     ///< Get the attributes for this presentity.
-    Attributes & GetAttributes() { return m_attributes; }
+    PStringOptions & GetAttributes() { return m_attributes; }
 
-    static const PString & AuthNameKey();         ///< Key for authentication name attribute
-    static const PString & AuthPasswordKey();     ///< Key for authentication password attribute
-    static const PString & FullNameKey();         ///< Key for full name attribute
-    static const PString & SchemeKey();           ///< Key for scheme used attribute
-    static const PString & TimeToLiveKey();       ///< Key for Time-To-Live attribute, in seconds for underlying protocol
+    ///< Get all attribute names for this presentity class.
+    virtual PStringArray GetAttributeNames() const = 0;
+
+    ///< Get all attribute types for this presentity class.
+    virtual PStringArray GetAttributeTypes() const = 0;
+
+    static const PCaselessString & AuthNameKey();         ///< Key for authentication name attribute
+    static const PCaselessString & AuthPasswordKey();     ///< Key for authentication password attribute
+    static const PCaselessString & TimeToLiveKey();       ///< Key for Time-To-Live attribute, in seconds for underlying protocol
 
     /** Get the address-of-record for the presentity.
         This is typically a URL which represents our local identity in the
@@ -265,6 +264,14 @@ class OpalPresentity : public PSafeObject
       const PString & note = PString::Empty()   ///< Additional note attached to the state change
     );
 
+    /** Get our presence state
+    */
+    virtual bool GetLocalPresence(
+      OpalPresenceInfo::State & state, 
+      PString & note
+    );
+
+
     /** Low level function to create a command.
         As commands have protocol specific implementations, we use a factory
         to create them.
@@ -358,66 +365,161 @@ class OpalPresentity : public PSafeObject
       PURL    m_presentity;   ///< Typicall URI address-of-record
       PString m_displayName;  ///< Human readable name
 
+      // RFC4482 contact fields, note most of these are duplicated
+      // in the vCard structure
+      PvCard  m_vCard;        /**< vCard for the buddy. This is
+                                   either a URI pointing to the image, or the image
+                                   itself in the form: "data:text/x-vcard,xxxxx
+                                   as per RFC2397 */
+      PURL    m_icon;         /**< The icon/avatar/photo for the buddy. This is
+                                   either a URI pointing to the image, or the image
+                                   itself in the form: "data:image/jpeg;base64,xxxxx
+                                   as per RFC2397 */
+      PURL    m_map;          /**< The map reference for the buddy. This is may be URL
+                                   to a GIS system or an image file similar to m_icon */
+      PURL    m_sound;        /**< The sound relating to the buddy. This is
+                                   either a URI pointing to the sound, or the sound
+                                   itself in the form: "data:audio/mpeg;base64,xxxxx
+                                   as per RFC2397 */
+      PURL    m_homePage;     ///< Home page for buddy
+
+      // Extra field for protocol dependent "get out of gaol" card
       PString m_contentType;  ///< MIME type code for XML
       PString m_rawXML;       ///< Raw XML of buddy list entry
     };
 
     typedef std::list<BuddyInfo> BuddyList;
 
+    enum BuddyStatus {
+      BuddyStatus_GenericFailure             = -1,
+      BuddyStatus_OK                         = 0,
+      BuddyStatus_SpecifiedBuddyNotFound,
+      BuddyStatus_ListFeatureNotImplemented,
+      BuddyStatus_ListTemporarilyUnavailable,
+      BuddyStatus_ListMayBeIncomplete,
+      BuddyStatus_BadBuddySpecification,
+      BuddyStatus_ListSubscribeFailed,
+      BuddyStatus_AccountNotLoggedIn
+    };
+
     /**Get complete buddy list.
       */
-    virtual bool GetBuddyList(
+    virtual BuddyStatus GetBuddyListEx(
       BuddyList & buddies   ///< List of buddies
     );
+    virtual bool GetBuddyList(
+      BuddyList & buddies   ///< List of buddies
+    )
+    { return GetBuddyListEx(buddies) == BuddyStatus_OK; }
 
     /**Set complete buddy list.
       */
-    virtual bool SetBuddyList(
+    virtual BuddyStatus SetBuddyListEx(
       const BuddyList & buddies   ///< List of buddies
     );
+    virtual bool SetBuddyList(
+      const BuddyList & buddies   ///< List of buddies
+    )
+    { return SetBuddyListEx(buddies) == BuddyStatus_OK; }
+
 
     /**Delete the buddy list.
       */
-    virtual bool DeleteBuddyList();
+    virtual BuddyStatus DeleteBuddyListEx();
+    virtual bool DeleteBuddyList() { return DeleteBuddyListEx() == BuddyStatus_OK; }
 
     /**Get a specific buddy from the buddy list.
        Note the buddy.m_presentity field must be preset to the URI to search
        the buddy list for.
       */
-    virtual bool GetBuddy(
+    virtual BuddyStatus GetBuddyEx(
       BuddyInfo & buddy
     );
+    virtual bool GetBuddy(
+      BuddyInfo & buddy
+    )
+    { return GetBuddyEx(buddy) == BuddyStatus_OK; }
 
     /**Set/Add a buddy to the buddy list.
       */
-    virtual bool SetBuddy(
+    virtual BuddyStatus SetBuddyEx(
       const BuddyInfo & buddy
     );
+    virtual bool SetBuddy(
+      const BuddyInfo & buddy
+    )
+    { return SetBuddyEx(buddy) == BuddyStatus_OK; }
 
     /**Delete a buddy to the buddy list.
       */
-    virtual bool DeleteBuddy(
+    virtual BuddyStatus DeleteBuddyEx(
       const PURL & presentity
     );
+    virtual bool DeleteBuddy(
+      const PURL & presentity
+    )
+    { return DeleteBuddyEx(presentity) == BuddyStatus_OK; }
 
     /**Subscribe to buddy list.
        Send a subscription for the presence of every presentity in the current
        buddy list. This might cause multiple calls to SubscribeToPresence() or
        if the underlying protocol allows a single call for all.
       */
-    virtual bool SubscribeBuddyList(
+    virtual BuddyStatus SubscribeBuddyListEx(
+      PINDEX & successfulCount,
       bool subscribe = true
     );
+    virtual bool SubscribeBuddyList(
+      bool subscribe = true
+    )
+    { PINDEX successfulCount; return SubscribeBuddyListEx(successfulCount, subscribe) == BuddyStatus_OK; }
 
     /**Unsubscribe to buddy list.
        Send an unsubscription for the presence of every presentity in the current
        buddy list. This might cause multiple calls to UnsubscribeFromPresence() or
        if the underlying protocol allows a single call for all.
       */
-    virtual bool UnsubscribeBuddyList();
+    virtual BuddyStatus UnsubscribeBuddyListEx();
+    virtual bool UnsubscribeBuddyList()
+    { return UnsubscribeBuddyListEx() == BuddyStatus_OK; }
+  //@}
+  
+  
+  /**@name Instant Messaging */
+  //@{
+    virtual bool SendMessageTo(
+      const OpalIM & message
+    );
+
+    /** Callback when presentity receives a message
+
+        Default implementation calls m_onReceivedMessageNotifier.
+      */
+    virtual void OnReceivedMessage(
+      const OpalIM & message ///< incoming message
+    );
+
+    typedef PNotifierTemplate<const OpalIM &> ReceivedMessageNotifier;
+    #define PDECLARE_ReceivedMessageNotifier(cls, fn) PDECLARE_NOTIFIER2(OpalPresentity, cls, fn, const OpalIM &)
+    #define PCREATE_ReceivedMessageNotifier(fn) PCREATE_NOTIFIER2(fn, const OpalIM &)
+
+    /// Set the notifier for the OnPresenceChange() function.
+    void SetReceivedMessageNotifier(
+      const ReceivedMessageNotifier & notifier   ///< Notifier to be called by OnReceivedMessage()
+    );
+  //@}
+
+    /** Used to set the AOR after the presentity is created
+        This override allows the descendant class to convert the internal URL into a real AOR,
+        usually by changing the scheme.
+      */
+    virtual void SetAOR(
+      const PURL & aor
+    );
+
+    void Internal_SendMessageToCommand(const OpalSendMessageToCommand & cmd);
 
     virtual PString GetID() const;
-  //@}
 
   protected:
     OpalPresentityCommand * InternalCreateCommand(const char * cmdName);
@@ -425,13 +527,18 @@ class OpalPresentity : public PSafeObject
     OpalManager        * m_manager;
     PGloballyUniqueID    m_guid;
     PURL                 m_aor;
-    Attributes           m_attributes;
+    PStringOptions       m_attributes;
 
     AuthorisationRequestNotifier m_onAuthorisationRequestNotifier;
     PresenceChangeNotifier       m_onPresenceChangeNotifier;
+    ReceivedMessageNotifier      m_onReceivedMessageNotifier;
 
     PMutex m_notificationMutex;
     PAtomicInteger::IntegerType m_idNumber;
+
+    bool m_temporarilyUnavailable;
+    OpalPresenceInfo::State m_localState;      ///< our presentity state
+    PString m_localStateNote;                  ///< Additional note attached to the 
 };
 
 
@@ -449,6 +556,7 @@ class OpalPresentityWithCommandThread : public OpalPresentity
   protected:
     /// Construct the presentity class that uses a command thread.
     OpalPresentityWithCommandThread();
+    OpalPresentityWithCommandThread(const OpalPresentityWithCommandThread & other);
 
   public:
     /** Destory the presentity class that uses a command thread.
@@ -475,12 +583,21 @@ class OpalPresentityWithCommandThread : public OpalPresentity
     );
   //@}
 
-  /**@name Overrides from OpalPresentity */
+  /**@name new functions */
   //@{
     /**Start the background thread to handle commands.
        This is typically called from the concrete classes Open() function.
+
+       If the argument is true (the default) then the thread starts processing
+       queue entries ASAP.
+
+       If the argument is false, the thread is still created and still runs, 
+       but it does not process queue entries. This allows for presenties that
+       may need to allow commands to be paused, for example during initialisation
       */
-    void StartThread();
+    void StartThread(
+      bool startQueue = true
+    );
 
     /**Stop the background thread to handle commands.
        This is typically called from the concrete classes Close() function.
@@ -488,6 +605,13 @@ class OpalPresentityWithCommandThread : public OpalPresentity
        before the object is destroyed.
       */
     void StopThread();
+
+    /**Start/resume processing of queue commands
+      */
+    void StartQueue(
+      bool startQueue = true
+    );
+    
   //@}
 
   protected:
@@ -500,10 +624,10 @@ class OpalPresentityWithCommandThread : public OpalPresentity
     PSyncPoint     m_commandQueueSync;
 
     bool      m_threadRunning;
+    bool      m_queueRunning;
     PThread * m_thread;
 };
 
-
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 /**Abstract class for all OpelPresentity commands.
@@ -536,7 +660,7 @@ class OpalPresentityCommand {
     public: virtual void Process(OpalPresentity & presentity) { dynamic_cast<entity &>(presentity).func(*this); } \
   }; \
   static PFactory<OpalPresentityCommand>::Worker<entity##_##command> \
-  s_entity##_##command(PDefaultPFactoryKey(entity::Class())+typeid(command).name())
+  s_##entity##_##command(PDefaultPFactoryKey(entity::Class())+typeid(command).name())
 
 
 /** Command for subscribing to the status of another presentity.
@@ -576,13 +700,21 @@ class OpalSetLocalPresenceCommand : public OpalPresentityCommand, public OpalPre
 };
 
 
+/** Command for sending an IM 
+  */
+class OpalSendMessageToCommand : public OpalPresentityCommand
+{
+  public:
+    OpalSendMessageToCommand() { }
+
+    OpalIM m_message;
+};
+
 ///////////////////////////////////////////////////////////////////////////////
 
 // Include concrete classes here so the factories are initialised
 #if OPAL_SIP && OPAL_PTLIB_EXPAT
-PFACTORY_LOAD(SIPLocal_Presentity);
-PFACTORY_LOAD(SIPXCAP_Presentity);
-PFACTORY_LOAD(SIPOMA_Presentity);
+PFACTORY_LOAD(SIP_Presentity);
 #endif
 
 
