@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26283 $
+ * $Revision: 26833 $
  * $Author: rjongbloed $
- * $Date: 2011-08-08 02:41:14 -0500 (Mon, 08 Aug 2011) $
+ * $Date: 2011-12-22 23:24:13 -0600 (Thu, 22 Dec 2011) $
  */
 
 #ifndef OPAL_SIP_HANDLERS_H
@@ -98,10 +98,10 @@ public:
   virtual void SetExpire(int e);
 
   virtual int GetExpire()
-    { return expire; }
+    { return m_currentExpireTime; }
 
-  virtual PString GetCallID()
-    { return callID; }
+  virtual const PString & GetCallID() const
+    { return m_callID; }
 
   virtual void SetBody(const PString & /*body*/) { }
 
@@ -157,17 +157,15 @@ protected:
   SIP_PDU::Methods            m_method;
   SIPURL                      m_addressOfRecord;
   SIPURL                      m_remoteAddress;
-  PString                     callID;
-  int                         expire;
-  int                         originalExpire;
-  int                         offlineExpire;
-  unsigned                    authenticationAttempts;
+  PString                     m_callID;
+  unsigned                    m_lastCseq;
+  int                         m_currentExpireTime;
+  int                         m_originalExpireTime;
+  int                         m_offlineExpireTime;
   State                       m_state;
   queue<State>                m_stateQueue;
   bool                        m_receivedResponse;
-  PTimer                      expireTimer; 
-  PTimeInterval               retryTimeoutMin; 
-  PTimeInterval               retryTimeoutMax; 
+  PTimer                      m_expireTimer; 
   SIPURL                      m_proxy;
   OpalProductInfo             m_productInfo;
 
@@ -212,10 +210,11 @@ protected:
   virtual PBoolean SendRequest(SIPHandler::State state);
   void SendStatus(SIP_PDU::StatusCodes code, State state);
 
-  SIPRegister::Params m_parameters;
-  unsigned            m_sequenceNumber;
-  SIPURLList          m_contactAddresses;
-  SIPURLList          m_serviceRoute;
+  SIPRegister::Params  m_parameters;
+  unsigned             m_sequenceNumber;
+  SIPURLList           m_contactAddresses;
+  SIPURLList           m_serviceRoute;
+  OpalTransportAddress m_externalAddress;
 };
 
 
@@ -409,6 +408,11 @@ class SIPHandlersList
     /**
      * Find the SIPHandler object with the specified authRealm
      */
+    PSafePtr<SIPHandler> FindSIPHandlerByAuthRealm(const PString & authRealm, PSafetyMode m);
+
+    /**
+     * Find the SIPHandler object with the specified authRealm & user
+     */
     PSafePtr<SIPHandler> FindSIPHandlerByAuthRealm(const PString & authRealm, const PString & userName, PSafetyMode m);
 
     /**
