@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22097 $
+ * $Revision: 26927 $
  * $Author: rjongbloed $
- * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
+ * $Date: 2012-02-01 16:09:34 -0600 (Wed, 01 Feb 2012) $
  */
 
 #ifndef OPAL_SIP_SIPCON_H
@@ -44,23 +44,85 @@
 #include <opal/rtpconn.h>
 #include <sip/sippdu.h>
 #include <sip/handlers.h>
+
 #if OPAL_VIDEO
 #include <opal/pcss.h>                  // for OpalPCSSConnection
 #include <codec/vidcodec.h>             // for OpalVideoUpdatePicture command
 #endif
 
+#if OPAL_HAS_IM
+#include <im/sipim.h>
+#include <im/rfc4103.h>
+#endif
+
 class OpalCall;
 class SIPEndPoint;
 
-#define SIP_HEADER_PREFIX   "SIP-Header:"
-#define SIP_HEADER_REPLACES SIP_HEADER_PREFIX"Replaces"
+
+/**OpalConnection::StringOption key to a boolean indicating the SDP ptime
+   parameter should be included in audio session streams. Default false.
+  */
+#define OPAL_OPT_OFFER_SDP_PTIME "Offer-SDP-PTime"
+
+/**OpalConnection::StringOption key to a boolean indicating the the state
+   of the "Refer-Sub" header in the REFER request. Default true.
+  */
+#define OPAL_OPT_REFER_SUB       "Refer-Sub"
+
+/**OpalConnection::StringOption key to an integer indicating the the mode
+   for the reliable provisional response system. See PRACKMode for more
+   information. Default is from SIPEndPoint::GetDefaultPRACKMode() which
+   in turn defaults to e_prackSupported.
+  */
+#define OPAL_OPT_PRACK_MODE      "PRACK-Mode"
+
+/**OpalConnection::StringOption key to a boolean indicating that we should
+   make initial SDP offer. Default true.
+  */
+#define OPAL_OPT_INITIAL_OFFER "Initial-Offer"
+
+/**OpalConnection::StringOption key to a string for a regular expression to
+   match the product information, which if matching the remote system, will
+   indicate the remote does not support asymmetric hold as required by the
+   standard.
+   
+   This fault is when SDP sendonly is sent (us putting them on hold), and
+   they reply inactive, which implies them putting us on hold. When we
+   subsequently send recvonly to release our hold to them, they continue to
+   send inactive, and hold is never released.
+
+   Note the OpalProductInfo vendor, name & version strings are concatenated
+   before comparison with the regular expression.
+
+   Defaults to empty string.
+  */
+#define OPAL_OPT_SYMMETRIC_HOLD_PRODUCT "Symmetric-Hold-Product"
+
+/**OpalConnection::StringOption key to a string representing the precise SDP
+   to be included in the INVITE offer. No media streams are opened or any
+   checks whstsoever made on the string. It is simply included as the body of
+   the INVITE.
+   
+   Note this options presence also prevents handling of the response SDP
+   
+   Defaults to empty string which generates SDP from available
+   media formats and media streams.
+  */
+#define OPAL_OPT_EXTERNAL_SDP "External-SDP"
+
+#define SIP_HEADER_PREFIX      "SIP-Header:"
+#define SIP_HEADER_REPLACES    SIP_HEADER_PREFIX"Replaces"
+#define SIP_HEADER_REFERRED_BY SIP_HEADER_PREFIX"Referred-By"
+#define SIP_HEADER_CONTACT     SIP_HEADER_PREFIX"Contact"
+
+#define OPAL_SIP_REFERRED_CONNECTION "Referred-Connection"
 
 
 /////////////////////////////////////////////////////////////////////////
 
 /**Session Initiation Protocol connection.
  */
-//class SIPConnection : public OpalConnection
+
 class SIPConnection : public OpalRTPConnection
 {
   PCLASSINFO(SIPConnection, OpalRTPConnection);
@@ -105,13 +167,8 @@ class SIPConnection : public OpalRTPConnection
      */
     virtual PString GetIdentifier() const;
 
-    /**Get the remote party address as URL.
-       This will return the "best guess" at an address to use in a
-       to call the user again later. Note that under some circumstances this may be
-       different to the value GetRemotePartyAddress() value returns. In particular
-       if a gatekeeper is involved.
-      */
-    virtual PString GetRemotePartyURL() const;
+    /// Call back for connection to act on changed string options
+    virtual void OnApplyStringOptions();
 
     /**Start an outgoing connection.
        This function will initiate the connection to the remote entity, for
@@ -168,28 +225,58 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual bool SetAlertingType(const PString & info);
 
+    /**Get call information of an incoming call.
+       This is protocol dependent information provided about the call. The
+       details are outside the scope of this help.
+
+       For SIP this corresponds to the string contained in the "Call-Info"
+       header field of the INVITE.
+      */
+    virtual PString GetCallInfo() const;
+
     /**Initiate the transfer of an existing call (connection) to a new remote 
        party.
 
-       If remoteParty is a valid call token, then the remote party is transferred
-       to that party (consultation transfer) and both calls are cleared.
+       A REFER command is sent to the remote endpoint to cause it to move the
+       call it has with this endpoint to a new address. This call will, in the
+       end, be cleared. The OnTransferNotify() function can be used to monitor
+       the progress of the transfer in case it fails.
+
+       If remoteParty is a valid call token, then this is short hand for
+       the REFER to the remote endpoint of this call to do an INVITE with
+       Replaces header to the remote party of the supplied tokens call. This
+       is used for consultation transfer where A calls B, B holds A, B calls C,
+       B transfers A to C. The last step is a REFER to A with call details of
+       C that are extracted from the B to C call leg. This short cut is
+       possible because A nd C may be other endpoints but both B's are in this
+       instance of OPAL.
+       
+       In the end, both calls are cleared. The OnTransferNotify() function can
+       be used to monitor the progress of the transfer in case it fails.
      */
     virtual bool TransferConnection(
       const PString & remoteParty   ///<  Remote party to transfer the existing call to
     );
 
     /**Put the current connection on hold, suspending all media streams.
-     */
-    virtual bool HoldConnection();
+       The \p fromRemote parameter indicates if we a putting the remote on
+       hold (false) or it is a request for the remote to put us on hold (true).
 
-    /**Retrieve the current connection from hold, activating all media 
-     * streams.
+       The /p placeOnHold parameter indicates of teh command/request is for
+       going on hold or retrieving from hold.
      */
-    virtual bool RetrieveConnection();
+    virtual bool Hold(
+      bool fromRemote,  ///< Flag for if remote has us on hold, or we have them
+      bool placeOnHold  ///< Flag for setting on or off hold
+    );
 
-    /**Return PTrue if the current connection is on hold.
+    /**Return true if the current connection is on hold.
+       The \p fromRemote parameter indicates if we are testing if the remote
+       system has us on hold, or we have them on hold.
      */
-    virtual PBoolean IsConnectionOnHold();
+    virtual bool IsOnHold(
+      bool fromRemote  ///< Flag for if remote has us on hold, or we have them
+    );
 
     /**Indicate to remote endpoint an alert is in progress.
        If this is an incoming connection and the AnswerCallResponse is in a
@@ -203,7 +290,7 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual PBoolean SetAlerting(
       const PString & calleeName,   ///<  Name of endpoint being alerted.
-      PBoolean withMedia
+      PBoolean withMedia            ///<  Flag to alert with/without media
     );
 
     /**Indicate to remote endpoint we are connected.
@@ -232,6 +319,13 @@ class SIPConnection : public OpalRTPConnection
       OpalMediaStream & stream  ///< Stream to close
     );
 
+    /**Pause media streams for connection.
+      */
+    virtual void OnPauseMediaStream(
+      OpalMediaStream & strm,     ///< Media stream paused/un-paused
+      bool paused                 ///< Flag for pausing/un-pausing
+    );
+
     /**Clean up the termination of the connection.
        This function can do any internal cleaning up and waiting on background
        threads that may be using the connection object.
@@ -256,7 +350,7 @@ class SIPConnection : public OpalRTPConnection
        function when an application wishes to redirect an unwanted incoming
        call.
 
-       The return value is PTrue if the call is to be forwarded, PFalse 
+       The return value is true if the call is to be forwarded, false 
        otherwise. Note that if the call is forwarded, the current connection
        is cleared with the ended call code set to EndedByCallForwarded.
       */
@@ -271,6 +365,16 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual SendUserInputModes GetRealSendUserInputMode() const;
 
+    /**Send a user input indication to the remote endpoint.
+       This is for sending arbitrary strings as user indications.
+
+       The default behaviour is to call SendUserInputTone() for each character
+       in the string.
+      */
+    virtual PBoolean SendUserInputString(
+      const PString & value                   ///<  String value of indication
+    );
+
     /**Send a user input indication to the remote endpoint.
        This sends DTMF emulation user input. If something more sophisticated
        than the simple tones that can be sent using the SendUserInput()
@@ -333,6 +437,13 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual void OnReceivedNOTIFY(SIP_PDU & pdu);
 
+    /**Callback function on receipt of an allowed NOTIFY message.
+       Allowed events are determined by the m_allowedEvents member variable.
+      */
+    virtual void OnAllowedEventNotify(
+      const PString & eventName  ///< Name of event
+    );
+
     /**Handle an incoming REFER PDU
       */
     virtual void OnReceivedREFER(SIP_PDU & pdu);
@@ -345,6 +456,10 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual void OnReceivedPING(SIP_PDU & pdu);
 
+    /**Handle an incoming PRACK PDU
+      */
+    virtual void OnReceivedPRACK(SIP_PDU & pdu);
+
     /**Handle an incoming BYE PDU
       */
     virtual void OnReceivedBYE(SIP_PDU & pdu);
@@ -371,7 +486,10 @@ class SIPConnection : public OpalRTPConnection
 
     /**Handle an incoming Trying response PDU
       */
-    virtual void OnReceivedTrying(SIP_PDU & pdu);
+    virtual void OnReceivedTrying(
+      SIPTransaction & transaction,
+      SIP_PDU & response
+    );
   
     /**Handle an incoming Ringing response PDU
       */
@@ -382,7 +500,7 @@ class SIPConnection : public OpalRTPConnection
     virtual void OnReceivedSessionProgress(SIP_PDU & pdu);
   
     /**Handle an incoming Proxy Authentication Required response PDU
-       Returns: PTrue if handled, if PFalse is returned connection is released.
+       Returns: true if handled, if false is returned connection is released.
       */
     virtual PBoolean OnReceivedAuthenticationRequired(
       SIPTransaction & transaction,
@@ -403,87 +521,169 @@ class SIPConnection : public OpalRTPConnection
   
     /**Handle a sending INVITE request
       */
-    virtual void OnCreatingINVITE(SIP_PDU & pdu);
+    virtual void OnCreatingINVITE(SIPInvite & pdu);
 
-    /**Send a "200 OK" response for the received INVITE message.
-     */
-    virtual PBoolean SendInviteOK(const SDPSessionDescription & sdp);
-	
-    /**Send a response for the received INVITE message.
-     */
-    virtual PBoolean SendInviteResponse(
-      SIP_PDU::StatusCodes code,
-      const char * contact = NULL,
-      const char * extra = NULL,
-      const SDPSessionDescription * sdp = NULL
+    enum TypeOfINVITE {
+      IsNewINVITE,
+      IsDuplicateINVITE,
+      IsReINVITE,
+      IsLoopedINVITE
+    };
+
+    /// Indicate if this is a duplicate or multi-path INVITE
+    TypeOfINVITE CheckINVITE(
+      const SIP_PDU & pdu
+    ) const;
+
+    /**Send an OPTIONS command within this calls dialog.
+       Note if \p reply is non-NULL, this function will block until the
+       transaction completes. Care must be executed in this case that
+       no deadlocks occur.
+      */
+    bool SendOPTIONS(
+      const SIPOptions::Params & params,  ///< Parameters for OPTIONS command
+      SIP_PDU * reply = NULL              ///< Reply to message
+    );
+
+    /**Send an INFO command within this calls dialog.
+       Note if \p reply is non-NULL, this function will block until the
+       transaction completes. Care must be executed in this case that
+       no deadlocks occur.
+      */
+    bool SendINFO(
+      const SIPInfo::Params & params,  ///< Parameters for OPTIONS command
+      SIP_PDU * reply = NULL              ///< Reply to message
     );
   //@}
 
     OpalTransportAddress GetDefaultSDPConnectAddress(WORD port = 0) const;
 
     OpalTransport & GetTransport() const { return *transport; }
+    bool SetTransport(const SIPURL & destination);
 
     SIPEndPoint & GetEndPoint() const { return endpoint; }
     SIPDialogContext & GetDialog() { return m_dialog; }
     const SIPDialogContext & GetDialog() const { return m_dialog; }
-    SIPAuthentication * GetAuthenticator() const { return authentication; }
+    SIPAuthentication * GetAuthenticator() const { return m_authentication; }
+
+    /// Mode for reliable provisional responses.
+    enum PRACKMode {
+      e_prackDisabled,  /**< Do not use PRACK if remote asks for 100rel in Supported
+                             field, refuse call with 420 Bad Extension if 100rel is
+                             in Require header field. */
+                             
+      e_prackSupported, /** Add 100rel to Supported header in outgoing INVITE. For
+                            incoming INVITE enable PRACK is either Supported or
+                            Require headers include 100rel. */
+      e_prackRequired   /** Add 100rel to Require header in outgoing INVITE. For
+                            incoming INVITE enable PRACK is either Supported or
+                            Require headers include 100rel, fail the call with
+                            a 421 Extension Required if missing. */
+    };
+    /**Get active PRACK mode. See PRACKMode enum for details.
+      */
+    PRACKMode GetPRACKMode() const { return m_prackMode; }
+
+    /** Return a bit mask of the allowed SIP methods.
+      */
+    virtual unsigned GetAllowedMethods() const;
 
 #if OPAL_VIDEO
     /**Call when SIP INFO of type application/media_control+xml is received.
 
-       Return PFalse if default reponse of Failure_UnsupportedMediaType is to be returned
+       Return false if default reponse of Failure_UnsupportedMediaType is to be returned
 
       */
     virtual PBoolean OnMediaControlXML(SIP_PDU & pdu);
 #endif
 
-    virtual void OnMediaCommand(OpalMediaCommand & note, INT extra);
+    /** Callback for media commands.
+        Calls the SendIntraFrameRequest on the rtp session
+
+       @returns true if command is handled.
+      */
+    virtual bool OnMediaCommand(
+      OpalMediaStream & stream,         ///< Stream command executed on
+      const OpalMediaCommand & command  ///< Media command being executed
+    );
 
     virtual void OnStartTransaction(SIPTransaction & transaction);
 
     virtual void OnReceivedMESSAGE(SIP_PDU & pdu);
-    virtual void OnMessageReceived(const SIPURL & from, const SIP_PDU & pdu);
+
+    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const SIPURL & /*from*/, const SIP_PDU & /*pdu*/));
+    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const SIP_PDU & /*pdu*/));
+
+#if 0 // OPAL_HAS_IM
+     virtual bool TransmitExternalIM(
+      const OpalMediaFormat & format, 
+      RTP_IMFrame & body
+    );
+#endif
+
+    PString GetLocalPartyURL() const;
 
   protected:
+    virtual bool GarbageCollection();
+    PDECLARE_NOTIFIER(PTimer, SIPConnection, OnSessionTimeout);
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnInviteResponseRetry);
-    PDECLARE_NOTIFIER(PTimer, SIPConnection, OnAckTimeout);
+    PDECLARE_NOTIFIER(PTimer, SIPConnection, OnInviteResponseTimeout);
 
-    virtual RTP_UDP *OnUseRTPSession(
-      const unsigned rtpSessionId,
-      const OpalMediaType & mediaType,
-      const OpalTransportAddress & mediaAddress,
-      OpalTransportAddress & localAddress
-    );
-
-    virtual bool OnSendSDP(
-      bool isAnswerSDP,
+    virtual bool OnSendOfferSDP(
       OpalRTPSessionManager & rtpSessions,
-      SDPSessionDescription & sdpOut
+      SDPSessionDescription & sdpOut,
+      bool offerCurrentOnly
     );
-    virtual bool OfferSDPMediaDescription(
+    virtual bool OnSendOfferSDPSession(
       const OpalMediaType & mediaType,
       unsigned sessionID,
+      OpalRTPSessionManager & rtpSessions,
+      SDPSessionDescription & sdpOut,
+      bool offerOpenMediaStreamOnly
+    );
+
+    virtual bool OnSendAnswerSDP(
       OpalRTPSessionManager & rtpSessions,
       SDPSessionDescription & sdpOut
     );
-    virtual bool AnswerSDPMediaDescription(
+    virtual bool OnSendAnswerSDPSession(
       const SDPSessionDescription & sdpIn,
       unsigned sessionIndex,
       SDPSessionDescription & sdpOut
     );
 
-    virtual void OnReceivedSDP(
+    virtual void OnReceivedAnswerSDP(
       SIP_PDU & pdu
     );
-    virtual bool OnReceivedSDPMediaDescription(
+    virtual bool OnReceivedAnswerSDPSession(
       SDPSessionDescription & sdp,
-      unsigned sessionId
+      unsigned sessionId,
+      bool & multipleFormats
     );
+
+    virtual OpalMediaSession * SetUpMediaSession(
+      const unsigned rtpSessionId,
+      const OpalMediaType & mediaType,
+      const SDPMediaDescription & mediaDescription,
+      OpalTransportAddress & localAddress,
+      bool & remoteChanged
+    );
+
+    bool SendReINVITE(PTRACE_PARAM(const char * msg));
+    void StartPendingReINVITE();
+
     friend class SIPInvite;
     static PBoolean WriteINVITE(OpalTransport & transport, void * param);
-    bool WriteINVITE(OpalTransport & transport);
+    bool WriteINVITE();
 
-    OpalTransport * CreateTransport(const OpalTransportAddress & address, PBoolean isLocalAddress = PFalse);
+    virtual bool SendInviteOK();
+    virtual PBoolean SendInviteResponse(
+      SIP_PDU::StatusCodes code,
+      const SDPSessionDescription * sdp = NULL
+    );
+    virtual void AdjustInviteResponse(
+      SIP_PDU & response
+    );
 
     void UpdateRemoteAddresses();
 
@@ -498,6 +698,8 @@ class SIPConnection : public OpalRTPConnection
     SIPEndPoint         & endpoint;
     OpalTransport       * transport;
     bool                  deleteTransport;
+    unsigned              m_allowedMethods;
+    PStringList           m_allowedEvents;
 
     enum HoldState {
       eHoldOff,
@@ -509,26 +711,43 @@ class SIPConnection : public OpalRTPConnection
     };
     HoldState             m_holdToRemote;
     bool                  m_holdFromRemote;
-    PString               forwardParty;
+    PString               m_forwardParty;
+    SIPURL                m_contactAddress;
+    SIPURL                m_ciscoRemotePartyID;
 
     SIP_PDU             * originalInvite;
     PTime                 originalInviteTime;
-
-    bool                  needReINVITE;
+    time_t                m_sdpSessionId;
+    unsigned              m_sdpVersion; // Really a sequence number
+    bool                  m_needReINVITE;
+    bool                  m_handlingINVITE;
+    bool                  m_symmetricOpenStream;
     SIPDialogContext      m_dialog;
     OpalGloballyUniqueID  m_dialogNotifyId;
     int                   m_appearanceCode;
     PString               m_alertInfo;
-    SIPAuthentication   * authentication;
+    SIPAuthentication   * m_authentication;
+    unsigned              m_authenticatedCseq;
+    PTimer                sessionTimer;
 
     std::map<SIP_PDU::Methods, unsigned> m_lastRxCSeq;
 
-    PTimer                    ackTimer;
-    PTimer                    ackRetry;
-    SIP_PDU                   ackPacket;
-    bool                      ackReceived;
-    PSafePtr<SIPTransaction>  referTransaction;
+    PRACKMode      m_prackMode;
+    bool           m_prackEnabled;
+    unsigned       m_prackSequenceNumber;
+    queue<SIP_PDU> m_responsePackets;
+    PTimer         m_responseFailTimer;
+    PTimer         m_responseRetryTimer;
+    unsigned       m_responseRetryCount;
+
+    bool                      m_referInProgress;
     PSafeList<SIPTransaction> forkedInvitations; // Not for re-INVITE
+    PSafeList<SIPTransaction> pendingInvitations; // For re-INVITE
+    PSafeList<SIPTransaction> m_pendingTransactions;
+
+#if OPAL_FAX
+    bool m_switchedToFaxMode;
+#endif
 
     enum {
       ReleaseWithBYE,
@@ -537,14 +756,22 @@ class SIPConnection : public OpalRTPConnection
       ReleaseWithNothing,
     } releaseMethod;
 
-    OpalMediaFormatList remoteFormatList;
+    OpalMediaFormatList m_remoteFormatList;
+    OpalMediaFormatList m_answerFormatList;
+    bool SetRemoteMediaFormats(SDPSessionDescription * sdp);
 
-    PString GetLocalPartyURL() const;
+    std::map<std::string, SIP_PDU *> m_responses;
 
-  protected:
-    PTimer sessionTimer;
-  public:
-    PDECLARE_NOTIFIER(PTimer, SIPConnection, OnSessionTimeout);
+#if OPAL_HAS_IM
+    PSafePtr<OpalSIPIMContext> m_imContext;
+#endif
+
+  private:
+    P_REMOVE_VIRTUAL_VOID(OnCreatingINVITE(SIP_PDU&));
+    P_REMOVE_VIRTUAL_VOID(OnReceivedTrying(SIP_PDU &));
+
+  friend class SIPTransaction;
+  friend class SIP_RTP_Session;
 };
 
 
@@ -559,7 +786,7 @@ class SIP_RTP_Session : public RTP_UserData
     /**Create a new channel.
      */
     SIP_RTP_Session(
-      const SIPConnection & connection  ///<  Owner of the RTP session
+      SIPConnection & connection  ///<  Owner of the RTP session
     );
   //@}
 
@@ -606,7 +833,7 @@ class SIP_RTP_Session : public RTP_UserData
     virtual void SessionFailing(RTP_Session & /*session*/);
 
   protected:
-    const SIPConnection & connection; /// Owner of the RTP session
+    SIPConnection & connection; /// Owner of the RTP session
 };
 
 
