@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22553 $
- * $Author: csoutheren $
- * $Date: 2009-05-07 08:56:06 +0000 (Thu, 07 May 2009) $
+ * $Revision: 26939 $
+ * $Author: rjongbloed $
+ * $Date: 2012-02-07 01:17:49 -0600 (Tue, 07 Feb 2012) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -93,7 +93,9 @@ class SIPEndPoint : public OpalRTPEndPoint
     /**Create a new endpoint.
      */
     SIPEndPoint(
-      OpalManager & manager
+      OpalManager & manager,
+      unsigned maxConnectionThreads = 10,
+      unsigned maxHandlerThreads = 5
     );
 
     /**Destroy endpoint.
@@ -138,12 +140,12 @@ class SIPEndPoint : public OpalRTPEndPoint
 
        The proto field is optional when passed to a specific endpoint. If it
        is present, however, it must agree with the endpoints protocol name or
-       PFalse is returned.
+       false is returned.
 
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the assiciated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -151,7 +153,7 @@ class SIPEndPoint : public OpalRTPEndPoint
 
        The default behaviour is pure.
      */
-    virtual PBoolean MakeConnection(
+    virtual PSafePtr<OpalConnection> MakeConnection(
       OpalCall & call,                         ///<  Owner of connection
       const PString & party,                   ///<  Remote party to call
       void * userData,                         ///<  Arbitrary data to pass to connection
@@ -182,7 +184,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
     /** Execute garbage collection for endpoint.
-        Returns PTrue if all garbage has been collected.
+        Returns true if all garbage has been collected.
         Default behaviour deletes the objects in the connectionsActive list.
       */
     virtual PBoolean GarbageCollection();
@@ -215,13 +217,29 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
     
     /**Forward the connection using the same token as the specified connection.
-       Return PTrue if the connection is being redirected.
+       Return true if the connection is being redirected.
       */
     virtual PBoolean ForwardConnection(
       SIPConnection & connection,     ///<  Connection to be forwarded
       const PString & forwardParty    ///<  Remote party to forward to
     );
 
+    /**Clear a SIP connection by dialog identifer informataion.
+       This function does not require an OPAL connection to operate, it will
+       attempt to send a BYE to the dialog identified by the information in
+       the SIPDialogContext structure.
+
+       This feature can be useful for servers that had an "unexpected exit"
+       and various clients it was talking to at the time do not implement the
+       RFC4028 session timers, so continue to try and send media forever. They
+       need to be told to cease and desist.
+      */
+    bool ClearDialogContext(
+      const PString & descriptor  ///< Descriptor string for call clearance
+    );
+    bool ClearDialogContext(
+      SIPDialogContext & context  ///< Context for call clearance
+    );
   //@}
   
   /**@name Protocol handling routines */
@@ -267,28 +285,6 @@ class SIPEndPoint : public OpalRTPEndPoint
       SIP_PDU * pdu
     );
 
-    /**Handle an incoming IntervalTooBrief response PDU
-      */
-    virtual void OnReceivedIntervalTooBrief(
-      SIPTransaction & transaction, 
-      SIP_PDU & response)
-    ;
-  
-    /**Handle an incoming Proxy Authentication Required response PDU
-      */
-    virtual void OnReceivedAuthenticationRequired(
-      SIPTransaction & transaction,
-      SIP_PDU & response
-    );
-
-    /**Handle an incoming OK response PDU.
-       This actually gets any PDU of the class 2xx not just 200.
-      */
-    virtual void OnReceivedOK(
-      SIPTransaction & transaction,
-      SIP_PDU & response
-    );
-    
     /**Handle an incoming NOTIFY PDU.
       */
     virtual PBoolean OnReceivedNOTIFY(
@@ -353,38 +349,62 @@ class SIPEndPoint : public OpalRTPEndPoint
       */
     PSafePtr<SIPConnection> GetSIPConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite,
-      SIP_PDU::StatusCodes * errorCode = NULL
+      PSafetyMode mode = PSafeReadWrite,  ///< Lock mode
+      SIP_PDU::StatusCodes * errorCode = NULL ///< Error code if lock fails
     );
 
     virtual PBoolean IsAcceptedAddress(const SIPURL & toAddr);
 
 
-    /**Callback for SIP message received
-     */
-    virtual void OnMessageReceived (const SIPURL & from, const PString & body);
-    virtual void OnMessageReceived (const SIPURL & from, const SIP_PDU & pdu);
+    /**Register an entity to a registrar.
+       This function is asynchronous to permit several registrations to occur
+       at the same time. It can be called several times for different hosts
+       and users.
 
-    /**Register to a registrar. This function is asynchronous to permit
-       several registrations to occur at the same time. It can be
-       called several times for different hosts and users.
-       
-       The username can be of the form user@domain. In that case,
-       the address-of-record field will be set to that value. If not then
-       the address-of-record is constructed from the user and host fields.
-       
-       The realm can be specified when registering, this will
+       The params.m_addressOfRecord field is the only field required, though
+       typically params.m_password is also required. A registration for the
+       user part of params.m_addressOfRecord is made to the a registrar
+       associated with the domain part of the field. The authentication
+       identity is the same as the user field, though this may be set to
+       soemthing different via the params.m_authID field.
+
+       The params.m_registrarAddress may indicate the specific hostname to use
+       for the registrar rather than using the domain part of
+       params.m_addressOfRecord field.
+
+       To aid in flexbility if the params.m_addressOfRecord does not contain
+       a domain and the params.m_registrarAddress does, the the AOR is
+       constructed from them.
+
+       The params.m_realm can be specified when registering, this will
        allow to find the correct authentication information when being
        requested. If no realm is specified, authentication will
        occur with the "best guess" of authentication parameters.
+
+       The Contact address is normally constructed from the listeners active
+       on the SIPEndPoint. This may be overridden to an explicit value via the
+       params.m_contactAddress field.
+
+       The returned "token" is a string that can be used in functions
+       such as Unregister() or IsRegistered(). While it possible to use the
+       AOR for those functions, it is not recommended as a) there may be more
+       than one registration for an AOR and b) the AOR may be constructed from
      */
     bool Register(
-      const SIPRegister::Params & params, ///< Registration paarameters
-      PString & aor                       ///< Resultant address-of-record for unregister
+      const SIPRegister::Params & params, ///< Registration parameters
+      PString & aor,                      ///< Resultant address-of-record for unregister
+      SIP_PDU::StatusCodes * reason = NULL ///< If not null, wait for completion, may take some time
+    );
+
+    // For backward compatibility
+    bool Register(
+      const SIPRegister::Params & params, ///< Registration parameters
+      PString & aor,                      ///< Resultant address-of-record for unregister
+      bool asynchronous                   ///< Wait for completion, may take some time
     );
 
     /// Registration function for backward compatibility.
-    bool Register(
+    bool P_DEPRECATED Register(
       const PString & host,
       const PString & user = PString::Empty(),
       const PString & autName = PString::Empty(),
@@ -395,27 +415,43 @@ class SIPEndPoint : public OpalRTPEndPoint
       const PTimeInterval & maxRetryTime = PMaxTimeInterval
     );
 
-    /**Unregister from a registrar.
-       This will unregister the specified address-of-record. If an empty
-       string is provided then ALL registrations are removed.
-     */
-    bool Unregister(const PString & aor);
+    /**Determine if there is a registration for the entity.
+       The "token" parameter string is typically the string returned by the
+       Register() function which is guaranteed to uniquely identify the
+       specific registration.
 
-    /**Unregister all current registrations.
-      */
-    bool UnregisterAll();
+       For backward compatibility, the AOR can also be used, but as it is
+       possible to have two registrations to the same AOR, this should be
+       avoided.
 
-    /**Returns PTrue if the given URL has been registered 
-     * (e.g.: 6001@seconix.com). The includeOffline parameter indicates
-       if the caller is interested in if we are, to the best of our knowlegde,
-       currently registered (have had recent confirmation) or we are not sure
-       if we are registered or not, but are continually re-trying.
+       The includeOffline parameter indicates if the caller is interested in
+       if we are, to the best of our knowledge, currently registered (have
+       had recent confirmation) or we are not sure if we are registered or
+       not, but are continually re-trying.
      */
     PBoolean IsRegistered(
-      const PString & aor,          ///< Address of record to check
+      const PString & aor,          ///< AOR returned by Register()
       bool includeOffline = false   ///< Include offline registrations
     );
 
+    /**Unregister the address-of-record from a registrar.
+       The "token" parameter string is typically the string returned by the
+       Register() function which is guaranteed to uniquely identify the
+       specific registration.
+
+       For backward compatibility, the AOR can also be used, but as it is
+       possible to have two registrations to the same AOR, this should be
+       avoided.
+     */
+    bool Unregister(
+      const PString & aor    ///< AOR returned by Register()
+    );
+
+    /**Unregister all current registrations.
+       Returns true if at least one registrar is unregistered.
+      */
+    bool UnregisterAll();
+
     /** Returns the number of registered accounts.
      */
     unsigned GetRegistrationsCount() const { return activeSIPHandlers.GetCount(SIP_PDU::Method_REGISTER); }
@@ -428,11 +464,13 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     /** Information provided on the registration status. */
     struct RegistrationStatus {
+      SIPRegisterHandler * m_handler;           ///< Handler for registration
       PString              m_addressofRecord;   ///< Address of record for registration
       bool                 m_wasRegistering;    ///< Was registering or unregistering
       bool                 m_reRegistering;     ///< Was a registration refresh
       SIP_PDU::StatusCodes m_reason;            ///< Reason for status change
       OpalProductInfo      m_productInfo;       ///< Server product info from registrar if available.
+      void               * m_userData;          ///< User data corresponding to this registration
     };
 
     /**Callback called when a registration to a SIP registrar status.
@@ -467,37 +505,57 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
 
-    /**Subscribe to a notifier. This function is asynchronous to permit
-     * several subscriptions to occur at the same time.
+    /**Subscribe to an agent to get event notifications.
+       This function is asynchronous to permit several subscriptions to occur
+       at the same time. It can be called several times for different hosts
+       and users.
+
+       The params.m_eventPackage and params.m_addressOfRecord field are the
+       only field required, though typically params.m_password is also
+       required. A subscription for the user part of params.m_addressOfRecord
+       is made to the an agent associated with the domain part of the field.
+       The authentication identity is the same as the user field, though this
+       may be set to soemthing different via the params.m_authID field.
+
+       The params.m_agentAddress may indicate the specific hostname to use
+       for the registrar rather than using the domain part of
+       params.m_addressOfRecord field.
+
+       To aid in flexbility if the params.m_addressOfRecord does not contain
+       a domain and the params.m_agentAddress does, the the AOR is
+       constructed from them.
+
+       The params.m_realm can be specified when subcribing, this will
+       allow to find the correct authentication information when being
+       requested. If no realm is specified, authentication will
+       occur with the "best guess" of authentication parameters.
+
+       The Contact address is normally constructed from the SIPEndPoint local
+       identity.
+
+       The returned "token" is a string that can be used in functions
+       such as Unregister() or IsRegistered(). While it possible to use the
+       user supplied AOR for those functions, it is not recommended as a) there
+       may be more than one registration for an AOR and b) the AOR may be
+       constructed from fields in the params argument and not be what the user
+       expects.
+
+       The tokenIsAOR can also be used, if false, to return the globally unique
+       ID used for the handler. This is the preferred method, even though the
+       default is to not use it, that is for backward compatibility reasons.
      */
+    bool Subscribe(
+      const SIPSubscribe::Params & params, ///< Subscription parameters
+      PString & token,                     ///< Resultant token for unsubscribe
+      bool tokenIsAOR = true               ///< Token is the address-of-record
+    );
+
+    // For backward compatibility
     bool Subscribe(
       SIPSubscribe::PredefinedPackages eventPackage, ///< Event package being unsubscribed
       unsigned expire,                               ///< Expiry time in seconds
       const PString & aor                            ///< Address-of-record for subscription
     );
-    bool Subscribe(
-      const SIPSubscribe::Params & params, ///< Subscription paarameters
-      PString & aor                        ///< Resultant address-of-record for unsubscribe
-    );
-
-
-    bool Unsubscribe(
-      SIPSubscribe::PredefinedPackages eventPackage,  ///< Event package being unsubscribed
-      const PString & aor                             ///< Address-of-record for subscription
-    );
-    bool Unsubscribe(
-      const PString & eventPackage,  ///< Event package being unsubscribed
-      const PString & aor            ///< Address-of-record for subscription
-    );
-
-    /**Unsubscribe all current subscriptions.
-      */
-    bool UnsubcribeAll(
-      SIPSubscribe::PredefinedPackages eventPackage  ///< Event package being unsubscribed
-    );
-    bool UnsubcribeAll(
-      const PString & eventPackage  ///< Event package being unsubscribed
-    );
 
     /**Returns true if the endpoint is subscribed to some
        event for the given to address. The includeOffline parameter indicates
@@ -505,10 +563,48 @@ class SIPEndPoint : public OpalRTPEndPoint
        currently subscribed (have had recent confirmation) or we are not sure
        if we are subscribed or not, but are continually re-trying.
      */
-    PBoolean IsSubscribed(
+    bool IsSubscribed(
+      const PString & aor,           ///< AOR returned by Subscribe()
+      bool includeOffline = false    ///< Include offline subscription
+    );
+    bool IsSubscribed(
       const PString & eventPackage,  ///< Event package being unsubscribed
       const PString & aor,           ///< Address-of-record for subscription
-      bool includeOffline = false    ///< Include offline subscriptions
+      bool includeOffline = false    ///< Include offline subscription
+    );
+
+    /**Unsubscribe a current subscriptions.
+       The "token" parameter string is typically the string returned by the
+       Subscribe() function which is guaranteed to uniquely identify the
+       specific registration.
+
+       For backward compatibility, the AOR can also be used, but as it is
+       possible to have two susbcriptions to the same AOR, this should be
+       avoided.
+      */
+    bool Unsubscribe(
+      const PString & aor,             ///< Unique token for registration
+      bool invalidateNotifiers = false ///< Notifiers in SIPSubscribe::Params are to be reset
+    );
+    bool Unsubscribe(
+      SIPSubscribe::PredefinedPackages eventPackage,  ///< Event package being unsubscribed
+      const PString & aor,                            ///< Address-of-record for subscription
+      bool invalidateNotifiers = false                ///< Notifiers in SIPSubscribe::Params are to be reset
+    );
+    bool Unsubscribe(
+      const PString & eventPackage,  ///< Event package being unsubscribed
+      const PString & aor,             ///< Address-of-record for subscription
+      bool invalidateNotifiers = false ///< Notifiers in SIPSubscribe::Params are to be reset
+    );
+
+    /**Unsubscribe all current subscriptions.
+       Returns true if at least one subscription is unsubscribed.
+      */
+    bool UnsubcribeAll(
+      SIPSubscribe::PredefinedPackages eventPackage  ///< Event package being unsubscribed
+    );
+    bool UnsubcribeAll(
+      const PString & eventPackage  ///< Event package being unsubscribed
     );
 
     /** Returns the number of registered accounts.
@@ -524,8 +620,18 @@ class SIPEndPoint : public OpalRTPEndPoint
       bool includeOffline = false   ///< Include offline subscriptions
     ) const { return activeSIPHandlers.GetAddresses(includeOffline, SIP_PDU::Method_REGISTER, eventPackage); }
 
+    /** Information provided on the subscription status. */
+    typedef SIPSubscribe::SubscriptionStatus SubscriptionStatus;
+
     /**Callback called when a subscription to a SIP UA status changes.
      */
+    virtual void OnSubscriptionStatus(
+      const SubscriptionStatus & status   ///< Status of subscription request
+    );
+
+    /**Callback called when a subscription to a SIP UA status changes.
+       Now deprecated - called by OnSubscriptionStatus that accepts SIPHandler
+     */
     virtual void OnSubscriptionStatus(
       const PString & eventPackage, ///< Event package subscribed to
       const SIPURL & uri,           ///< Target URI for the subscription.
@@ -534,6 +640,14 @@ class SIPEndPoint : public OpalRTPEndPoint
       SIP_PDU::StatusCodes reason   ///< Status of subscription
     );
 
+    virtual void OnSubscriptionStatus(
+      SIPSubscribeHandler & handler, ///< Event subscription paramaters
+      const SIPURL & uri,            ///< Target URI for the subscription.
+      bool wasSubscribing,           ///< Indication the subscribing or unsubscribing
+      bool reSubscribing,            ///< If subscribing then indication was refeshing subscription
+      SIP_PDU::StatusCodes reason    ///< Status of subscription
+    );
+
     /** Indicate notifications for the specified event package are supported.
       */
     virtual bool CanNotify(
@@ -560,28 +674,64 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
 
-    /**Send a message to the given URL.
+    /**Send IM text
      */
-    PBoolean Message(
-      const PString & to, 
-      const PString & body
+    virtual PBoolean Message(
+      OpalIM & message
     );
-    PBoolean Message(
-      const PString & to, 
-      const PString & body,
-      const PString & remoteContact, 
-      const PString & callID
-    );
-    
-    /**Callback called when a message sent by the endpoint didn't reach
-     * its destination or when the proxy or remote endpoint returns
-     * an error code.
+
+    /**Send SIP message
      */
-    virtual void OnMessageFailed(
-      const SIPURL & messageUrl,
+    bool SendMESSAGE(
+      SIPMessage::Params & params
+    );
+
+    /**Callback called when a message completes in any manner
+     */
+    virtual void OnMESSAGECompleted(
+      const SIPMessage::Params & params,
       SIP_PDU::StatusCodes reason
     );
 
+    struct ConnectionlessMessageInfo {
+      ConnectionlessMessageInfo(OpalTransport & transport, SIP_PDU & pdu)
+        : m_pdu(pdu), m_transport(transport), m_status(ResponseSent)
+      { }
+
+      SIP_PDU & m_pdu;
+      OpalTransport & m_transport;
+      enum {
+        NotHandled,
+        SendOK,
+        MethodNotAllowed,
+        ResponseSent
+      } m_status;
+    };
+
+    typedef PNotifierTemplate<ConnectionlessMessageInfo &> ConnectionlessMessageNotifier;
+    #define PDECLARE_ConnectionlessMessageNotifier(cls, fn) PDECLARE_NOTIFIER2(SIPEndPoint, cls, fn, SIPEndPoint::ConnectionlessMessageInfo &)
+    #define PCREATE_ConnectionlessMessageNotifier(fn) PCREATE_NOTIFIER2(fn, SIPEndPoint::ConnectionlessMessageInfo &)
+
+    void SetConnectionlessMessageNotifier(
+      const ConnectionlessMessageNotifier & notifier
+    )
+    { m_onConnectionlessMessage = notifier; }
+
+
+    /**Send SIP OPTIONS
+     */
+    virtual bool SendOPTIONS(
+      const SIPOptions::Params & params
+    );
+
+    /**Callback called when an OPTIONS command is completed, either
+       successfully or with error.
+     */
+    virtual void OnOptionsCompleted(
+      const SIPOptions::Params & params,    ///< Original parameter for SendOPTIONS() call
+      const SIP_PDU & response              ///< Response packet, check GetStatusCode() for result
+    );
+
 
     /**Publish new state information.
        A expire time of zero will cease to automatically update the publish.
@@ -628,9 +778,16 @@ class SIPEndPoint : public OpalRTPEndPoint
     /**Send a SIP PING to the remote host
       */
     PBoolean Ping(
-      const PString & to
+      const PURL & to
     );
 
+    /**Get default mode for PRACK support.
+      */
+    SIPConnection::PRACKMode GetDefaultPRACKMode() const { return m_defaultPrackMode; }
+
+    /**Set default mode for PRACK support.
+      */
+    void SetDefaultPRACKMode(SIPConnection::PRACKMode mode) { m_defaultPrackMode = mode; }
 
     void SetMIMEForm(PBoolean v) { mimeForm = v; }
     PBoolean GetMIMEForm() const { return mimeForm; }
@@ -660,6 +817,11 @@ class SIPEndPoint : public OpalRTPEndPoint
     ) { inviteTimeout = t; }
     const PTimeInterval & GetInviteTimeout() const { return inviteTimeout; }
 
+    void SetProgressTimeout(
+      const PTimeInterval & t
+    ) { m_progressTimeout = t; }
+    const PTimeInterval & GetProgressTimeout() const { return m_progressTimeout; }
+
     void SetAckTimeout(
       const PTimeInterval & t
     ) { ackTimeout = t; }
@@ -691,10 +853,22 @@ class SIPEndPoint : public OpalRTPEndPoint
      */
     unsigned GetNextCSeq() { return ++lastSentCSeq; }
 
+    /**Set registration search mode.
+       If true then only the user indicated as "local" address
+       (e.g. from OPAL_OPT_CALLING_PARTY_NAME etc) by the will be used
+       in searches of the registrations. No default to first user of the
+       same domain will be performed. */
+    void SetRegisteredUserMode(bool v) { m_registeredUserMode = v; }
+    bool GetRegisteredYserMode() const { return m_registeredUserMode; }
+
     /**Return the SIPAuthentication for a specific realm.
      */
-    PBoolean GetAuthentication(const PString & authRealm, PString & realm, PString & user, PString & password); 
+    bool GetAuthentication(const PString & authRealm, PString & user, PString & password); 
     
+    /**Return the registered proxy URL for the given host.
+     */
+    virtual SIPURL GetRegisteredProxy(const SIPURL & remoteURL);
+
     /**Return the registered party name URL for the given host.
      *
      * That URL can be used in the FORM field of the PDU's. 
@@ -708,19 +882,19 @@ class SIPEndPoint : public OpalRTPEndPoint
     virtual SIPURL GetDefaultRegisteredPartyName(const OpalTransport & transport);
     
 
-    /**Return the contact URL for the given host and user name
-     * based on the listening port of the registration to that host.
-     * 
-     * That URL can be used as as contact field in outgoing
-     * requests.
-     *
-     * The URL is translated if required.
-     *
-     * If no active registration is used, return the result of GetLocalURL
-     * on the given transport.
+    /**Adjust the outgoing PDU to registered information.
+       Various header fields of the PDU must be adjusted to agree with values
+       provided to/from the active registration for the domain the call is being
+       made to. For example the "From" field must agree exactly with
+     
+       If no active registration is available, the result of GetLocalURL() on
+       the given transport is set to the Contact field.
      */
-    SIPURL GetContactURL(const OpalTransport &transport, const SIPURL & localURI);
-
+    void AdjustToRegistration(
+      SIP_PDU & pdu,
+      const OpalTransport & transport,
+      const SIPConnection * connection
+    );
 
     /**Return the local URL for the given transport and user name.
      * That URL can be used as via address, and as contact field in outgoing
@@ -800,9 +974,7 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     virtual void OnStartTransaction(SIPConnection & conn, SIPTransaction & transaction);
 
-#if OPAL_HAS_SIPIM
-    virtual OpalSIPIMManager & GetSIPIMManager() { return m_sipIMManager; }
-#endif
+    void UpdateHandlerIndexes(SIPHandler * handler) { activeSIPHandlers.Update(handler); }
 
   protected:
     PDECLARE_NOTIFIER(PThread, SIPEndPoint, TransportThreadMain);
@@ -811,6 +983,8 @@ class SIPEndPoint : public OpalRTPEndPoint
     SIPURL        proxy;
     PString       userAgentString;
 
+    SIPConnection::PRACKMode m_defaultPrackMode;
+
     bool          mimeForm;
     unsigned      maxRetries;
     PTimeInterval retryTimeoutMin;   // T1
@@ -818,10 +992,12 @@ class SIPEndPoint : public OpalRTPEndPoint
     PTimeInterval nonInviteTimeout;  // T3
     PTimeInterval pduCleanUpTimeout; // T4
     PTimeInterval inviteTimeout;
+    PTimeInterval m_progressTimeout;
     PTimeInterval ackTimeout;
     PTimeInterval registrarTimeToLive;
     PTimeInterval notifierTimeToLive;
     PTimeInterval natBindingTimeout;
+    bool          m_registeredUserMode;
 
     bool              m_shuttingDown;
     SIPHandlersList   activeSIPHandlers;
@@ -834,46 +1010,40 @@ class SIPEndPoint : public OpalRTPEndPoint
     PAtomicInteger          lastSentCSeq;
     int                     m_defaultAppearanceCode;
 
-    struct SIP_PDU_Work
+    struct RegistrationCompletion {
+      PSyncPoint           m_sync;
+      SIP_PDU::StatusCodes m_reason;
+      RegistrationCompletion() : m_reason(SIP_PDU::Information_Trying) { }
+    };
+    std::map<PString, RegistrationCompletion> m_registrationComplete;
+
+    ConnectionlessMessageNotifier m_onConnectionlessMessage;
+
+    // Thread pooling
+    class SIP_Work
     {
       public:
-        SIP_PDU_Work(SIPEndPoint & ep, const PString & token, SIP_PDU * pdu);
-        ~SIP_PDU_Work();
+        SIP_Work(SIPEndPoint & ep, SIP_PDU * pdu, const PString & token);
+        virtual ~SIP_Work();
 
-        void OnReceivedPDU();
+        virtual void Work();
 
         SIPEndPoint & m_endpoint;
-        PString       m_token;
         SIP_PDU     * m_pdu;
+        PString       m_token;
     };
 
-    class PDUThreadPool : public PThreadPool<SIP_PDU_Work>
+    class WorkThreadPool : public PQueuedThreadPool<SIP_Work>
     {
       public:
+        WorkThreadPool(unsigned maxWorkers)
+          : PQueuedThreadPool<SIP_Work>(maxWorkers)
+        { }
         virtual WorkerThreadBase * CreateWorkerThread();
-    } threadPool;
+    } m_connectionThreadPool, m_handlerThreadPool;
 
-    typedef std::queue<SIP_PDU_Work *> SIP_PDUWorkQueue;
-
-    class SIP_PDU_Thread : public PDUThreadPool::WorkerThread
-    {
-      public:
-        SIP_PDU_Thread(PDUThreadPool & pool_);
-
-        void AddWork(SIP_PDU_Work * work);
-        void RemoveWork(SIP_PDU_Work * work);
-        unsigned GetWorkSize() const;
-
-        void Main();
-        void Shutdown();
-
-      protected:
-        PSyncPoint m_sync;
-        SIP_PDUWorkQueue m_pduQueue;
-    };
-
-    virtual void AddWork(SIP_PDU_Work * work);
 
+    // Network interface checking
     enum {
       HighPriority = 80,
       LowPriority  = 30,
@@ -896,9 +1066,12 @@ class SIPEndPoint : public OpalRTPEndPoint
     friend void InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEntry & entry);
     friend void InterfaceMonitor::OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
 
-#if OPAL_HAS_SIPIM
-    OpalSIPIMManager m_sipIMManager;
-#endif
+    bool m_disableTrying;
+
+    P_REMOVE_VIRTUAL_VOID(OnReceivedIntervalTooBrief(SIPTransaction &, SIP_PDU &));
+    P_REMOVE_VIRTUAL_VOID(OnReceivedAuthenticationRequired(SIPTransaction &, SIP_PDU &));
+    P_REMOVE_VIRTUAL_VOID(OnReceivedOK(SIPTransaction &, SIP_PDU &));
+    P_REMOVE_VIRTUAL_VOID(OnMessageFailed(const SIPURL &, SIP_PDU::StatusCodes));
 };
 
 
