@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25988 $
+ * $Revision: 26937 $
  * $Author: rjongbloed $
- * $Date: 2011-06-08 22:06:13 -0500 (Wed, 08 Jun 2011) $
+ * $Date: 2012-02-06 19:54:19 -0600 (Mon, 06 Feb 2012) $
  */
 
 #ifndef OPAL_SIP_SIPPDU_H
@@ -56,6 +56,7 @@ class SIPConnection;
 class SIP_PDU;
 class SIPSubscribeHandler;
 class SIPDialogContext;
+class SIPMIMEInfo;
 
 
 /////////////////////////////////////////////////////////////////////////
@@ -72,12 +73,22 @@ class SIPURL : public PURL
   public:
     SIPURL();
 
+    SIPURL(
+      const PURL & url
+    ) : PURL(url) { }
+    SIPURL & operator=(
+      const PURL & url
+    ) { PURL::operator=(url); return *this; }
+
     /** str goes straight to Parse()
       */
     SIPURL(
       const char * cstr,    ///<  C string representation of the URL.
       const char * defaultScheme = NULL ///<  Default scheme for URL
     );
+    SIPURL & operator=(
+      const char * cstr
+    ) { Parse(cstr); return *this; }
 
     /** str goes straight to Parse()
       */
@@ -85,6 +96,9 @@ class SIPURL : public PURL
       const PString & str,  ///<  String representation of the URL.
       const char * defaultScheme = NULL ///<  Default scheme for URL
     );
+    SIPURL & operator=(
+      const PString & str
+    ) { Parse(str); return *this; }
 
     /** If name does not start with 'sip' then construct URI in the form
         <pre><code>
@@ -102,9 +116,17 @@ class SIPURL : public PURL
     );
 
     SIPURL(
-      const OpalTransportAddress & _address, 
+      const OpalTransportAddress & address, 
       WORD listenerPort = 0
     );
+    SIPURL & operator=(
+      const OpalTransportAddress & address
+    );
+
+    SIPURL(
+      const SIPMIMEInfo & mime,
+      const char * name
+    );
 
     /**Compare the two SIPURLs and return their relative rank.
        Note that does an intelligent comparison according to the rules
@@ -140,6 +162,10 @@ class SIPURL : public PURL
       */
     OpalTransportAddress GetHostAddress() const;
 
+    /**Set the host and port as a transport address.
+      */
+    void SetHostAddress(const OpalTransportAddress & addr);
+
     enum UsageContext {
       ExternalURI,   ///< URI used anywhere outside of protocol
       RequestURI,    ///< Request-URI (after the INVITE)
@@ -182,28 +208,16 @@ class SIPURL : public PURL
   protected:
     void ParseAsAddress(const PString & name, const OpalTransportAddress & _address, WORD listenerPort = 0);
 
-    /** Parses name-addr, like:
-        <pre>
-        "displayname"<scheme:user:password\@host:port;transport=type>;tag=value
-        into:
-        displayname (quotes around name are optional, all before '<' is used)
-        scheme
-        username
-        password
-        hostname
-        port
-        pathStr
-        path
-        paramVars
-        queryVars
-        fragment
-        </pre>
-        Note that tag parameter outside of <> will be lost,
-        but tag in URL without <> will be kept until Sanitise()
-     */
+    // Override from PURL()
     virtual PBoolean InternalParse(
       const char * cstr,
       const char * defaultScheme
+    ) { return ReallyInternalParse(false, cstr, defaultScheme); }
+
+    bool ReallyInternalParse(
+      bool fromField,
+      const char * cstr,
+      const char * defaultScheme
     );
 
     PString        m_displayName;
@@ -260,7 +274,7 @@ class SIPMIMEInfo : public PMIMEInfo
     SIPMIMEInfo(bool compactForm = false);
 
     virtual void PrintOn(ostream & strm) const;
-    virtual void ReadFrom(istream & strm);
+    virtual bool InternalAddMIME(const PString & fieldName, const PString & fieldValue);
 
     void SetCompactForm(bool form) { compactForm = form; }
 
@@ -270,13 +284,13 @@ class SIPMIMEInfo : public PMIMEInfo
     PCaselessString GetContentEncoding() const;
     void SetContentEncoding(const PString & v);
 
-    PString GetFrom() const;
+    SIPURL GetFrom() const;
     void SetFrom(const PString & v);
 
-    PString GetPAssertedIdentity() const;
+    SIPURL GetPAssertedIdentity() const;
     void SetPAssertedIdentity(const PString & v);
 
-    PString GetPPreferredIdentity() const;
+    SIPURL GetPPreferredIdentity() const;
     void SetPPreferredIdentity(const PString & v);
 
     PString GetAccept() const;
@@ -295,14 +309,14 @@ class SIPMIMEInfo : public PMIMEInfo
     PString GetCallID() const;
     void SetCallID(const PString & v);
 
-    PString GetContact() const;
+    SIPURL GetContact() const;
     bool GetContacts(SIPURLList & contacts) const;
     void SetContact(const PString & v);
 
     PString GetSubject() const;
     void SetSubject(const PString & v);
 
-    PString GetTo() const;
+    SIPURL GetTo() const;
     void SetTo(const PString & v);
 
     PString GetVia() const;
@@ -314,10 +328,10 @@ class SIPMIMEInfo : public PMIMEInfo
     PString GetFirstVia() const;
     OpalTransportAddress GetViaReceivedAddress() const;
 
-    PString GetReferTo() const;
+    SIPURL GetReferTo() const;
     void SetReferTo(const PString & r);
 
-    PString GetReferredBy() const;
+    SIPURL GetReferredBy() const;
     void SetReferredBy(const PString & r);
 
     PINDEX  GetContentLength() const;
@@ -670,8 +684,10 @@ class SIP_PDU : public PSafeObject
     StatusCodes GetStatusCode () const       { return m_statusCode; }
     void SetStatusCode (StatusCodes c)       { m_statusCode = c; }
     const SIPURL & GetURI() const            { return m_uri; }
+    void SetURI(const SIPURL & newuri)       { m_uri = newuri; }
     unsigned GetVersionMajor() const         { return m_versionMajor; }
     unsigned GetVersionMinor() const         { return m_versionMinor; }
+    void SetCSeq(unsigned cseq);
     const PString & GetEntityBody() const    { return m_entityBody; }
     void SetEntityBody(const PString & body) { m_entityBody = body; }
     void SetEntityBody();
@@ -679,7 +695,6 @@ class SIP_PDU : public PSafeObject
     void SetInfo(const PString & info)       { m_info = info; }
     const SIPMIMEInfo & GetMIME() const      { return m_mime; }
           SIPMIMEInfo & GetMIME()            { return m_mime; }
-    void SetURI(const SIPURL & newuri)       { m_uri = newuri; }
     SDPSessionDescription * GetSDP(const OpalMediaFormatList & masterList);
     void SetSDP(SDPSessionDescription * sdp);
 
@@ -728,21 +743,18 @@ class SIPDialogContext
 
     const SIPURL & GetRequestURI() const { return m_requestURI; }
     void SetRequestURI(const SIPURL & url) { m_requestURI = url; }
-    bool SetRequestURI(const PString & uri) { return m_requestURI.Parse(uri); }
 
     const PString & GetLocalTag() const { return m_localTag; }
     void SetLocalTag(const PString & tag) { m_localTag = tag; }
 
     const SIPURL & GetLocalURI() const { return m_localURI; }
     void SetLocalURI(const SIPURL & url);
-    bool SetLocalURI(const PString & uri);
 
     const PString & GetRemoteTag() const { return m_remoteTag; }
     void SetRemoteTag(const PString & tag) { m_remoteTag = tag; }
 
     const SIPURL & GetRemoteURI() const { return m_remoteURI; }
     void SetRemoteURI(const SIPURL & url);
-    bool SetRemoteURI(const PString & uri);
 
     const SIPURLList & GetRouteSet() const { return m_routeSet; }
     void SetRouteSet(const PString & str) { m_routeSet.FromString(str); }
@@ -1027,10 +1039,14 @@ class SIPRegister : public SIPTransaction
                                              contact field. Correct behaviour is to return
                                              a contact with the fields it can accept in
                                              the 200 OK */
-      e_CannotRegisterPrivateContacts   /**< Registrar refuses to register any RFC
+      e_CannotRegisterPrivateContacts,  /**< Registrar refuses to register any RFC
                                              contact field. Correct behaviour is to return
                                              a contact with the fields it can accept in
                                              the 200 OK */
+      e_HasApplicationLayerGateway      /**< Router has Application Layer Gateway code that
+                                             is doing address transations, so we do not try
+                                             to do it ourselves as well or it goes horribly
+                                             wrong. */
     };
 
     /// Registrar parameters
@@ -1255,7 +1271,8 @@ class SIPRefer : public SIPTransaction
     SIPRefer(
       SIPConnection & connection,
       const SIPURL & referTo,
-      const SIPURL & referred_by = SIPURL()
+      const SIPURL & referred_by,
+      bool referSub
     );
 
     virtual SIPTransaction * CreateDuplicate() const;
