@@ -31,9 +31,9 @@
  *                 Derek Smithies (derek@indranet.co.nz)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 26277 $
+ * $Revision: 26869 $
  * $Author: rjongbloed $
- * $Date: 2011-08-05 05:10:16 -0500 (Fri, 05 Aug 2011) $
+ * $Date: 2012-01-19 06:36:13 -0600 (Thu, 19 Jan 2012) $
  */
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
@@ -46,12 +46,7 @@
 #include <codec/opalplugin.hpp>
 
 #include "shared/h264frame.h"
-
-#ifdef WIN32
-#include "h264pipe_win32.h"
-#else
-#include "h264pipe_unix.h"
-#endif
+#include "shared/x264wrap.h"
 
 
 #define MY_CODEC      x264                                  // Name of codec (use C variable characters)
@@ -528,19 +523,6 @@ public:
   }
 
 
-#ifdef PLUGIN_CODEC_VERSION_INTERSECT
-  void ClearMediaPacketizations()
-  {
-    for (PluginCodec_Option ** options = (PluginCodec_Option **)m_options; *options != NULL; ++options) {
-      if (strcmp((*options)->m_name, PLUGINCODEC_MEDIA_PACKETIZATIONS) == 0) {
-        *options = NULL;
-        break;
-      }
-    }
-  }
-#endif
-
-
   static void ClampSizes(const LevelInfoStruct & info,
                          unsigned maxWidth,
                          unsigned maxHeight,
@@ -625,8 +607,8 @@ public:
       }
     }
 
-    Change(ProfileInfo[profileIndex].m_Name, original, changed, "Profile"); 
-    Change(LevelInfo[levelIndex].m_Name, original, changed, "Level");
+    Change(ProfileInfo[profileIndex].m_Name, original, changed, Profile.m_name); 
+    Change(LevelInfo[levelIndex].m_Name, original, changed, Level.m_name);
 
     unsigned maxFrameSizeInMB = std::max(LevelInfo[levelIndex].m_MaxFrameSize, String2Unsigned(original[MaxFS.m_name]));
     ClampSizes(LevelInfo[levelIndex],
@@ -652,7 +634,7 @@ public:
   virtual bool ToCustomised(OptionMap & original, OptionMap & changed)
   {
     // Determine the profile
-    std::string str = original["Profile"];
+    std::string str = original[Profile.m_name];
     if (str.empty())
       str = H264_PROFILE_STR_BASELINE;
 
@@ -665,7 +647,7 @@ public:
     Change(ProfileInfo[profileIndex].m_H241, original, changed, H241Profiles.m_name);
 
     // get the current level 
-    str = original["Level"];
+    str = original[Level.m_name];
     if (str.empty())
       str = H264_LEVEL_STR_1_3;
 
@@ -753,7 +735,7 @@ class MyEncoder : public PluginCodec<MY_CODEC>
     unsigned m_tsto;
     unsigned m_keyFramePeriod;
 
-    H264EncCtx m_encoder;
+    H264Encoder m_encoder;
 
   public:
     MyEncoder(const PluginCodec_Definition * defn)
@@ -766,7 +748,7 @@ class MyEncoder : public PluginCodec<MY_CODEC>
       , m_level(DefaultLevelInt)
       , m_constraints(0)
       , m_packetisationMode(1)
-      , m_maxRTPSize(1400)
+      , m_maxRTPSize(PluginCodec_RTP_MaxPacketSize)
       , m_maxNALUSize(1400)
       , m_tsto(31)
       , m_keyFramePeriod(0)
@@ -781,10 +763,8 @@ class MyEncoder : public PluginCodec<MY_CODEC>
          be done in the normal C++ constructor. */
 
       // ABR with bit rate tolerance = 1 is CBR...
-      if (FFMPEGLibraryInstance.Load() && m_encoder.Load(this)) {
-        m_encoder.call(H264ENCODERCONTEXT_CREATE);
+      if (FFMPEGLibraryInstance.Load() && m_encoder.Load(this))
         return true;
-      }
 
       PTRACE(1, MY_CODEC_LOG, "Could not open encoder.");
       return false;
@@ -822,23 +802,24 @@ class MyEncoder : public PluginCodec<MY_CODEC>
       if (strcasecmp(optionName, PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD) == 0)
         return SetOptionUnsigned(m_keyFramePeriod, optionValue, 0);
 
-      if (strcasecmp(optionName, H241Level.m_name) == 0) {
-        size_t i;
-        for (i = 0; i < sizeof(LevelInfo)/sizeof(LevelInfo[0])-1; i++) {
-          if ((unsigned)m_level <= LevelInfo[i].m_H264)
-            break;
+      if (strcasecmp(optionName, Level.m_name) == 0) {
+        for (size_t i = 0; i < sizeof(LevelInfo)/sizeof(LevelInfo[0]); i++) {
+          if (strcasecmp(optionValue, LevelInfo[i].m_Name) == 0) {
+            m_level = LevelInfo[i].m_H264;
+            return true;
+          }
         }
-        unsigned h241level = LevelInfo[i].m_H241;
+        return false;
+      }
 
-        if (!SetOptionUnsigned(h241level, optionValue, 15, 133))
-          return false;
-
-        for (i = 0; i < sizeof(LevelInfo)/sizeof(LevelInfo[0])-1; i++) {
-          if (h241level <= LevelInfo[i].m_H241)
-            break;
+      if (strcasecmp(optionName, Profile.m_name) == 0) {
+        for (size_t i = 0; i < sizeof(ProfileInfo)/sizeof(ProfileInfo[0]); ++i) {
+          if (strcasecmp(optionValue, ProfileInfo[i].m_Name) == 0) {
+            m_profile = ProfileInfo[i].m_H264;
+            return true;
+          }
         }
-        m_level = LevelInfo[i].m_H264;
-        return true;
+        return false;
       }
 
       if (
@@ -887,15 +868,16 @@ class MyEncoder : public PluginCodec<MY_CODEC>
          of them changed. This would do whatever is needed to set parmaeters
          for the actual codec. */
 
-      m_encoder.call(SET_PROFILE_LEVEL, (m_profile << 16) + (m_constraints << 8) + m_level);
-      m_encoder.call(SET_FRAME_WIDTH, m_width);
-      m_encoder.call(SET_FRAME_HEIGHT, m_height);
-      m_encoder.call(SET_FRAME_RATE, m_frameRate);
-      m_encoder.call(SET_TARGET_BITRATE, m_bitRate/1000);
-      m_encoder.call(SET_MAX_FRAME_SIZE, std::min(m_maxRTPSize, m_maxNALUSize));
-      m_encoder.call(SET_TSTO, m_tsto);
-      m_encoder.call(SET_MAX_KEY_FRAME_PERIOD, m_keyFramePeriod);
-      m_encoder.call(APPLY_OPTIONS);
+      m_encoder.SetProfileLevel(m_profile, m_level, m_constraints);
+      m_encoder.SetFrameWidth(m_width);
+      m_encoder.SetFrameHeight(m_height);
+      m_encoder.SetFrameRate(m_frameRate);
+      m_encoder.SetTargetBitrate(m_bitRate/1000);
+      m_encoder.SetMaxRTPPayloadSize(std::min(m_maxRTPSize-PluginCodec_RTP_MinHeaderSize, m_maxNALUSize));
+      m_encoder.SetTSTO(m_tsto);
+      m_encoder.SetMaxKeyFramePeriod(m_keyFramePeriod);
+      m_encoder.ApplyOptions();
+
       PTRACE(3, MY_CODEC_LOG, "Applied options: "
                               "prof=" << m_profile << " "
                               "lev=" << m_level << " "
@@ -914,14 +896,10 @@ class MyEncoder : public PluginCodec<MY_CODEC>
                              unsigned & toLen,
                              unsigned & flags)
     {
-      int ret = 0;
-      unsigned headerSize = PluginCodec_RTP_GetHeaderLength(toPtr);
-      m_encoder.call(ENCODE_FRAMES,
-                     (const u_char *)fromPtr, fromLen,
-                     (u_char *)toPtr, toLen,
-                     headerSize,
-                     flags, ret);
-      return ret != 0;
+      return m_encoder.EncodeFrames((const unsigned char *)fromPtr, fromLen,
+                                    (unsigned char *)toPtr, toLen,
+                                     PluginCodec_RTP_GetHeaderLength(toPtr),
+                                     flags);
     }
 };
 
@@ -934,6 +912,7 @@ class MyDecoder : public PluginCodec<MY_CODEC>
     AVCodecContext * m_context;
     AVFrame        * m_picture;
     H264Frame        m_fullFrame;
+    size_t           m_outputSize;
 
   public:
     MyDecoder(const PluginCodec_Definition * defn)
@@ -941,6 +920,7 @@ class MyDecoder : public PluginCodec<MY_CODEC>
       , m_codec(NULL)
       , m_context(NULL)
       , m_picture(NULL)
+      , m_outputSize(352*288*3/2 + sizeof(PluginCodec_Video_FrameHeader) + PluginCodec_RTP_MinHeaderSize)
     {
     }
 
@@ -973,6 +953,17 @@ class MyDecoder : public PluginCodec<MY_CODEC>
       if ((m_context = FFMPEGLibraryInstance.AvcodecAllocContext()) == NULL)
         return false;
 
+      m_context->workaround_bugs = FF_BUG_AUTODETECT;
+      m_context->error_recognition = FF_ER_AGGRESSIVE;
+      m_context->idct_algo = FF_IDCT_H264;
+      m_context->error_concealment = FF_EC_GUESS_MVS | FF_EC_DEBLOCK;
+      m_context->flags = CODEC_FLAG_INPUT_PRESERVED | CODEC_FLAG_EMU_EDGE;
+      m_context->flags2 = CODEC_FLAG2_BRDO |
+                          CODEC_FLAG2_MEMC_ONLY |
+                          CODEC_FLAG2_DROP_FRAME_TIMECODE |
+                          CODEC_FLAG2_SKIP_RD |
+                          CODEC_FLAG2_CHUNKS;
+
       if ((m_picture = FFMPEGLibraryInstance.AvcodecAllocFrame()) == NULL)
         return false;
 
@@ -985,14 +976,7 @@ class MyDecoder : public PluginCodec<MY_CODEC>
 
     virtual size_t GetOutputDataSize()
     {
-      /* Example just uses maximums. If they are very large, e.g. 4096x4096
-         then a VERY large buffer will be created by OPAL. It is more sensible
-         to use a "typical" size here and allow it to get bigger via the
-         PluginCodec_ReturnCoderBufferTooSmall mechanism if needed. */
-
-      return MyMaxWidth*MyMaxHeight*3/2 + 
-             sizeof(PluginCodec_Video_FrameHeader) +
-             PluginCodec_RTP_MinHeaderSize;
+      return m_outputSize;
     }
 
 
@@ -1006,11 +990,8 @@ class MyDecoder : public PluginCodec<MY_CODEC>
         return false;
 
       RTPFrame srcRTP((const unsigned char *)fromPtr, fromLen);
-      if (!m_fullFrame.SetFromRTPFrame(srcRTP, flags)) {
-        m_fullFrame.BeginNewFrame();
-        flags |= PluginCodec_ReturnCoderRequestIFrame;
+      if (!m_fullFrame.SetFromRTPFrame(srcRTP, flags))
         return true;
-      }
 
       // Wait for marker to indicate end of frame
       if (!srcRTP.GetMarker())
@@ -1039,7 +1020,6 @@ class MyDecoder : public PluginCodec<MY_CODEC>
       if (bytesDecoded <= 0) {
         // Should have output log error in logging callback dyna.cxx
         flags |= PluginCodec_ReturnCoderRequestIFrame;
-        m_fullFrame.BeginNewFrame();
         return true;
       }
 
@@ -1070,8 +1050,10 @@ class MyDecoder : public PluginCodec<MY_CODEC>
       size_t ySize = m_context->width * m_context->height;
       size_t uvSize = ySize/4;
       size_t newToLen = ySize+uvSize+uvSize+sizeof(PluginCodec_Video_FrameHeader)+PluginCodec_RTP_MinHeaderSize;
-      if (newToLen > toLen)
+      if (newToLen > toLen) {
+        m_outputSize = newToLen;
         flags |= PluginCodec_ReturnCoderBufferTooSmall;
+      }
       else {
         flags |= PluginCodec_ReturnCoderLastFrame;
 
@@ -1331,15 +1313,7 @@ extern "C"
     if (version < PLUGIN_CODEC_VERSION_OPTIONS)
       return NULL;
 
-#ifdef PLUGIN_CODEC_VERSION_INTERSECT
-    if (version < PLUGIN_CODEC_VERSION_INTERSECT) {
-      for (size_t i = 0; i < MyCodecDefinitionSize; ++i) {
-        MyPluginMediaFormat * info = (MyPluginMediaFormat *)MyCodecDefinition[i].userData;
-        if (info != NULL)
-          info->ClearMediaPacketizations();
-      }
-    }
-#endif
+    PluginCodec_MediaFormat::AdjustAllForVersion(version, MyCodecDefinition, MyCodecDefinitionSize);
 
     *count = MyCodecDefinitionSize;
     return MyCodecDefinition;
