@@ -30,25 +30,20 @@
 /*****************************************************************************/
 
 #include "h264frame.h"
-#include <stdlib.h>
-#include <string.h>
 
-#ifdef GPL_HELPER_APP
-  #include "../gpl/enc-ctx.h"
-  #ifndef _WIN32
-    #include "../gpl/x264loader_unix.h"
-    extern X264Library X264Lib;
-  #else
-    #define X264_NAL_ENCODE x264_nal_encode 
-  #endif
-#else
-  #include <codec/opalplugin.hpp>
-#endif
+#include <codec/opalplugin.hpp>
+#include <stdlib.h>
+#include <memory.h>
 
 
 #define MAX_FRAME_SIZE 128 * 1024
 
 
+#if PLUGINCODEC_TRACING
+  static char const FrameTraceName[] = "x264-frame";
+#endif
+
+
 H264Frame::H264Frame ()
 {
   m_timestamp = 0;
@@ -57,10 +52,11 @@ H264Frame::H264Frame ()
   m_NALs = NULL;
   m_numberOfNALsReserved = 0;
 
-  BeginNewFrame();
+  BeginNewFrame(0);
 }
 
-void H264Frame::BeginNewFrame ()
+
+void H264Frame::BeginNewFrame(uint32_t numberOfNALs)
 {
   m_encodedFrameLen = 0;
 
@@ -74,8 +70,14 @@ void H264Frame::BeginNewFrame ()
 
   m_currentFU = 0;
 
+  if (numberOfNALs > 0) {
+    if (m_NALs)
+      free(m_NALs);
+    m_NALs = (NALU *)malloc(numberOfNALs * sizeof(NALU));
+  }
 }
 
+
 H264Frame::~H264Frame ()
 {
   if (m_encodedFrame)
@@ -84,58 +86,34 @@ H264Frame::~H264Frame ()
     free(m_NALs);
 }
 
-#ifdef GPL_HELPER_APP
-void H264Frame::SetFromFrame(x264_nal_t *NALs, int numberOfNALs)
+
+void H264Frame::AddNALU(uint8_t type, uint32_t length, const uint8_t * payload)
 {
-  int vopBufferLen;
-  int currentNAL = 0;
-
   uint8_t* currentPositionInFrame=(uint8_t*) m_encodedFrame;
-  if (m_NALs) free(m_NALs);
-  m_NALs = (h264_nal_t *)malloc(numberOfNALs * sizeof(h264_nal_t));
 
-  m_encodedFrameLen = 0;
-  m_numberOfNALsInFrame = 0;
-  m_currentNAL=0;
+  m_NALs[m_numberOfNALsInFrame].type = type;
+  m_NALs[m_numberOfNALsInFrame].length = length;
+  m_NALs[m_numberOfNALsInFrame].offset = m_encodedFrameLen;
+  memcpy(m_encodedFrame+m_encodedFrameLen, payload, length);
 
-  // read the nals out of the encoder and create meta data about their location and size in the frame
-  for (currentNAL = 0; currentNAL < numberOfNALs; currentNAL++) {
-    m_NALs[m_numberOfNALsInFrame].type = NALs[currentNAL].i_type;
-    m_NALs[m_numberOfNALsInFrame].length = NALs[currentNAL].i_payload-4;
-    m_NALs[m_numberOfNALsInFrame].offset = m_encodedFrameLen;
-    memcpy(currentPositionInFrame, NALs[currentNAL].p_payload+4, m_NALs[m_numberOfNALsInFrame].length);
-
-    uint32_t header = 0;
-    if (IsStartCode(currentPositionInFrame))
-    {
-      header = currentPositionInFrame[2] == 1 ? 3 : 4;
-    }
-    m_NALs[m_numberOfNALsInFrame].length -= header;
-    m_NALs[m_numberOfNALsInFrame].offset += header;
-    PTRACE(6, "x264", "Loaded NAL unit #" << currentNAL << " - type " << NALs[currentNAL].i_type);
-
-    uint8_t* NALptr = NALs[currentNAL].p_payload;
-    if (PTRACE_CHECK(4) && (NALs[currentNAL].i_type == H264_NAL_TYPE_SEQ_PARAM)) {
-      PTRACE(4, "x264", "Profile: " << (int)NALptr[0] << 
-                              " Level: "   << (int)NALptr[2] << 
-		   	      " Constraints: " << (NALptr[1] & 0x80 ? 1 : 0) 
-			                        << (NALptr[1] & 0x40 ? 1 : 0) 
-					        << (NALptr[1] & 0x20 ? 1 : 0) 
-					        << (NALptr[1] & 0x10 ? 1 : 0));
-    }
-
-    m_numberOfNALsInFrame++;
-    m_encodedFrameLen += NALs[currentNAL].i_payload;
-    currentPositionInFrame += NALs[currentNAL].i_payload;
+  if (type == H264_NAL_TYPE_SEQ_PARAM) {
+    PTRACE(4, FrameTraceName, "Profile: "   << (unsigned)payload[0] << 
+                      " Level: "    << (unsigned)payload[2] << 
+		   " Constraints: " << (payload[1] & 0x80 ? 1 : 0) 
+			            << (payload[1] & 0x40 ? 1 : 0) 
+				    << (payload[1] & 0x20 ? 1 : 0) 
+				    << (payload[1] & 0x10 ? 1 : 0));
   }
-  PTRACE(6, "x264", "Loaded an encoded frame of " << m_encodedFrameLen << " bytes consisiting of " << numberOfNALs << " NAL units");
+
+  m_numberOfNALsInFrame++;
+  m_encodedFrameLen += length;
 }
-#endif
+
 
 bool H264Frame::GetRTPFrame(RTPFrame & frame, unsigned int & flags)
 {
   flags = 0;
-  flags |= (IsSync()) ? isIFrame : 0;
+  flags |= (IsSync()) ? PluginCodec_ReturnCoderIFrame : 0;
   if (m_currentNAL < m_numberOfNALsInFrame) 
   { 
     uint32_t curNALLen = m_NALs[m_currentNAL].length;
@@ -168,9 +146,9 @@ bool H264Frame::GetRTPFrame(RTPFrame & frame, unsigned int & flags)
         memcpy(frame.GetPayloadPtr(), curNALPtr, curNALLen);
         frame.SetTimestamp(m_timestamp);
         frame.SetMarker((m_currentNAL + 1) >= m_numberOfNALsInFrame ? 1 : 0);
-        flags |= frame.GetMarker() ? isLastFrame : 0;  // marker bit on last frame of video
+        flags |= frame.GetMarker() ? PluginCodec_ReturnCoderLastFrame : 0;  // marker bit on last frame of video
 
-        PTRACE(6, "x264", "Encapsulating NAL unit #" << m_currentNAL << "/" << (m_numberOfNALsInFrame-1) << " of " << curNALLen << " bytes as a regular NAL unit");
+        PTRACE(6, FrameTraceName, "Encapsulating NAL unit #" << m_currentNAL << "/" << (m_numberOfNALsInFrame-1) << " of " << curNALLen << " bytes as a regular NAL unit");
         m_currentNAL++;
         return true;
 #ifdef SEND_STAP_PACKETS
@@ -206,7 +184,7 @@ bool H264Frame::EncapsulateSTAP (RTPFrame & frame, unsigned int & flags) {
     highestNALNumberInSTAP--;
   }
 
-  PTRACE(6, "x264", "Encapsulating NAL units " << m_currentNAL << "-"<< (highestNALNumberInSTAP-1) << "/" << (m_numberOfNALsInFrame-1) << " as a STAP of " << STAPLen);
+  PTRACE(6, FrameTraceName, "Encapsulating NAL units " << m_currentNAL << "-"<< (highestNALNumberInSTAP-1) << "/" << (m_numberOfNALsInFrame-1) << " as a STAP of " << STAPLen);
 
   frame.SetPayloadSize(1); // for stap header
 
@@ -227,7 +205,7 @@ bool H264Frame::EncapsulateSTAP (RTPFrame & frame, unsigned int & flags) {
     memcpy ((uint8_t*)frame.GetPayloadPtr() + frame.GetPayloadSize() - curNALLen, (void *)curNALPtr, curNALLen);
 
     if ((*curNALPtr & 0x60) > maxNRI) maxNRI = *curNALPtr & 0x60;
-    PTRACE(6, "x264", "Adding NAL unit " << m_currentNAL << "/" << (m_numberOfNALsInFrame-1) << " of " << curNALLen << " bytes to STAP");
+    PTRACE(6, FrameTraceName, "Adding NAL unit " << m_currentNAL << "/" << (m_numberOfNALsInFrame-1) << " of " << curNALLen << " bytes to STAP");
     m_currentNAL++;
   }
 
@@ -237,7 +215,7 @@ bool H264Frame::EncapsulateSTAP (RTPFrame & frame, unsigned int & flags) {
   memset (frame.GetPayloadPtr(), 24 | maxNRI, 1);
   frame.SetTimestamp(m_timestamp);
   frame.SetMarker(m_currentNAL >= m_numberOfNALsInFrame ? 1 : 0);
-  flags |= frame.GetMarker() ? isLastFrame : 0;  // marker bit on last frame of video
+  flags |= frame.GetMarker() ? PluginCodec_ReturnCoderLastFrame : 0;  // marker bit on last frame of video
 
   return true;
 }
@@ -283,11 +261,11 @@ bool H264Frame::EncapsulateFU(RTPFrame & frame, unsigned int & flags) {
     memcpy ((uint8_t*)frame.GetPayloadPtr()+2, m_currentNALFURemainingDataPtr, curFULen);
     frame.SetTimestamp(m_timestamp);
     frame.SetMarker((last && ((m_currentNAL+1) >= m_numberOfNALsInFrame)) ? 1 : 0);
-    flags |= frame.GetMarker() ? isLastFrame : 0;  // marker bit on last frame of video
+    flags |= frame.GetMarker() ? PluginCodec_ReturnCoderLastFrame : 0;  // marker bit on last frame of video
 
     m_currentNALFURemainingDataPtr += curFULen;
     m_currentNALFURemainingLen -= curFULen;
-    PTRACE(6, "x264", "Encapsulating "<< curFULen << " bytes of NAL " << m_currentNAL<< "/" << (m_numberOfNALsInFrame-1) << " as a FU (" << m_currentNALFURemainingLen << " bytes remaining)");
+    PTRACE(6, FrameTraceName, "Encapsulating "<< curFULen << " bytes of NAL " << m_currentNAL<< "/" << (m_numberOfNALsInFrame-1) << " as a FU (" << m_currentNALFURemainingLen << " bytes remaining)");
   } 
   if (m_currentNALFURemainingLen==0)
   {
@@ -308,26 +286,32 @@ bool H264Frame::SetFromRTPFrame(RTPFrame & frame, unsigned int & flags)
       curNALType <= H264_NAL_TYPE_FILLER_DATA)
   {
     // regular NAL - put in buffer, adding the header.
-    PTRACE(6, "x264", "Deencapsulating a regular NAL unit NAL of " << frame.GetPayloadSize() - 1 << " bytes (type " << (int) curNALType << ")");
+    PTRACE(6, FrameTraceName, "Deencapsulating a regular NAL unit NAL of " << frame.GetPayloadSize() - 1 << " bytes (type " << (int) curNALType << ")");
     AddDataToEncodedFrame(frame.GetPayloadPtr() + 1, frame.GetPayloadSize() - 1, *(frame.GetPayloadPtr()), true);
+    return true;
   } 
   else if (curNALType == 24) 
   {
     // stap-A (single time aggregation packet )
-    return DeencapsulateSTAP (frame, flags);
+    if (DeencapsulateSTAP (frame, flags))
+      return true;
   } 
   else if (curNALType == 28) 
   {
     // Fragmentation Units
-    return DeencapsulateFU (frame, flags);
+    if (DeencapsulateFU (frame, flags))
+      return true;
   }
   else
   {
-    PTRACE(2, "x264", "Skipping unsupported NAL unit type " << (unsigned)curNALType);
-    return false;
+    PTRACE(2, FrameTraceName, "Skipping unsupported NAL unit type " << (unsigned)curNALType);
   }
-  return true;
+
+  BeginNewFrame(0);
+  flags |= PluginCodec_ReturnCoderRequestIFrame;
+  return false;
 }
+
 bool H264Frame::IsSync () {
   uint32_t i;
 
@@ -347,7 +331,7 @@ bool H264Frame::DeencapsulateSTAP (RTPFrame & frame, unsigned int & /*flags*/) {
   uint8_t* curSTAP = frame.GetPayloadPtr() + 1;
   uint32_t curSTAPLen = frame.GetPayloadSize() - 1; 
 
-  PTRACE(6, "x264", "Deencapsulating a STAP of " << curSTAPLen << " bytes");
+  PTRACE(6, FrameTraceName, "Deencapsulating a STAP of " << curSTAPLen << " bytes");
   while (curSTAPLen > 0)
   {
     // first, theres a 2 byte length field
@@ -355,13 +339,13 @@ bool H264Frame::DeencapsulateSTAP (RTPFrame & frame, unsigned int & /*flags*/) {
     curSTAP += 2;
     // then the header, followed by the body.  We'll add the header
     // in the AddDataToEncodedFrame - that's why the nal body is dptr + 1
-    PTRACE(6, "x264", "Deencapsulating an NAL unit of " << len << " bytes (type " << (int)(*curSTAP && 0x1f) << ") from STAP");
+    PTRACE(6, FrameTraceName, "Deencapsulating an NAL unit of " << len << " bytes (type " << (int)(*curSTAP && 0x1f) << ") from STAP");
     AddDataToEncodedFrame(curSTAP + 1,  len - 1, *curSTAP, true);
     curSTAP += len;
     if ((len + 2) > curSTAPLen)
     {
       curSTAPLen = 0;
-      PTRACE(2, "x264", "Error deencapsulating STAP, STAP header says its " << len + 2 << " bytes long but there are only " << curSTAPLen << " bytes left of the packet");
+      PTRACE(2, FrameTraceName, "Error deencapsulating STAP, STAP header says its " << len + 2 << " bytes long but there are only " << curSTAPLen << " bytes left of the packet");
       return false;
     }
     else
@@ -379,7 +363,7 @@ bool H264Frame::DeencapsulateFU (RTPFrame & frame, unsigned int & /*flags*/) {
 
   if ((curFUPtr[1] & 0x80) && !(curFUPtr[1] & 0x40))
   {
-    PTRACE(6, "x264", "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (Startbit, !Endbit)");
+    PTRACE(6, FrameTraceName, "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (Startbit, !Endbit)");
     if (m_currentFU) {
       m_currentFU=1;
     }
@@ -392,7 +376,7 @@ bool H264Frame::DeencapsulateFU (RTPFrame & frame, unsigned int & /*flags*/) {
   } 
   else if (!(curFUPtr[1] & 0x80) && !(curFUPtr[1] & 0x40))
   {
-    PTRACE(6, "x264", "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (!Startbit, !Endbit)");
+    PTRACE(6, FrameTraceName, "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (!Startbit, !Endbit)");
     if (m_currentFU)
     {
       m_currentFU++;
@@ -401,13 +385,13 @@ bool H264Frame::DeencapsulateFU (RTPFrame & frame, unsigned int & /*flags*/) {
     else
     {
       m_currentFU=0;
-      PTRACE(2, "x264", "Received an intermediate FU without getting the first - dropping!");
+      PTRACE(2, FrameTraceName, "Received an intermediate FU without getting the first - dropping!");
       return false;
     }
   } 
   else if (!(curFUPtr[1] & 0x80) && (curFUPtr[1] & 0x40))
   {
-    PTRACE(6, "x264", "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (!Startbit, Endbit)");
+    PTRACE(6, FrameTraceName, "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (!Startbit, Endbit)");
     if (m_currentFU) {
       m_currentFU=0;
       AddDataToEncodedFrame( curFUPtr + 2, curFULen - 2, 0, false);
@@ -415,14 +399,14 @@ bool H264Frame::DeencapsulateFU (RTPFrame & frame, unsigned int & /*flags*/) {
     else
     {
       m_currentFU=0;
-      PTRACE(2, "x264", "Received a last FU without getting the first - dropping!");
+      PTRACE(2, FrameTraceName, "Received a last FU without getting the first - dropping!");
       return false;
     }
   } 
   else if ((curFUPtr[1] & 0x80) && (curFUPtr[1] & 0x40))
   {
-    PTRACE(6, "x264", "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (Startbit, Endbit)");
-    PTRACE(2, "x264", "Received a FU with both Starbit and Endbit set - This MUST NOT happen!");
+    PTRACE(6, FrameTraceName, "Deencapsulating a FU of " << frame.GetPayloadSize() - 1 << " bytes (Startbit, Endbit)");
+    PTRACE(2, FrameTraceName, "Received a FU with both Starbit and Endbit set - This MUST NOT happen!");
     m_currentFU=0;
     return false;
   } 
@@ -436,11 +420,11 @@ void H264Frame::AddDataToEncodedFrame (uint8_t *data, uint32_t dataLen, uint8_t
 
   if (addHeader) 
   {
-    PTRACE(6, "x264", "Adding a NAL unit of " << dataLen << " bytes to buffer (type " << (int)(header & 0x1f) << ")"); 
+    PTRACE(6, FrameTraceName, "Adding a NAL unit of " << dataLen << " bytes to buffer (type " << (int)(header & 0x1f) << ")"); 
     uint8_t* NALptr = data;
     if (((header & 0x1f) == H264_NAL_TYPE_SEQ_PARAM) && (dataLen >= 3)) 
     {
-      PTRACE(4, "x264", "Profile: " << (int)NALptr[0] << 
+      PTRACE(4, FrameTraceName, "Profile: " << (int)NALptr[0] << 
                                 " Level: "   << (int)NALptr[2] << 
 			        " Constraints: " << (NALptr[1] & 0x80 ? 1 : 0) 
 			                         << (NALptr[1] & 0x40 ? 1 : 0) 
@@ -449,10 +433,10 @@ void H264Frame::AddDataToEncodedFrame (uint8_t *data, uint32_t dataLen, uint8_t
     }
   }
   else
-    PTRACE(6, "x264", "Adding a NAL unit of " << dataLen << " bytes to buffer");
+    PTRACE(6, FrameTraceName, "Adding a NAL unit of " << dataLen << " bytes to buffer");
 
   if (m_encodedFrameLen + dataLen + headerLen > MAX_FRAME_SIZE) {
-    PTRACE(2, "x264", "Frame too big (" << m_encodedFrameLen + dataLen + headerLen << ">" << MAX_FRAME_SIZE << ")");
+    PTRACE(2, FrameTraceName, "Frame too big (" << m_encodedFrameLen + dataLen + headerLen << ">" << MAX_FRAME_SIZE << ")");
     return;
   }
 
@@ -466,7 +450,7 @@ void H264Frame::AddDataToEncodedFrame (uint8_t *data, uint32_t dataLen, uint8_t
 
     if (m_numberOfNALsInFrame + 1 >(m_numberOfNALsReserved))
     {
-      m_NALs = (h264_nal_t *)realloc(m_NALs, (m_numberOfNALsReserved + 1) * sizeof(h264_nal_t));
+      m_NALs = (NALU *)realloc(m_NALs, (m_numberOfNALsReserved + 1) * sizeof(NALU));
       m_numberOfNALsReserved++;
     }
     if (m_NALs)
@@ -487,7 +471,7 @@ void H264Frame::AddDataToEncodedFrame (uint8_t *data, uint32_t dataLen, uint8_t
       m_NALs[m_numberOfNALsInFrame - 1].length += dataLen;
   }
 
-  PTRACE(6, "x264", "Reserved memory for  " << m_numberOfNALsReserved <<" NALs, Inframe/current: "<< m_numberOfNALsInFrame <<" Offset: "
+  PTRACE(6, FrameTraceName, "Reserved memory for  " << m_numberOfNALsReserved <<" NALs, Inframe/current: "<< m_numberOfNALsInFrame <<" Offset: "
     << m_NALs[m_numberOfNALsInFrame-1].offset << " Length: "<< m_NALs[m_numberOfNALsInFrame-1].length << " Type: "<< (int)(m_NALs[m_numberOfNALsInFrame-1].type));
 
   memcpy(currentPositionInFrame, data, dataLen);
