@@ -21,13 +21,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 24766 $
+ * $Author: rjongbloed $
+ * $Date: 2010-09-30 02:18:52 -0500 (Thu, 30 Sep 2010) $
  */
 
 #include <ptlib.h>
-#include <opal/manager.h>
+#include <opal/console_mgr.h>
 #include <opal/ivr.h>
 #include <sip/sipep.h>
 #include <h323/h323ep.h>
