@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22771 $
+ * $Revision: 24813 $
  * $Author: rjongbloed $
- * $Date: 2009-05-29 11:09:14 +0000 (Fri, 29 May 2009) $
+ * $Date: 2010-10-21 03:18:03 -0500 (Thu, 21 Oct 2010) $
  */
 
 #include "precompile.h"
@@ -120,6 +120,7 @@ PBoolean OpalGw::OnStart()
 
 void OpalGw::OnStop()
 {
+  manager.ShutDownEndpoints();
   PHTTPServiceProcess::OnStop();
 }
 
@@ -157,10 +158,7 @@ PBoolean OpalGw::Initialise(const char * initMsg)
   // Sert log level as early as possible
   SetLogLevel((PSystemLog::Level)cfg.GetInteger(LogLevelKey, GetLogLevel()));
 #if PTRACING
-  if (GetLogLevel() >= PSystemLog::Warning)
-    PTrace::SetLevel(GetLogLevel()-PSystemLog::Warning);
-  else
-    PTrace::SetLevel(0);
+  PTrace::SetLevel(GetLogLevel());
   PTrace::ClearOptions(PTrace::Timestamp);
   PTrace::SetOptions(PTrace::DateAndTime);
 #endif
@@ -169,8 +167,8 @@ PBoolean OpalGw::Initialise(const char * initMsg)
   PString username = cfg.GetString(UsernameKey);
   PString password = PHTTPPasswordField::Decrypt(cfg.GetString(PasswordKey));
 
-#if P_HAS_IPV6
   PString addressFamily = cfg.GetString(DefaultAddressFamilyKey, "IPV4");
+#if P_HAS_IPV6
   if(addressFamily *= "IPV6")
 	 PIPSocket::SetDefaultIpAddressFamilyV6();
 #endif
@@ -218,7 +216,7 @@ PBoolean OpalGw::Initialise(const char * initMsg)
 
 #if OPAL_H323
   // Create the status page
-  httpNameSpace.AddResource(new MainStatusPage(*this, authority), PHTTPSpace::Overwrite);
+  httpNameSpace.AddResource(new MainStatusPage(*this, *manager.gkServer, authority), PHTTPSpace::Overwrite);
 #endif // OPAL_H323
 
 
@@ -236,7 +234,7 @@ PBoolean OpalGw::Initialise(const char * initMsg)
          << PHTML::HotLink("Parameters") << "Parameters" << PHTML::HotLink()
          << PHTML::Paragraph();
 
-    if (!systemLogFileName && systemLogFileName != "-")
+    if (PIsDescendant(&PSystemLog::GetTarget(), PSystemLogToFile))
       html << PHTML::HotLink("logfile.txt") << "Full Log File" << PHTML::HotLink()
            << PHTML::BreakLine()
            << PHTML::HotLink("tail_logfile") << "Tail Log File" << PHTML::HotLink()
@@ -294,12 +292,6 @@ MyManager::MyManager()
 
 MyManager::~MyManager()
 {
-#if OPAL_LID
-  // Must do this before we destroy the manager or a crash will result
-  if (potsEP != NULL)
-    potsEP->RemoveAllLines();
-#endif
-
 #if OPAL_H323
   delete gkServer;
 #endif
@@ -366,8 +358,8 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
   rsrc->Add(new PHTTPIntegerField(RTPPortBaseKey, 0, 65535, GetRtpIpPortBase()));
   rsrc->Add(new PHTTPIntegerField(RTPPortMaxKey,  0, 65535, GetRtpIpPortMax()));
 
-  SetRtpIpTypeofService(cfg.GetInteger(RTPTOSKey, GetRtpIpTypeofService()));
-  rsrc->Add(new PHTTPIntegerField(RTPTOSKey,  0, 255, GetRtpIpTypeofService()));
+  SetMediaTypeOfService(cfg.GetInteger(RTPTOSKey, GetMediaTypeOfService()));
+  rsrc->Add(new PHTTPIntegerField(RTPTOSKey,  0, 255, GetMediaTypeOfService()));
 
   PString STUNServer = cfg.GetString(STUNServerKey, "stun.voxgratia.org");
   rsrc->Add(new PHTTPStringField(STUNServerKey, 25, STUNServer));
@@ -452,8 +444,9 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
   sipEP->SetProxy(cfg.GetString(SIPProxyKey, proxy));
   rsrc->Add(new PHTTPStringField(SIPProxyKey, 25, proxy));
 
-  PString registrar = cfg.GetString(SIPRegistrarKey);
-  rsrc->Add(new PHTTPStringField(SIPRegistrarKey, 25, registrar));
+  SIPRegister::Params registrar;
+  registrar.m_registrarAddress = cfg.GetString(SIPRegistrarKey);
+  rsrc->Add(new PHTTPStringField(SIPRegistrarKey, 25, registrar.m_registrarAddress));
 
   fieldArray = new PHTTPFieldArray(new PHTTPStringField(SIPListenersKey, 25), PFalse);
   if (!sipEP->StartListeners(fieldArray->GetStrings(cfg))) {
@@ -461,9 +454,10 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
   }
   rsrc->Add(fieldArray);
 
-  if (!registrar) {
-    if (sipEP->Register(registrar)) {
-      PSYSTEMLOG(Info, "Registered with registrar " << registrar);
+  if (!registrar.m_registrarAddress) {
+    PString aor;
+    if (sipEP->Register(registrar, aor)) {
+      PSYSTEMLOG(Info, "Registered " << aor);
     }
     else {
       PSYSTEMLOG(Error, "Could not register with registrar!");
