@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24363 $
+ * $Revision: 26925 $
  * $Author: rjongbloed $
- * $Date: 2010-05-21 00:57:20 -0500 (Fri, 21 May 2010) $
+ * $Date: 2012-02-01 01:36:38 -0600 (Wed, 01 Feb 2012) $
  */
 
 #ifndef _OpenPhone_MAIN_H
@@ -61,7 +61,9 @@
 class MyManager;
 
 class OpalLineEndPoint;
+class OpalCapiEndPoint;
 class OpalIVREndPoint;
+class OpalMixerEndPoint;
 class OpalFaxEndPoint;
 
 class wxSplitterWindow;
@@ -71,7 +73,9 @@ class wxListCtrl;
 class wxListEvent;
 class wxNotebook;
 class wxGrid;
+class wxGridEvent;
 class wxConfigBase;
+class wxImageList;
 
 
 class MyPCSSEndPoint : public OpalPCSSEndPoint
@@ -86,11 +90,13 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
 
     MyManager & m_manager;
 
+/*
     bool TransmitExternalIM(
       OpalConnection & conn, 
       const OpalMediaFormat & format, 
       RTP_IMFrame & frame
     );
+*/
 
   private:
     virtual PBoolean OnShowIncoming(const OpalPCSSConnection & connection);
@@ -99,19 +105,6 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
 
 class IMDialog;
 
-struct ReceivedMessageInfo {
-  ReceivedMessageInfo(const std::string & connId_, const OpalMediaFormat & fmt_)
-    : m_connId(connId_), m_mediaFormat(fmt_) { }
-
-  std::string m_connId;
-  OpalMediaFormat m_mediaFormat;
-  PURL    m_localURL;
-  PURL    m_remoteURL;
-  PString m_remoteName;
-  PString m_contentType;
-  PString m_body;
-};
-
 struct PresenceInfo {
   PString entity;
   PString status;
@@ -139,22 +132,14 @@ class MySIPEndPoint : public SIPEndPoint
 
   private:
     virtual void OnRegistrationStatus(
-      const PString & aor,
-      PBoolean wasRegistering,
-      PBoolean reRegistering,
-      SIP_PDU::StatusCodes reason
+      const RegistrationStatus & status
     );
     virtual void OnSubscriptionStatus(
-      const PString & eventPackage,
-      const SIPURL & uri,
-      bool wasSubscribing,
-      bool reSubscribing,
-      SIP_PDU::StatusCodes reason
+      const SubscriptionStatus & status
     );
     virtual void OnDialogInfoReceived(
       const SIPDialogNotification & info  ///< Information on dialog state change
     );
-    virtual void OnPresenceInfoReceived(const SIPPresenceInfo & info);
 
     MyManager & m_manager;
 };
@@ -176,14 +161,14 @@ class MyFaxEndPoint : public OpalFaxEndPoint
 class PresenceDialog : public wxDialog
 {
   public:
-    PresenceDialog(MyManager * manager, SIPEndPoint & sipEP);
+    PresenceDialog(MyManager * manager);
 
   private:
     bool TransferDataFromWindow();
 
-    SIPEndPoint & m_sipEP;
-    PwxString     m_address;
-    PwxString     m_status;
+    MyManager & m_manager;
+    PwxString   m_address;
+    PwxString   m_status;
 
     DECLARE_EVENT_TABLE()
 };
@@ -214,6 +199,7 @@ class VideoControlDialog : public wxDialog
 
     MyManager & m_manager;
     wxSlider  * m_TargetBitRate;
+    wxSlider  * m_FrameRate;
 
     DECLARE_EVENT_TABLE()
 };
@@ -257,16 +243,15 @@ class CallIMDialog : public wxDialog
   public:
     CallIMDialog(MyManager * manager);
 
+    PwxString m_Presentity;
     PwxString m_Address;
 
   private:
     void OnOK(int code);
-    void OnPage(wxCommandEvent & event);
-    void OnSession(wxCommandEvent & event);
-    void OnAddressChange(wxCommandEvent & event);
+    void OnChanged(wxCommandEvent & event);
 
-    wxComboBox * m_AddressCtrl;
     wxButton   * m_ok;
+    wxComboBox * m_AddressCtrl;
 
     DECLARE_EVENT_TABLE()
 };
@@ -275,37 +260,41 @@ class IMDialog : public wxDialog
 {
   public:
     IMDialog(MyManager * manager, 
-     const std::string & callId,
- const OpalMediaFormat & m_format,
+         const PString & conversationId,
+ const OpalMediaFormat & format,
          const PString & localName, 
             const PURL & remoteAddress, 
          const PString & remoteName);
     ~IMDialog();
 
     void AddTextToScreen(const wxString & text, bool fromUs);
+    void SetCompositionIndication(bool idle);
 
-    std::string m_connId;
+    PString m_conversationId;
 
   private:
     void OnSend(wxCommandEvent & event);
     void OnTextEnter(wxCommandEvent & event);
+    void OnText(wxCommandEvent & event);
     void OnCloseWindow(wxCloseEvent &event);
 
     void SendCurrentText();
+    void SendCompositionIndicationChanged();
 
+    OpalMediaFormat m_mediaFormat;
     MyManager * m_manager;
     PURL m_remoteAddress;
 
     wxString m_them;
     PString m_us;
     PString m_remoteAddrStr;
-    OpalMediaFormat m_mediaFormat;
 
     wxTextCtrl * m_enteredText;
     wxTextCtrl * m_textArea;
     wxTextAttr   m_defaultStyle;
     wxTextAttr   m_ourStyle;
     wxTextAttr   m_theirStyle;
+    wxStaticText * m_compositionIndication;
 
     DECLARE_EVENT_TABLE()
 };
@@ -377,7 +366,7 @@ struct StatisticsField
   virtual ~StatisticsField() { }
   void Init(wxWindow * panel);
   void Clear();
-  double CalculateBandwidth(DWORD bytes);
+  double CalculateBandwidth(PUInt64 bytes);
   double CalculateFrameRate(DWORD frames);
   virtual StatisticsField * Clone() const = 0;
   virtual void Update(const OpalConnection & connection, const OpalMediaStream & stream, const OpalMediaStatistics & statistics);
@@ -389,7 +378,7 @@ struct StatisticsField
   wxString        m_printFormat;
 
   PTimeInterval   m_lastBandwidthTick;
-  DWORD           m_lastBytes;
+  PUInt64         m_lastBytes;
 
   PTimeInterval   m_lastFrameTick;
   DWORD           m_lastFrames;
@@ -436,6 +425,7 @@ class InCallPanel : public CallPanelBase
   private:
     void OnHangUp(wxCommandEvent & event);
     void OnHoldRetrieve(wxCommandEvent & event);
+    void OnConference(wxCommandEvent & event);
     void OnSpeakerMute(wxCommandEvent & event);
     void OnMicrophoneMute(wxCommandEvent & event);
     void OnUserInput1(wxCommandEvent & event);
@@ -458,18 +448,19 @@ class InCallPanel : public CallPanelBase
     void SetVolume(bool microphone, int value, bool muted);
     void UpdateStatistics();
 
-    wxButton  * m_Hold;
-    wxButton  * m_SpeakerHandset;
-    wxCheckBox* m_SpeakerMute;
-    wxCheckBox* m_MicrophoneMute;
-    wxSlider  * m_SpeakerVolume;
-    wxSlider  * m_MicrophoneVolume;
-    wxGauge   * m_vuSpeaker;
-    wxGauge   * m_vuMicrophone;
-    wxTimer     m_vuTimer;
-    unsigned    m_updateStatistics;
-    bool        m_FirstTime;
-    bool        m_SwitchingHold;
+    wxButton     * m_Hold;
+    wxButton     * m_Conference;
+    wxButton     * m_SpeakerHandset;
+    wxCheckBox   * m_SpeakerMute;
+    wxCheckBox   * m_MicrophoneMute;
+    wxSlider     * m_SpeakerVolume;
+    wxSlider     * m_MicrophoneVolume;
+    wxGauge      * m_vuSpeaker;
+    wxGauge      * m_vuMicrophone;
+    wxTimer        m_vuTimer;
+    wxStaticText * m_callTime;
+    unsigned       m_updateStatistics;
+    bool           m_FirstTime;
 
     StatisticsPage m_pages[NumPages];
 
@@ -477,16 +468,22 @@ class InCallPanel : public CallPanelBase
 };
 
 
-class SpeedDialDialog : public wxDialog
+struct SpeedDialInfo
+{
+  PwxString m_Name;
+  PwxString m_Number;
+  PwxString m_Address;
+  PwxString m_Presentity;
+  PwxString m_Description;
+
+  bool operator<(const SpeedDialInfo & info) const { return m_Name < info.m_Name; }
+};
+
+
+class SpeedDialDialog : public wxDialog, public SpeedDialInfo
 {
   public:
-    SpeedDialDialog(MyManager *parent);
-
-    wxString m_Name;
-    wxString m_Number;
-    wxString m_Address;
-    wxString m_StateURL;
-    wxString m_Description;
+    SpeedDialDialog(MyManager *parent, const SpeedDialInfo & info);
 
   private:
     void OnChange(wxCommandEvent & event);
@@ -513,9 +510,7 @@ class RegistrationInfo
       Register,
       SubscribeMWI,
       SubcribePresence,
-      SubscribeMLA,
-      PublishPresence,
-      WatchPresence
+      SubscribeMLA
     };
 
     RegistrationInfo();
@@ -634,14 +629,19 @@ class OptionsDialog : public wxDialog
     int       m_SilenceDeadband;
     bool      m_DisableDetectInBandDTMF;
 
+    wxComboBox * m_soundPlayerCombo;
+    wxComboBox * m_soundRecorderCombo;
     wxComboBox * m_selectedLID;
     wxChoice   * m_selectedAEC;
     wxComboBox * m_selectedCountry;
+
+    void ChangedSoundPlayer(wxCommandEvent & event);
+    void ChangedSoundRecorder(wxCommandEvent & event);
     void SelectedLID(wxCommandEvent & event);
 
     ////////////////////////////////////////
     // Video fields
-    PwxString m_VideoGrabber;
+    PwxString m_VideoGrabDevice;
     int       m_VideoGrabFormat;
     int       m_VideoGrabSource;
     int       m_VideoGrabFrameRate;
@@ -653,12 +653,14 @@ class OptionsDialog : public wxDialog
     bool      m_VideoFlipRemote;
     PwxString m_VideoMinFrameSize;
     PwxString m_VideoMaxFrameSize;
+    int       m_VideoGrabBitRate;
+    int       m_VideoMaxBitRate;
 
-    wxComboBox * m_videoGrabDevice;
+    wxComboBox * m_videoGrabDeviceCombo;
     wxChoice   * m_videoSourceChoice;
 
     void AdjustVideoControls(const PwxString & device);
-    void ChangeVideoGrabber(wxCommandEvent & event);
+    void ChangeVideoGrabDevice(wxCommandEvent & event);
     void TestVideoCapture(wxCommandEvent & event);
 
     ////////////////////////////////////////
@@ -678,6 +680,20 @@ class OptionsDialog : public wxDialog
     int       m_VideoRecordingMode;
     PwxString m_VideoRecordingSize;
 
+    ////////////////////////////////////////
+    // Presence fields
+    wxListCtrl * m_Presentities;
+    wxGrid     * m_PresentityAttributes;
+    wxButton   * m_AddPresentity;
+    wxButton   * m_RemovePresentity;
+    void AddPresentity(wxCommandEvent & event);
+    void RemovePresentity(wxCommandEvent & event);
+    void SelectedPresentity(wxListEvent & event);
+    void DeselectedPresentity(wxListEvent & event);
+    void EditedPresentity(wxListEvent & event);
+    bool FillPresentityAttributes(OpalPresentity * presentity);
+    void ChangedPresentityAttribute(wxGridEvent & event);
+
     ////////////////////////////////////////
     // Codec fields
     wxButton     * m_AddCodec;
@@ -740,6 +756,8 @@ class OptionsDialog : public wxDialog
     PwxString m_SIPProxyUsername;
     PwxString m_SIPProxyPassword;
     int       m_LineAppearanceCode;
+    int       m_SIPUserInputMode;
+    int       m_SIPPRACKMode;
 
     wxListCtrl * m_Registrations;
     int          m_SelectedRegistration;
@@ -763,6 +781,7 @@ class OptionsDialog : public wxDialog
     // Routing fields
     wxString     m_ForwardingAddress;
     int          m_ForwardingTimeout;
+    PwxString    m_telURI;
     wxListCtrl * m_Routes;
     int          m_SelectedRoute;
     wxComboBox * m_RouteSource;
@@ -816,7 +835,6 @@ public:
   OpalMediaFormat mediaFormat;
   const wxChar  * validProtocols;
   int             preferenceOrder;
-  bool            dirty;
 };
 
 typedef std::list<MyMedia> MyMediaList;
@@ -879,6 +897,15 @@ class MyManager : public wxFrame, public OpalManager
       bool fromRemote,               ///<  Indicates remote has held local connection
       bool onHold                    ///<  Indicates have just been held/retrieved.
     );
+    virtual bool OnTransferNotify(
+      OpalConnection & connection,  ///< Connection being transferred.
+      const PStringToString & info  ///< Information on the transfer
+    );
+    virtual void AdjustMediaFormats(
+      bool local,                         ///<  Media formats a local ones to be presented to remote
+      const OpalConnection & connection,  ///<  Connection that is about to use formats
+      OpalMediaFormatList & mediaFormats  ///<  Media formats to use
+    ) const;
     virtual PBoolean OnOpenMediaStream(
       OpalConnection & connection,  /// Connection that owns the media stream
       OpalMediaStream & stream    /// New media stream being opened
@@ -922,6 +949,7 @@ class MyManager : public wxFrame, public OpalManager
     void OnStateChange(wxCommandEvent & event);
     void OnStreamsChanged(wxCommandEvent &);
     void OnRxMessage(wxCommandEvent &);
+    void OnRxCompositionIndicationChanged(wxCommandEvent &);
     void OnPresence(wxCommandEvent &);
 
     void OnMenuQuit(wxCommandEvent& event);
@@ -950,6 +978,7 @@ class MyManager : public wxFrame, public OpalManager
     void OnOptions(wxCommandEvent& event);
     void OnRequestHold(wxCommandEvent& event);
     void OnRetrieve(wxCommandEvent& event);
+    void OnConference(wxCommandEvent& event);
     void OnTransfer(wxCommandEvent& event);
     void OnStartRecording(wxCommandEvent& event);
     void OnStopRecording(wxCommandEvent& event);
@@ -965,7 +994,8 @@ class MyManager : public wxFrame, public OpalManager
     void OnSashPositioned(wxSplitterEvent& event);
     void OnSpeedDialActivated(wxListEvent& event);
     void OnSpeedDialColumnResize(wxListEvent& event);
-    void OnRightClick(wxListEvent& event);
+    void OnSpeedDialRightClick(wxListEvent& event);
+    void OnSpeedDialEndEdit(wxListEvent& event);
 
     void OnMyPresence(wxCommandEvent& event);
     void OnStartInstantMessage(wxCommandEvent& event);
@@ -978,6 +1008,13 @@ class MyManager : public wxFrame, public OpalManager
     bool CanDoFax() const;
     bool CanDoIM() const;
 
+    IMDialog * GetOrCreateIMDialog(const PString & conversationId, const PString & local, const PString & remote);
+    void RemoveIMDialog(IMDialog * dialog);
+
+    PDECLARE_NewConversationNotifier(MyManager, OnNewConversation);
+    PDECLARE_IncomingIMNotifier(MyManager, OnIncomingIM);
+    PDECLARE_CompositionIndicationChangedNotifier(MyManager, OnCompositionIndicationChanged);
+
     enum SpeedDialViews {
       e_ViewLarge,
       e_ViewSmall,
@@ -990,8 +1027,12 @@ class MyManager : public wxFrame, public OpalManager
       SpeedDialViews view
     );
     void EditSpeedDial(
+      int index
+    );
+    bool UpdateSpeedDial(
       int index,
-      bool newItem
+      const SpeedDialInfo & info,
+      bool saveConfig
     );
 
     enum {
@@ -999,7 +1040,7 @@ class MyManager : public wxFrame, public OpalManager
       e_StatusColumn,
       e_NumberColumn,
       e_AddressColumn,
-      e_StateUrlColumn,
+      e_PresentityColumn,
       e_DescriptionColumn,
       e_NumColumns
     };
@@ -1015,10 +1056,14 @@ class MyManager : public wxFrame, public OpalManager
     wxNotebook       * m_tabs;
     wxTextCtrl       * m_logWindow;
     wxListCtrl       * m_speedDials;
+    bool               m_speedDialDetail;
     wxImageList      * m_imageListNormal;
     wxImageList      * m_imageListSmall;
     wxDataFormat       m_ClipboardFormat;
 
+    set<SpeedDialInfo> m_speedDialInfo;
+    SpeedDialInfo * GetSelectedSpeedDial() const;
+
     MyPCSSEndPoint   * pcssEP;
     OpalLineEndPoint * potsEP;
     void StartLID();
@@ -1048,21 +1093,27 @@ class MyManager : public wxFrame, public OpalManager
     void StartRegistrations();
     void ReplaceRegistrations(const RegistrationList & newRegistrations);
 
-    typedef std::map<std::string, IMDialog *> ConversationMapType;
-    PMutex conversationMapMutex;
+    // cannot use PDictionary as IMDialog is not descended from PObject
+    typedef std::map<std::string, IMDialog *> IMDialogMap;
+    IMDialogMap m_imDialogMap;
+    PMutex m_imDialogMapMutex;
 
-    IMDialog * GetOrCreateConversation(const ReceivedMessageInfo & messageInfo);
+    bool MonitorPresence(const PString & presentity, const PString & uri, bool start);
+    PDECLARE_PresenceChangeNotifier(MyManager, OnPresenceChange);
+#endif
 
-    ConversationMapType conversationMap;
-
-    bool SubscribePresence(wxString & uri);
-    void OnPresenceInfoReceived(const SIPPresenceInfo & info);
+#if OPAL_CAPI
+    OpalCapiEndPoint  * capiEP;
 #endif
 
 #if OPAL_IVR
     OpalIVREndPoint  * ivrEP;
 #endif
 
+#if OPAL_HAS_MIXER
+    OpalMixerEndPoint * m_mcuEP;
+#endif
+
 #if OPAL_FAX
     OpalFaxEndPoint  * m_faxEP;
 #endif
@@ -1089,6 +1140,8 @@ class MyManager : public wxFrame, public OpalManager
     PwxString m_VideoGrabFrameSize;
     PwxString m_VideoMinFrameSize;
     PwxString m_VideoMaxFrameSize;
+    int       m_VideoMaxBitRate;
+    int       m_VideoTargetBitRate;
     bool AdjustVideoFormats();
 
     enum ExtendedVideoRoles {
@@ -1115,7 +1168,7 @@ class MyManager : public wxFrame, public OpalManager
     void StopRingSound();
     void UpdateAudioDevices();
 
-    PString            m_incomingToken;
+    PwxString          m_incomingToken;
     PSafePtr<OpalCall> m_activeCall;
 
     void AddCallOnHold(
@@ -1128,6 +1181,7 @@ class MyManager : public wxFrame, public OpalManager
       const PString & token,
       bool onHold
     );
+    void AddToConference(OpalCall & call);
 
     struct CallsOnHold {
       CallsOnHold() { }
@@ -1135,9 +1189,11 @@ class MyManager : public wxFrame, public OpalManager
 
       PSafePtr<OpalCall> m_call;
       int                m_retrieveMenuId;
+      int                m_conferenceMenuId;
       int                m_transferMenuId;
     };
     list<CallsOnHold>    m_callsOnHold;
+    PwxString            m_switchHoldToken;
 
     OpalRecordManager::Options m_recordingOptions;
     PwxString                  m_lastRecordFile;
