@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24813 $
+ * $Revision: 26805 $
  * $Author: rjongbloed $
- * $Date: 2010-10-21 03:18:03 -0500 (Thu, 21 Oct 2010) $
+ * $Date: 2011-12-17 01:27:03 -0600 (Sat, 17 Dec 2011) $
  */
 
 #include <ptlib.h>
@@ -159,6 +159,8 @@ void SimpleOpalProcess::Main()
              "-grabchannel:"
              "-display:"
              "-displaydriver:"
+             "-preview:"
+             "-previewdriver:"
              "-video-size:"
              "-video-rate:"
              "-video-bitrate:"
@@ -210,8 +212,10 @@ void SimpleOpalProcess::Main()
             "     --grabber dev        : Set the video grabber device.\n"
             "     --grabdriver dev     : Set the video grabber driver (if device name is ambiguous).\n"
             "     --grabchannel num    : Set the video grabber device channel.\n"
-            "     --display dev        : Set the video display device.\n"
-            "     --displaydriver dev  : Set the video display driver (if device name is ambiguous).\n"
+            "     --display dev        : Set the remote video display device.\n"
+            "     --displaydriver dev  : Set the remote video display driver (if device name is ambiguous).\n"
+            "     --preview dev        : Set the local video preview device.\n"
+            "     --previewdriver dev  : Set the local video preview driver (if device name is ambiguous).\n"
             "     --video-size size    : Set the size of the video for all video formats, use\n"
             "                          : \"qcif\", \"cif\", WxH etc\n"
             "     --video-rate rate    : Set the frame rate of video for all video formats\n"
@@ -436,6 +440,7 @@ PBoolean MyManager::Initialise(PArgList & args)
   if (args.HasOption("no-tx-video"))
     SetAutoStartTransmitVideo(false);
 
+  // video input options
   if (args.HasOption("grabber")) {
     PVideoDevice::OpenArgs video = GetVideoInputDevice();
     video.deviceName = args.GetOptionString("grabber");
@@ -450,6 +455,7 @@ PBoolean MyManager::Initialise(PArgList & args)
     }
   }
 
+  // remote video display options
   if (args.HasOption("display")) {
     PVideoDevice::OpenArgs video = GetVideoOutputDevice();
     video.deviceName = args.GetOptionString("display");
@@ -460,6 +466,18 @@ PBoolean MyManager::Initialise(PArgList & args)
       return PFalse;
     }
   }
+
+  // local video preview options
+  if (args.HasOption("preview")) {
+    PVideoDevice::OpenArgs video = GetVideoPreviewDevice();
+    video.deviceName = args.GetOptionString("preview");
+    video.driverName = args.GetOptionString("previewdriver");
+	if (!SetVideoPreviewDevice(video)) {
+      cerr << "Unknown preview device " << video.deviceName << "\n"
+              "Available devices are:" << setfill(',') << PVideoOutputDevice::GetDriversDeviceNames("") << endl;
+      return PFalse;
+    }
+  }
 #endif
 
   if (args.HasOption('j')) {
@@ -599,6 +617,7 @@ PBoolean MyManager::Initialise(PArgList & args)
     cout << "Sound output device: \"" << pcssEP->GetSoundChannelPlayDevice() << "\"\n"
             "Sound  input device: \"" << pcssEP->GetSoundChannelRecordDevice() << "\"\n"
 #if OPAL_VIDEO
+			"Video preview device: \"" << GetVideoPreviewDevice().deviceName << "\"\n"
             "Video output device: \"" << GetVideoOutputDevice().deviceName << "\"\n"
             "Video  input device: \"" << GetVideoInputDevice().deviceName << '"'
 #endif
