@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21857 $
+ * $Revision: 24299 $
  * $Author: rjongbloed $
- * $Date: 2008-12-22 04:03:10 +0000 (Mon, 22 Dec 2008) $
+ * $Date: 2010-04-28 08:52:07 -0500 (Wed, 28 Apr 2010) $
  */
 
 #ifndef _SimpleOpal_MAIN_H
@@ -45,6 +45,8 @@ class SIPEndPoint;
 class H323EndPoint;
 class H323SEndPoint;
 class IAX2EndPoint;
+class OpalCapiEndPoint;
+
 
 class MyPCSSEndPoint : public OpalPCSSEndPoint
 {
@@ -108,6 +110,9 @@ class MyManager : public OpalManager
 #if OPAL_IAX2
     IAX2EndPoint     * iax2EP;
 #endif
+#if OPAL_CAPI
+    OpalCapiEndPoint * capiEP;
+#endif
 #if OPAL_IVR
     OpalIVREndPoint  * ivrEP;
 #endif
