@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23557 $
- * $Author: csoutheren $
- * $Date: 2009-09-30 02:34:00 -0500 (Wed, 30 Sep 2009) $
+ * $Revision: 26521 $
+ * $Author: rjongbloed $
+ * $Date: 2011-10-04 23:57:06 -0500 (Tue, 04 Oct 2011) $
  */
 
 #include <ptlib.h>
@@ -385,7 +385,6 @@ class PWAVFilePluginFactory : public PObject, public Factory::WorkerBase
       , m_mediaFormat(info.m_name)
       , m_extendedHeader(info.m_extendedHeaderData, info.m_extendedHeaderSize)
     {
-      Factory::WorkerBase::isDynamic = true;
       Factory::Register(key, this);
     }
 
