@@ -22,9 +22,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26234 $
+ * $Revision: 27023 $
  * $Author: rjongbloed $
- * $Date: 2011-07-20 23:07:12 -0500 (Wed, 20 Jul 2011) $
+ * $Date: 2012-02-21 16:43:42 -0600 (Tue, 21 Feb 2012) $
  */
 
 #include <ptlib.h>
@@ -145,10 +145,12 @@ static void AddEventsOption(OpalMediaFormat & mediaFormat,
 {
   OpalRFC288EventsOption * option = new OpalRFC288EventsOption(OpalRFC288EventsName(),
                                                                false,
-                                                               OpalMediaOption::AndMerge,
+                                                               OpalMediaOption::IntersectionMerge,
                                                                OpalRFC2833EventsMask(defaultValues));
+#if OPAL_SIP
   option->SetFMTPName("FMTP");
   option->SetFMTPDefault(fmtpDefaults);
+#endif
   mediaFormat.AddOption(option);
 }
 
