@@ -6,8 +6,8 @@
 #include <math.h>
 #include <memory.h>
 
-#if !(defined(P_MACOSX) || defined(P_FREEBSD))
-#include <malloc.h>   // RMF
+#if !defined __STDC__
+#include <malloc.h>
 #endif
 
 #ifdef __cplusplus
