@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22197 $
+ * $Revision: 26728 $
  * $Author: rjongbloed $
- * $Date: 2009-03-11 20:11:14 -0500 (Wed, 11 Mar 2009) $
+ * $Date: 2011-12-01 22:59:14 -0600 (Thu, 01 Dec 2011) $
  */
 
 #include <ptlib.h>
@@ -145,7 +145,6 @@ H323Channel::H323Channel(H323Connection & conn, const H323Capability & cap)
 {
   capability = (H323Capability *)cap.Clone();
   bandwidthUsed = 0;
-  terminating = PFalse;
   opened = PFalse;
   paused = PTrue;
 }
@@ -171,6 +170,12 @@ unsigned H323Channel::GetSessionID() const
 }
 
 
+bool H323Channel::SetSessionID(unsigned /*sessionID*/)
+{
+  return true;
+}
+
+
 PBoolean H323Channel::GetMediaTransportAddress(OpalTransportAddress & /*data*/,
                                            OpalTransportAddress & /*control*/) const
 {
@@ -180,11 +185,13 @@ PBoolean H323Channel::GetMediaTransportAddress(OpalTransportAddress & /*data*/,
 
 void H323Channel::Close()
 {
-  if (!opened || terminating)
-    return;
+  if (opened && m_terminating++ == 0)
+    InternalClose();
+}
 
-  terminating = PTrue;
 
+void H323Channel::InternalClose()
+{
   // Signal to the connection that this channel is on the way out
   connection.OnClosedLogicalChannel(*this);
 
@@ -217,9 +224,19 @@ void H323Channel::OnSendOpenAck(const H245_OpenLogicalChannel & /*pdu*/,
 }
 
 
-void H323Channel::OnFlowControl(long PTRACE_PARAM(bitRateRestriction))
+void H323Channel::OnFlowControl(long bitRateRestriction)
 {
   PTRACE(3, "LogChan\tOnFlowControl: " << bitRateRestriction);
+
+  OpalMediaStreamPtr stream = GetMediaStream();
+  if (stream != NULL) {
+    long bitsPerSecond = bitRateRestriction*100;
+    OpalMediaFormat format = stream->GetMediaFormat();
+    format.SetOptionInteger(OpalMediaFormat::MaxBitRateOption(), bitsPerSecond);
+    if (format.GetOptionInteger(OpalMediaFormat::TargetBitRateOption()) > bitsPerSecond)
+      format.SetOptionInteger(OpalMediaFormat::TargetBitRateOption(), bitsPerSecond);
+    stream->UpdateMediaFormat(format);
+  }
 }
 
 
@@ -231,29 +248,46 @@ void H323Channel::OnMiscellaneousCommand(const H245_MiscellaneousCommand_type &
     return;
 
 #if OPAL_VIDEO
-  switch (type.GetTag()) {
-    case H245_MiscellaneousCommand_type::e_videoFastUpdatePicture :
+  switch (type.GetTag())
+  {
+    case H245_MiscellaneousCommand_type::e_videoFreezePicture :
+      mediaStream->ExecuteCommand(OpalVideoFreezePicture());
+      break;
+
+    case H245_MiscellaneousCommand_type::e_videoFastUpdatePicture:
       mediaStream->ExecuteCommand(OpalVideoUpdatePicture());
       break;
 
     case H245_MiscellaneousCommand_type::e_videoFastUpdateGOB :
-      {
-        const H245_MiscellaneousCommand_type_videoFastUpdateGOB & vfuGOB = type;
-        mediaStream->ExecuteCommand(OpalVideoUpdatePicture(vfuGOB.m_firstGOB, -1, vfuGOB.m_numberOfGOBs));
-      }
+    case H245_MiscellaneousCommand_type::e_videoFastUpdateMB :
+      mediaStream->ExecuteCommand(OpalVideoPictureLoss());
       break;
 
-    case H245_MiscellaneousCommand_type::e_videoFastUpdateMB :
-      {
-        const H245_MiscellaneousCommand_type_videoFastUpdateMB & vfuMB = type;
-        mediaStream->ExecuteCommand(OpalVideoUpdatePicture(vfuMB.m_firstGOB, vfuMB.m_firstMB, vfuMB.m_numberOfMBs));
-      }
+    case H245_MiscellaneousCommand_type::e_videoTemporalSpatialTradeOff :
+      mediaStream->ExecuteCommand(OpalTemporalSpatialTradeOff((const PASN_Integer &)type));
+      break;
+
+    default:
       break;
   }
 #endif
 }
 
 
+bool H323Channel::OnMediaCommand(const OpalMediaCommand & command)
+{
+#if OPAL_VIDEO
+  if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
+    H323ControlPDU pdu;
+    pdu.BuildMiscellaneousCommand(GetNumber(), H245_MiscellaneousCommand_type::e_videoFastUpdatePicture);
+    connection.WriteControlPDU(pdu);
+    return true;
+  }
+#endif
+  return false;
+}
+
+
 void H323Channel::OnMiscellaneousIndication(const H245_MiscellaneousIndication_type & PTRACE_PARAM(type))
 {
   PTRACE(3, "LogChan\tOnMiscellaneousIndication: chan=" << number
@@ -352,7 +386,7 @@ PBoolean H323UnidirectionalChannel::Open()
   }
 
   if (ok) {
-    capability->SetMediaFormatOptions(mediaStream->GetMediaFormat());
+    capability->UpdateMediaFormat(mediaStream->GetMediaFormat());
     return H323Channel::Open();
   }
 
@@ -375,11 +409,8 @@ PBoolean H323UnidirectionalChannel::Start()
 }
 
 
-void H323UnidirectionalChannel::Close()
+void H323UnidirectionalChannel::InternalClose()
 {
-  if (terminating)
-    return;
-
   PTRACE(4, "H323RTP\tCleaning up media stream on " << number);
 
   // If we have source media stream close it
@@ -389,87 +420,7 @@ void H323UnidirectionalChannel::Close()
     mediaStream.SetNULL();
   }
 
-  H323Channel::Close();
-}
-
-
-void H323UnidirectionalChannel::OnMiscellaneousCommand(const H245_MiscellaneousCommand_type & type)
-{
-  H323Channel::OnMiscellaneousCommand(type);
-
-  if (mediaStream == NULL)
-    return;
-
-#if OPAL_VIDEO
-  switch (type.GetTag())
-  {
-    case H245_MiscellaneousCommand_type::e_videoFreezePicture :
-      mediaStream->ExecuteCommand(OpalVideoFreezePicture());
-      break;
-
-    case H245_MiscellaneousCommand_type::e_videoFastUpdatePicture:
-      mediaStream->ExecuteCommand(OpalVideoUpdatePicture());
-      break;
-
-    case H245_MiscellaneousCommand_type::e_videoFastUpdateGOB :
-    {
-      const H245_MiscellaneousCommand_type_videoFastUpdateGOB & fuGOB = type;
-      mediaStream->ExecuteCommand(OpalVideoUpdatePicture(fuGOB.m_firstGOB, -1, fuGOB.m_numberOfGOBs));
-    }
-    break;
-
-    case H245_MiscellaneousCommand_type::e_videoFastUpdateMB :
-    {
-      const H245_MiscellaneousCommand_type_videoFastUpdateMB & vfuMB = type;
-      mediaStream->ExecuteCommand(OpalVideoUpdatePicture(vfuMB.HasOptionalField(H245_MiscellaneousCommand_type_videoFastUpdateMB::e_firstGOB) ? (int)vfuMB.m_firstGOB : -1,
-                                                         vfuMB.HasOptionalField(H245_MiscellaneousCommand_type_videoFastUpdateMB::e_firstMB)  ? (int)vfuMB.m_firstMB  : -1,
-                                                         vfuMB.m_numberOfMBs));
-    }
-    break;
-
-    case H245_MiscellaneousCommand_type::e_videoTemporalSpatialTradeOff :
-      mediaStream->ExecuteCommand(OpalTemporalSpatialTradeOff((const PASN_Integer &)type));
-      break;
-    default:
-      break;
-  }
-#endif
-}
-
-
-void H323UnidirectionalChannel::OnMediaCommand(OpalMediaCommand & command)
-{
-#if OPAL_VIDEO
-  if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
-    H323ControlPDU pdu;
-    const OpalVideoUpdatePicture & updatePicture = (const OpalVideoUpdatePicture &)command;
-
-    if (updatePicture.GetNumBlocks() <= 0)
-      pdu.BuildMiscellaneousCommand(GetNumber(), H245_MiscellaneousCommand_type::e_videoFastUpdatePicture);
-    else if (updatePicture.GetFirstMB() < 0) {
-      H245_MiscellaneousCommand_type_videoFastUpdateGOB & fuGOB = 
-            pdu.BuildMiscellaneousCommand(GetNumber(), H245_MiscellaneousCommand_type::e_videoFastUpdateGOB).m_type;
-      fuGOB.m_firstGOB = updatePicture.GetFirstGOB();
-      fuGOB.m_numberOfGOBs = updatePicture.GetNumBlocks();
-    }
-    else {
-      H245_MiscellaneousCommand_type_videoFastUpdateMB & vfuMB =
-            pdu.BuildMiscellaneousCommand(GetNumber(), H245_MiscellaneousCommand_type::e_videoFastUpdateMB).m_type;
-      if (updatePicture.GetFirstGOB() >= 0) {
-        vfuMB.IncludeOptionalField(H245_MiscellaneousCommand_type_videoFastUpdateMB::e_firstGOB);
-        vfuMB.m_firstGOB = updatePicture.GetFirstGOB();
-      }
-      if (updatePicture.GetFirstMB() >= 0) {
-        vfuMB.IncludeOptionalField(H245_MiscellaneousCommand_type_videoFastUpdateMB::e_firstMB);
-        vfuMB.m_firstMB = updatePicture.GetFirstMB();
-      }
-      vfuMB.m_numberOfMBs = updatePicture.GetNumBlocks();
-    }
-
-    connection.WriteControlPDU(pdu);
-    return;
-  }
-#endif
+  H323Channel::InternalClose();
 }
 
 
@@ -538,6 +489,17 @@ PBoolean H323_RealTimeChannel::OnSendingPDU(H245_OpenLogicalChannel & open) cons
 }
 
 
+PBoolean H323_RealTimeChannel::OnSendingPDU(H245_H2250LogicalChannelParameters & param) const
+{
+  // If we are master (or a standard ID) then we can set the session ID, otherwise zero
+  unsigned sessionID = GetSessionID();
+  if (connection.IsH245Master() || sessionID <= H323Capability::DefaultDataSessionID)
+    param.m_sessionID = sessionID;
+
+  return true;
+}
+
+
 void H323_RealTimeChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
                                          H245_OpenLogicalChannelAck & ack) const
 {
@@ -550,25 +512,19 @@ void H323_RealTimeChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
   ack.m_forwardMultiplexAckParameters.SetTag(
 	  H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters);
 
-  // get H225 parms
-  H245_H2250LogicalChannelAckParameters & param = ack.m_forwardMultiplexAckParameters;
+  OnSendOpenAck(ack.m_forwardMultiplexAckParameters);
+}
 
-  // set session ID
-  param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID);
-  const H245_H2250LogicalChannelParameters & openparam =
-                          open.m_forwardLogicalChannelParameters.m_multiplexParameters;
-  unsigned sessionID = openparam.m_sessionID;
-  
-  // Let the H.245 master assign the session ID if needed
-  if (connection.IsH245Master() && sessionID == 0) {
-    PTRACE(3, "H323\tOBTAINING EXTERNAL");
-    sessionID = connection.GetExternalSessionID(GetSessionID(), GetCapability().GetMediaFormat().GetMediaType());
+
+void H323_RealTimeChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & param) const
+{
+  // We are master, we use OUR session ID
+  if (connection.IsH245Master()) {
+    param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID);
+    param.m_sessionID = GetSessionID();
   }
-  param.m_sessionID = sessionID;
 
-  OnSendOpenAck(param);
-
-  PTRACE(3, "H323RTP\tSending open logical channel ACK: sessionID=" << sessionID);
+  PTRACE(3, "H323RTP\tSending open logical channel ACK: sessionID=" << GetSessionID());
 }
 
 
@@ -607,6 +563,30 @@ PBoolean H323_RealTimeChannel::OnReceivedPDU(const H245_OpenLogicalChannel & ope
 }
 
 
+PBoolean H323_RealTimeChannel::OnReceivedPDU(const H245_H2250LogicalChannelParameters & param,
+                                          unsigned & errorCode)
+{
+  unsigned sessionID = param.m_sessionID;
+
+  if (connection.IsH245Master()) {
+    if (sessionID == 0)
+      return true;
+  }
+  else {
+    if (sessionID != 0)
+      SetSessionID(sessionID);
+  }
+
+  if (sessionID != GetSessionID()) {
+    PTRACE(1, "H323RTP\tOpen of " << *this << " with invalid session: " << param.m_sessionID);
+    errorCode = H245_OpenLogicalChannelReject_cause::e_invalidSessionID;
+    return false;
+  }
+
+  return true;
+}
+
+
 PBoolean H323_RealTimeChannel::OnReceivedAckPDU(const H245_OpenLogicalChannelAck & ack)
 {
   PTRACE(3, "H323RTP\tOnReceiveOpenAck");
@@ -626,6 +606,27 @@ PBoolean H323_RealTimeChannel::OnReceivedAckPDU(const H245_OpenLogicalChannelAck
 }
 
 
+PBoolean H323_RealTimeChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAckParameters & param)
+{
+  unsigned sessionID = 0;
+  if (param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID))
+    sessionID = param.m_sessionID;
+
+  if (connection.IsH245Master() || sessionID <= H323Capability::DefaultDataSessionID) {
+    PTRACE_IF(2, sessionID != 0 && sessionID != GetSessionID(),
+              "LogChan\tAck contains invalid session ID " << param.m_sessionID << ", ignoring");
+    return true;
+  }
+
+  if (sessionID == 0) {
+    PTRACE(2, "LogChan\tNo session ID in ACK from master!");
+    return false;
+  }
+
+  return SetSessionID(sessionID);
+}
+
+
 PBoolean H323_RealTimeChannel::SetDynamicRTPPayloadType(int newType)
 {
   PTRACE(4, "H323RTP\tAttempting to set dynamic RTP payload type: " << newType);
@@ -675,9 +676,6 @@ H323_RTPChannel::H323_RTPChannel(H323Connection & conn,
 
 H323_RTPChannel::~H323_RTPChannel()
 {
-  // Finished with the RTP session, this will delete the session if it is no
-  // longer referenced by any logical channels.
-  connection.ReleaseSession(GetSessionID());
 }
 
 
@@ -687,30 +685,40 @@ unsigned H323_RTPChannel::GetSessionID() const
 }
 
 
+bool H323_RTPChannel::SetSessionID(unsigned sessionID)
+{
+  unsigned oldSessionID = GetSessionID();
+  if (sessionID == oldSessionID)
+    return true;
+
+  return connection.ChangeSessionID(oldSessionID, sessionID);
+}
+
+
 PBoolean H323_RTPChannel::OnSendingPDU(H245_H2250LogicalChannelParameters & param) const
 {
-  param.m_sessionID = connection.GetExternalSessionID(GetSessionID(), GetCapability().GetMediaFormat().GetMediaType());
-
-  return rtpCallbacks.OnSendingPDU(*this, param);
+  return rtpCallbacks.OnSendingPDU(*this, param) && H323_RealTimeChannel::OnSendingPDU(param);
 }
 
 
 void H323_RTPChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & param) const
 {
   rtpCallbacks.OnSendingAckPDU(*this, param);
+  H323_RealTimeChannel::OnSendOpenAck(param);
 }
 
 
 PBoolean H323_RTPChannel::OnReceivedPDU(const H245_H2250LogicalChannelParameters & param,
                                     unsigned & errorCode)
 {
-  return rtpCallbacks.OnReceivedPDU(*this, param, errorCode);
+  return rtpCallbacks.OnReceivedPDU(*this, param, errorCode) &&
+         H323_RealTimeChannel::OnReceivedPDU(param, errorCode);
 }
 
 
 PBoolean H323_RTPChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAckParameters & param)
 {
-  return rtpCallbacks.OnReceivedAckPDU(*this, param);
+  return rtpCallbacks.OnReceivedAckPDU(*this, param) && H323_RealTimeChannel::OnReceivedAckPDU(param);
 }
 
 
@@ -823,8 +831,6 @@ void H323_ExternalRTPChannel::Transmit()
 
 PBoolean H323_ExternalRTPChannel::OnSendingPDU(H245_H2250LogicalChannelParameters & param) const
 {
-  param.m_sessionID = connection.GetExternalSessionID(sessionID);
-
   param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaGuaranteedDelivery);
   param.m_mediaGuaranteedDelivery = PFalse;
 
@@ -853,7 +859,7 @@ PBoolean H323_ExternalRTPChannel::OnSendingPDU(H245_H2250LogicalChannelParameter
   if (H323SetRTPPacketization(param.m_mediaPacketization, GetCapability().GetMediaFormat(), GetDynamicRTPPayloadType()))
     param.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaPacketization);
 
-  return PTrue;
+  return H323_RealTimeChannel::OnSendingPDU(param);
 }
 
 
@@ -866,18 +872,16 @@ void H323_ExternalRTPChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameter
   // set mediaChannel
   param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);
   externalMediaAddress.SetPDU(param.m_mediaChannel);
+
+  H323_RealTimeChannel::OnSendOpenAck(param);
 }
 
 
 PBoolean H323_ExternalRTPChannel::OnReceivedPDU(const H245_H2250LogicalChannelParameters & param,
                                           unsigned & errorCode)
 {
-  // Only support a single audio session
-  if (connection.GetInternalSessionID(param.m_sessionID, GetCapability().GetMediaFormat().GetMediaType()) != sessionID) {
-    PTRACE(1, "LogChan\tOpen for invalid session: " << param.m_sessionID);
-    errorCode = H245_OpenLogicalChannelReject_cause::e_invalidSessionID;
-    return PFalse;
-  }
+  if (!H323_RealTimeChannel::OnReceivedPDU(param, errorCode))
+    return false;
 
   if (!param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel)) {
     PTRACE(1, "LogChan\tNo mediaControlChannel specified");
@@ -902,20 +906,18 @@ PBoolean H323_ExternalRTPChannel::OnReceivedPDU(const H245_H2250LogicalChannelPa
     remoteMediaAddress = OpalTransportAddress(addr, port-1);
   }
 
-  unsigned id = param.m_sessionID;
+  unsigned id = GetSessionID();
   if (!remoteMediaAddress.IsEmpty() && (connection.GetMediaTransportAddresses().GetAt(id) == NULL))
     connection.GetMediaTransportAddresses().SetAt(id, new OpalTransportAddress(remoteMediaAddress));
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean H323_ExternalRTPChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAckParameters & param)
 {
-   if (param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID) && (param.m_sessionID != sessionID)) {
-     PTRACE(2, "LogChan\tAck for invalid session: " << param.m_sessionID);
-  }
-
+  if (!H323_RealTimeChannel::OnReceivedAckPDU(param))
+    return false;
 
   if (!param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaControlChannel)) {
     PTRACE(1, "LogChan\tNo mediaControlChannel specified");
@@ -1003,11 +1005,8 @@ H323DataChannel::~H323DataChannel()
 }
 
 
-void H323DataChannel::Close()
+void H323DataChannel::InternalClose()
 {
-  if (terminating)
-    return;
-
   PTRACE(4, "LogChan\tCleaning up data channel " << number);
 
   // Break any I/O blocks and wait for the thread that uses this object to
@@ -1017,7 +1016,7 @@ void H323DataChannel::Close()
   if (transport != NULL)
     transport->Close();
 
-  H323UnidirectionalChannel::Close();
+  H323UnidirectionalChannel::InternalClose();
 }
 
 
@@ -1036,8 +1035,6 @@ PBoolean H323DataChannel::OnSendingPDU(H245_OpenLogicalChannel & open) const
   open.m_forwardLogicalChannelParameters.m_multiplexParameters.SetTag(
               H245_OpenLogicalChannel_forwardLogicalChannelParameters_multiplexParameters
                   ::e_h2250LogicalChannelParameters);
-  H245_H2250LogicalChannelParameters & fparam = open.m_forwardLogicalChannelParameters.m_multiplexParameters;
-  fparam.m_sessionID = connection.GetExternalSessionID(GetSessionID());
 
   if (separateReverseChannel)
     return PTrue;
@@ -1048,8 +1045,6 @@ PBoolean H323DataChannel::OnSendingPDU(H245_OpenLogicalChannel & open) const
   open.m_reverseLogicalChannelParameters.m_multiplexParameters.SetTag(
               H245_OpenLogicalChannel_reverseLogicalChannelParameters_multiplexParameters
                   ::e_h2250LogicalChannelParameters);
-  H245_H2250LogicalChannelParameters & rparam = open.m_reverseLogicalChannelParameters.m_multiplexParameters;
-  rparam.m_sessionID = connection.GetExternalSessionID(GetSessionID());
 
   return capability->OnSendingPDU(open.m_reverseLogicalChannelParameters.m_dataType);
 }
@@ -1065,8 +1060,6 @@ void H323DataChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
 
   PTRACE(3, "LogChan\tOnSendOpenAck for channel: " << number);
   
-  unsigned theSessionID;
-
   H245_H2250LogicalChannelAckParameters * param;
 
   if (separateReverseChannel) {
@@ -1074,9 +1067,6 @@ void H323DataChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
     ack.m_forwardMultiplexAckParameters.SetTag(
               H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters);
     param = (H245_H2250LogicalChannelAckParameters*)&ack.m_forwardMultiplexAckParameters.GetObject();
-    const H245_H2250LogicalChannelParameters & openparam =
-      open.m_forwardLogicalChannelParameters.m_multiplexParameters;
-    theSessionID = openparam.m_sessionID;
   }
   else {
     ack.IncludeOptionalField(H245_OpenLogicalChannelAck::e_reverseLogicalChannelParameters);
@@ -1085,15 +1075,6 @@ void H323DataChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
                   ::e_h2250LogicalChannelParameters);
     param = (H245_H2250LogicalChannelAckParameters*)
                 &ack.m_reverseLogicalChannelParameters.m_multiplexParameters.GetObject();
-    const H245_H2250LogicalChannelParameters & openparam =
-      open.m_reverseLogicalChannelParameters.m_multiplexParameters;
-    theSessionID = openparam.m_sessionID;
-  }
-
-  unsigned session = GetSessionID();
-  if (session != 0) {
-    param->IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID);
-    param->m_sessionID = theSessionID; // ensure same session ID is used as in OLC
   }
 
   H323TransportAddress address;
