@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26046 $
+ * $Revision: 26995 $
  * $Author: rjongbloed $
- * $Date: 2011-06-19 20:32:51 -0500 (Sun, 19 Jun 2011) $
+ * $Date: 2012-02-16 19:30:39 -0600 (Thu, 16 Feb 2012) $
  */
 
 #include <ptlib.h>
@@ -233,7 +233,8 @@ protected:
 PMutex MSRPInitialiser::mutex;
 OpalMSRPManager * MSRPInitialiser::manager = NULL;
 
-static PFactory<PProcessStartup>::Worker<MSRPInitialiser> opalpluginStartupFactory("MSRP", true);
+PFACTORY_CREATE_SINGLETON(PProcessStartupFactory, MSRPInitialiser);
+
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -362,18 +363,8 @@ void OpalMSRPMediaSession::SetConnection(PSafePtr<OpalMSRPManager::Connection> &
 
 OpalMediaSession * OpalMSRPMediaType::CreateMediaSession(OpalConnection & conn, unsigned sessionID) const
 {
-  // as this is called in the constructor of an OpalConnection descendant, 
-  // we can't use a virtual function on OpalConnection
-#if OPAL_SIP
-  if (conn.GetPrefixName() *= "sip") {
-    PTRACE(2, "MSRP\tCreating MSRP media session for SIP connection");
-    return new OpalMSRPMediaSession(conn, sessionID);
-  }
-#endif
-
-  PTRACE(2, "MSRP\tCannot create MSRP media session for unknown connection type " << conn.GetPrefixName());
-  
-  return NULL;
+  PTRACE(2, "MSRP\tCreating MSRP media session for SIP connection");
+  return new OpalMSRPMediaSession(conn, sessionID);
 }
 
 ////////////////////////////////////////////////////////
