@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22465 $
- * $Author: csoutheren $
- * $Date: 2009-04-24 13:24:46 -0500 (Fri, 24 Apr 2009) $
+ * $Revision: 26649 $
+ * $Author: rjongbloed $
+ * $Date: 2011-11-13 19:59:25 -0600 (Sun, 13 Nov 2011) $
  */
 
 #include <ptlib.h>
@@ -56,11 +56,14 @@
 
 OpalCall::OpalCall(OpalManager & mgr)
   : manager(mgr)
-  , myToken(mgr.GetNextCallToken())
+  , myToken(mgr.GetNextToken('C'))
   , isEstablished(PFalse)
   , isClearing(PFalse)
   , callEndReason(OpalConnection::NumCallEndReasons)
   , endCallSyncPoint(NULL)
+#if OPAL_HAS_MIXER
+  , m_recordManager(NULL)
+#endif
 {
   manager.activeCalls.SetAt(myToken, this);
 
@@ -76,7 +79,11 @@ OpalCall::OpalCall(OpalManager & mgr)
 
 OpalCall::~OpalCall()
 {
-  PTRACE(3, "Call\t" << *this << " destroyed.");
+#if OPAL_HAS_MIXER
+  delete m_recordManager;
+#endif
+
+  PTRACE(3, "Call\tDestroyed " << *this);
 }
 
 
@@ -88,6 +95,7 @@ void OpalCall::PrintOn(ostream & strm) const
 
 void OpalCall::OnEstablishedCall()
 {
+  PTRACE(3, "Call\tEstablished " << *this);
   manager.OnEstablishedCall(*this);
 }
 
@@ -122,6 +130,8 @@ void OpalCall::Clear(OpalConnection::CallEndReason reason, PSyncPoint * sync)
 
   UnlockReadWrite();
 
+  InternalOnClear();
+
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadWrite))
     connection->Release(reason);
@@ -131,7 +141,10 @@ void OpalCall::Clear(OpalConnection::CallEndReason reason, PSyncPoint * sync)
 void OpalCall::OnCleared()
 {
   manager.OnClearedCall(*this);
-  manager.GetRecordManager().Close(myToken);
+
+#if OPAL_HAS_MIXER
+  StopRecording();
+#endif
 
   if (!LockReadWrite())
     return;
@@ -205,7 +218,7 @@ OpalConnection::AnswerCallResponse OpalCall::OnAnswerCall(OpalConnection & PTRAC
                                                           const PString & PTRACE_PARAM(caller))
 {
   PTRACE(3, "Call\tOnAnswerCall " << connection << " caller \"" << caller << '"');
-  return OpalConnection::AnswerCallPending;
+  return OpalConnection::AnswerCallDeferred;
 }
 
 
@@ -216,17 +229,20 @@ PBoolean OpalCall::OnConnected(OpalConnection & connection)
   if (isClearing || !LockReadOnly())
     return false;
 
-  bool ok = connectionsActive.GetSize() == 1 && !m_partyB.IsEmpty();
+  bool havePartyB = connectionsActive.GetSize() == 1 && !m_partyB.IsEmpty();
 
   UnlockReadOnly();
 
-  if (ok) {
-    if (!manager.MakeConnection(*this, m_partyB, NULL, 0,
-                                const_cast<OpalConnection::StringOptions *>(&connection.GetStringOptions())))
-      connection.Release(OpalConnection::EndedByNoUser);
-    return OnSetUp(connection);
+  if (havePartyB) {
+    if (manager.MakeConnection(*this, m_partyB, NULL, 0,
+                                const_cast<OpalConnection::StringOptions *>(&connection.GetStringOptions())) != NULL)
+      return OnSetUp(connection);
+
+    connection.Release(OpalConnection::EndedByNoUser);
+    return false;
   }
 
+  bool ok = false;
   PSafePtr<OpalConnection> otherConnection;
   while (EnumerateConnections(otherConnection, PSafeReadWrite, &connection)) {
     if (otherConnection->GetPhase() >= OpalConnection::ConnectedPhase ||
@@ -270,7 +286,7 @@ PBoolean OpalCall::OnEstablished(OpalConnection & connection)
 
 PSafePtr<OpalConnection> OpalCall::GetOtherPartyConnection(const OpalConnection & connection) const
 {
-  PTRACE(3, "Call\tGetOtherPartyConnection " << connection);
+  PTRACE(4, "Call\tGetOtherPartyConnection " << connection);
 
   PSafePtr<OpalConnection> otherConnection;
   EnumerateConnections(otherConnection, PSafeReference, &connection);
@@ -286,7 +302,7 @@ bool OpalCall::Hold()
 
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadWrite)) {
-    if (connection->HoldConnection())
+    if (connection->IsNetworkConnection() && connection->Hold(false, true))
       ok = true;
   }
 
@@ -302,7 +318,7 @@ bool OpalCall::Retrieve()
 
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadWrite)) {
-    if (connection->RetrieveConnection())
+    if (connection->IsNetworkConnection() && connection->Hold(false, false))
       ok = true;
   }
 
@@ -314,7 +330,7 @@ bool OpalCall::IsOnHold() const
 {
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadOnly)) {
-    if (connection->IsConnectionOnHold())
+    if (connection->IsNetworkConnection() && connection->IsOnHold(false))
       return true;
   }
 
@@ -322,41 +338,73 @@ bool OpalCall::IsOnHold() const
 }
 
 
-bool OpalCall::Transfer(OpalConnection & connection, const PString & newAddress)
+bool OpalCall::Transfer(const PString & newAddress, OpalConnection * connection)
 {
-  if (newAddress.NumCompare(connection.GetPrefixName()+':') == EqualTo)
-    return connection.TransferConnection(newAddress);
+  PCaselessString prefix;
+  PINDEX colon = newAddress.Find(':');
+  if (colon != P_MAX_INDEX)
+    prefix = newAddress.Left(colon);
 
-  PSafePtr<OpalConnection> connectionToKeep;
-  EnumerateConnections(connectionToKeep, PSafeReference, &connection);
+  if (connection == NULL) {
+    for (PSafePtr<OpalConnection> conn = GetConnection(0); conn != NULL; ++conn) {
+      if (prefix == conn->GetPrefixName() && !conn->IsReleased())
+        return conn->TransferConnection(newAddress);
+    }
 
-  if (!manager.MakeConnection(*this, newAddress))
+    PTRACE(2, "Call\tUnable to resolve transfer to \"" << newAddress << '"');
+    return false;
+  }
+
+  if (connection->IsReleased()) {
+    PTRACE(2, "Call\tCannot transfer to released connection " << *connection);
+    return false;
+  }
+
+  if (prefix == "*")
+    return connection->TransferConnection(connection->GetPrefixName() + newAddress.Mid(1));
+
+  if (prefix.IsEmpty() || prefix == connection->GetPrefixName() || manager.HasCall(newAddress))
+    return connection->TransferConnection(newAddress);
+
+  PTRACE(3, "Call\tTransferring " << *connection << " to \"" << newAddress << '"');
+
+  PSafePtr<OpalConnection> connectionToKeep = GetOtherPartyConnection(*connection);
+  if (connectionToKeep == NULL)
     return false;
 
-  connection.Release(OpalConnection::EndedByCallForwarded);
+  PSafePtr<OpalConnection> newConnection = manager.MakeConnection(*this, newAddress);
+  if (newConnection == NULL)
+    return false;
 
-  // Close streams, but as we Released above should not do re-INVITE/OLC
-  connection.CloseMediaStreams();
+  OpalConnection::Phases oldPhase = connection->GetPhase();
+  connection->SetPhase(OpalConnection::ForwardingPhase);
 
   // Restart with new connection
-  return OnSetUp(*connectionToKeep);
+  if (newConnection->SetUpConnection() && newConnection->OnSetUpConnection()) {
+    connectionToKeep->AutoStartMediaStreams(true);
+    connection->Release(OpalConnection::EndedByCallForwarded);
+    newConnection->StartMediaStreams();
+    return true;
+  }
+
+  newConnection->Release(OpalConnection::EndedByTemporaryFailure);
+  connection->SetPhase(oldPhase);
+  return false;
 }
 
 
-OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection,
-                                              PBoolean includeSpecifiedConnection)
+OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection)
 {
   OpalMediaFormatList commonFormats;
 
-  PBoolean first = PTrue;
+  bool first = true;
 
   PSafePtr<OpalConnection> otherConnection;
-  while (EnumerateConnections(otherConnection, PSafeReadOnly, includeSpecifiedConnection ? NULL : &connection)) {
+  while (EnumerateConnections(otherConnection, PSafeReadOnly, &connection)) {
     OpalMediaFormatList possibleFormats = OpalTranscoder::GetPossibleFormats(otherConnection->GetMediaFormats());
-    otherConnection->AdjustMediaFormats(possibleFormats);
     if (first) {
       commonFormats = possibleFormats;
-      first = PFalse;
+      first = false;
     }
     else {
       // Want intersection of the possible formats for all connections.
@@ -369,24 +417,57 @@ OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection,
     }
   }
 
-  connection.AdjustMediaFormats(commonFormats);
+  // If nothing comes out of the above, e.g. when this is called very early
+  // in the call sequence and there is no second connection in call yet, then
+  // return all the possible media formats.
+  if (first)
+    commonFormats = OpalTranscoder::GetPossibleFormats(manager.GetCommonMediaFormats(false, true));
 
-  PTRACE(4, "Call\tGetMediaFormats for " << connection << '\n'
-         << setfill('\n') << commonFormats << setfill(' '));
+  connection.AdjustMediaFormats(true, NULL, commonFormats);
+
+  PTRACE(4, "Call\tGetMediaFormats for " << connection << "\n    "
+         << setfill(',') << commonFormats << setfill(' '));
 
   return commonFormats;
 }
 
 
+void OpalCall::AdjustMediaFormats(bool local, const OpalConnection & connection, OpalMediaFormatList & mediaFormats) const
+{
+  PSafePtr<OpalConnection> otherConnection;
+  while (EnumerateConnections(otherConnection, PSafeReadOnly, &connection))
+    otherConnection->AdjustMediaFormats(local, &connection, mediaFormats);
+}
+
+
 PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection, 
                                      const OpalMediaType & mediaType,
                                                   unsigned sessionID, 
-                                   const OpalMediaFormat & preselectedFormat)
+                                   const OpalMediaFormat & preselectedFormat
+#if OPAL_VIDEO
+                            , OpalVideoFormat::ContentRole contentRole
+#endif
+                                   )
 {
-  PSafeLockReadWrite lock(*this);
+  PSafeLockReadOnly lock(*this);
   if (isClearing || !lock.IsLocked())
     return false;
 
+#if PTRACING
+  PStringStream traceText;
+  if (PTrace::CanTrace(2)) {
+    traceText << " for " << mediaType << " session " << sessionID;
+    if (preselectedFormat.IsValid())
+      traceText << " (" << preselectedFormat << ')';
+    traceText << " on " << connection;
+  }
+#endif
+
+  if (IsOnHold()) {
+    PTRACE(3, "Call\tOpenSourceMediaStreams (call on hold)" << traceText);
+    return false;
+  }
+
   // Check if already done
   OpalMediaStreamPtr sinkStream;
   OpalMediaStreamPtr sourceStream = connection.GetMediaStream(sessionID, true);
@@ -396,22 +477,29 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
       sinkStream = patch->GetSink();
     if (sourceStream->GetMediaFormat() == preselectedFormat ||
         (sinkStream != NULL && sinkStream->GetMediaFormat() == preselectedFormat)) {
-      PTRACE(3, "Call\tOpenSourceMediaStreams (already opened) for session " << sessionID << " on " << connection);
+      if (sourceStream->IsPaused()) {
+        sourceStream->SetPaused(false);
+        PTRACE(3, "Call\tOpenSourceMediaStreams (un-pausing)" << traceText);
+      }
+      else {
+        PTRACE(3, "Call\tOpenSourceMediaStreams (already opened)" << traceText);
+      }
       return true;
     }
     if (sinkStream == NULL && sourceStream->IsOpen()) {
-      PTRACE(3, "Call\tOpenSourceMediaStreams (is opening) for session " << sessionID << " on " << connection);
+      PTRACE(3, "Call\tOpenSourceMediaStreams (is opening)" << traceText);
       return true;
     }
   }
 
-  PTRACE(3, "Call\tOpenSourceMediaStreams " << (sourceStream != NULL ? "replacing" : "opening")
-         << ' ' << mediaType << " session " << sessionID << " on " << connection);
+  if (sessionID == 0)
+    sessionID = connection.GetNextSessionID(mediaType, true);
+
+  PTRACE(3, "Call\tOpenSourceMediaStreams " << (sourceStream != NULL ? "replace" : "open") << traceText);
   sourceStream.SetNULL();
 
   // Create the sinks and patch if needed
   bool startedOne = false;
-  OpalMediaPatch * patch = NULL;
   OpalMediaFormat sourceFormat, sinkFormat;
 
   // Reorder destinations so we give preference to symmetric codecs
@@ -420,63 +508,113 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
 
   PSafePtr<OpalConnection> otherConnection;
   while (EnumerateConnections(otherConnection, PSafeReadWrite, &connection)) {
+    PStringArray order;
+    if (sinkFormat.IsValid())
+      order += sinkFormat.GetName(); // Preferential treatment to format already in use
+    order += '@' + mediaType;        // And media of the same type
+
     OpalMediaFormatList sinkMediaFormats = otherConnection->GetMediaFormats();
-    if (preselectedFormat.IsValid()  && sinkMediaFormats.HasFormat(preselectedFormat))
+    if (sinkMediaFormats.IsEmpty()) {
+      PTRACE(2, "Call\tOpenSourceMediaStreams failed with no sink formats" << traceText);
+      return false;
+    }
+
+    if (preselectedFormat.IsValid() && sinkMediaFormats.HasFormat(preselectedFormat))
       sinkMediaFormats = preselectedFormat;
     else
-      sinkMediaFormats.Reorder(sinkFormat.GetName()); // Preferential treatment to format already in use
+      sinkMediaFormats.Reorder(order);
 
     OpalMediaFormatList sourceMediaFormats;
     if (sourceFormat.IsValid())
       sourceMediaFormats = sourceFormat; // Use the source format already established
     else {
-      OpalMediaFormatList connectionMediaFormats = connection.GetMediaFormats();
-      if (preselectedFormat.IsValid()  && connectionMediaFormats.HasFormat(preselectedFormat))
-        sourceMediaFormats = preselectedFormat;
-      else {
-        // Only include media formats that are of the same media type
-        for (OpalMediaFormatList::iterator i = connectionMediaFormats.begin(); i != connectionMediaFormats.end(); ++i) {
-          if (i->GetMediaType() == mediaType)
-            sourceMediaFormats += *i;
-        }
+      sourceMediaFormats = connection.GetMediaFormats();
+      if (sourceMediaFormats.IsEmpty()) {
+        PTRACE(2, "Call\tOpenSourceMediaStreams failed with no source formats" << traceText);
+        return false;
       }
+
+      if (preselectedFormat.IsValid() && sourceMediaFormats.HasFormat(preselectedFormat))
+        sourceMediaFormats = preselectedFormat;
+      else
+        sourceMediaFormats.Reorder(order);
     }
 
     // Get other media directions format so we give preference to symmetric codecs
-    OpalMediaStreamPtr otherDirection = otherConnection->GetMediaStream(sessionID, true);
+    OpalMediaStreamPtr otherDirection = sessionID != 0 ? otherConnection->GetMediaStream(sessionID, true)
+                                                       : otherConnection->GetMediaStream(mediaType, true);
     if (otherDirection != NULL) {
       PString priorityFormat = otherDirection->GetMediaFormat().GetName();
       sourceMediaFormats.Reorder(priorityFormat);
       sinkMediaFormats.Reorder(priorityFormat);
     }
 
-    if (!SelectMediaFormats(sourceMediaFormats,
+#if OPAL_VIDEO
+    if (contentRole != OpalVideoFormat::eNoRole) {
+      // Remove all media formats no supporting the role
+      OpalMediaFormatList * lists[2] = { &sourceMediaFormats, &sinkMediaFormats };
+      for (PINDEX i = 0; i < 2; i++) {
+        OpalMediaFormatList::iterator format = lists[i]->begin();
+        while (format != lists[i]->end()) {
+          if ( format->GetMediaType() != mediaType ||
+              (format->IsTransportable() &&
+              (format->GetOptionInteger(OpalVideoFormat::ContentRoleMaskOption())&OpalVideoFormat::ContentRoleBit(contentRole)) == 0))
+            *lists[i] -= *format++;
+          else
+            ++format;
+        }
+        if (lists[i]->IsEmpty()) {
+          PTRACE(2, "Call\tUnsupported Content Role " << contentRole << traceText);
+          return false;
+        }
+      }
+    }
+#endif
+
+    OpalMediaFormatList localMediaFormats = OpalMediaFormat::GetAllRegisteredMediaFormats();
+    if (!SelectMediaFormats(mediaType,
+                            sourceMediaFormats,
                             sinkMediaFormats,
-                            connection.GetLocalMediaFormats(),
+                            localMediaFormats,
                             sourceFormat,
                             sinkFormat))
       return false;
 
-    if (sessionID == 0)
-      sessionID = mediaType.GetDefinition()->GetDefaultSessionId();
+#if OPAL_VIDEO
+    sourceFormat.SetOptionEnum(OpalVideoFormat::ContentRoleOption(), contentRole);
+    sinkFormat.SetOptionEnum(OpalVideoFormat::ContentRoleOption(), contentRole);
+#endif
+
+    if (sessionID == 0) {
+      sessionID = otherConnection->GetNextSessionID(mediaType, false);
+      PTRACE(3, "Call\tOpenSourceMediaStreams using session " << sessionID << " on " << connection);
+    }
 
     // Finally have the negotiated formats, open the streams
     if (sourceStream == NULL) {
       sourceStream = connection.OpenMediaStream(sourceFormat, sessionID, true);
       if (sourceStream == NULL)
         return false;
+      // If re-opening, need to disconnect from old patch. If new then this sets NULL to NULL.
+      sourceStream->SetPatch(NULL);
     }
 
     sinkStream = otherConnection->OpenMediaStream(sinkFormat, sessionID, false);
     if (sinkStream != NULL) {
       startedOne = true;
+      if (!sourceStream.SetSafetyMode(PSafeReadOnly))
+        return false;
+
+      OpalMediaPatch * patch = sourceStream->GetPatch();
       if (patch == NULL) {
         patch = manager.CreateMediaPatch(*sourceStream, sinkStream->RequiresPatchThread(sourceStream) &&
                                                         sourceStream->RequiresPatchThread(sinkStream));
         if (patch == NULL)
           return false;
       }
+
       patch->AddSink(sinkStream);
+      sourceStream.SetSafetyMode(PSafeReference);
     }
   }
 
@@ -486,34 +624,51 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     return false;
   }
 
-  if (patch != NULL) {
-    // if a patch was created, make sure the callback is called, just once per connection
-    while (EnumerateConnections(otherConnection, PSafeReadWrite))
-      otherConnection->OnPatchMediaStream(otherConnection == &connection, *patch);
+  // if a patch was created, make sure the callback is called, just once per
+  // connection. Note must lock connection before stream or we can deadlock.
+  while (EnumerateConnections(otherConnection, PSafeReadWrite)) {
+    if (!sourceStream.SetSafetyMode(PSafeReadOnly))
+      return false;
+
+    OpalMediaPatch * patch = sourceStream->GetPatch();
+    if (patch == NULL)
+      return false;
+
+    otherConnection->OnPatchMediaStream(otherConnection == &connection, *patch);
+    sourceStream.SetSafetyMode(PSafeReference);
   }
 
   return true;
 }
 
 
-bool OpalCall::SelectMediaFormats(const OpalMediaFormatList & srcFormats,
+bool OpalCall::SelectMediaFormats(const OpalMediaType & mediaType,
+                                  const OpalMediaFormatList & srcFormats,
                                   const OpalMediaFormatList & dstFormats,
                                   const OpalMediaFormatList & allFormats,
                                   OpalMediaFormat & srcFormat,
                                   OpalMediaFormat & dstFormat) const
 {
-  if (OpalTranscoder::SelectFormats(srcFormats, dstFormats, allFormats, srcFormat, dstFormat)) {
+  if (OpalTranscoder::SelectFormats(mediaType, srcFormats, dstFormats, allFormats, srcFormat, dstFormat)) {
     PTRACE(3, "Call\tSelected media formats " << srcFormat << " -> " << dstFormat);
     return true;
   }
 
-  PTRACE(2, "Call\tSelectMediaFormats could not find compatible media format:\n"
+  PTRACE(2, "Call\tSelectMediaFormats could not find compatible " << mediaType << " format:\n"
             "  source formats=" << setfill(',') << srcFormats << "\n"
             "   sink  formats=" << dstFormats << setfill(' '));
   return false;
 }
 
 
+void OpalCall::StartMediaStreams()
+{
+  PSafePtr<OpalConnection> connection;
+  while (EnumerateConnections(connection, PSafeReadWrite))
+    connection->StartMediaStreams();
+}
+
+
 void OpalCall::CloseMediaStreams()
 {
   PSafePtr<OpalConnection> connection;
@@ -583,6 +738,13 @@ void OpalCall::OnReleased(OpalConnection & connection)
     if (last != NULL)
       last->Release(connection.GetCallEndReason());
   }
+
+  InternalOnClear();
+}
+
+
+void OpalCall::InternalOnClear()
+{
   if (connectionsActive.IsEmpty() && manager.activeCalls.Contains(GetToken())) {
     OnCleared();
     manager.activeCalls.RemoveAt(GetToken());
@@ -596,51 +758,94 @@ void OpalCall::OnHold(OpalConnection & /*connection*/,
 {
 }
 
+#if OPAL_HAS_MIXER
 
-PBoolean OpalCall::StartRecording(const PFilePath & fn, bool mono)
+bool OpalCall::StartRecording(const PFilePath & fn, const OpalRecordManager::Options & options)
 {
+  StopRecording();
+
+  OpalRecordManager * newManager = OpalRecordManager::Factory::CreateInstance(fn.GetType());
+  if (newManager == NULL) {
+    PTRACE(2, "OPAL\tCannot record to file type " << fn);
+    return false;
+  }
+
   // create the mixer entry
-  if (!manager.GetRecordManager().Open(myToken, fn, mono))
-    return PFalse;
+  if (!newManager->Open(fn, options)) {
+    delete newManager;
+    return false;
+  }
+
+  PSafeLockReadWrite lock(*this);
+  if (!lock.IsLocked())
+    return false;
+
+  m_recordManager = newManager;
 
   // tell each connection to start sending data
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadWrite))
     connection->EnableRecording();
 
-  return PTrue;
+  return true;
 }
 
-
 bool OpalCall::IsRecording() const
 {
-  return manager.GetRecordManager().IsOpen(myToken);
+  PSafeLockReadOnly lock(*this);
+  return m_recordManager != NULL && m_recordManager->IsOpen();
 }
 
 
 void OpalCall::StopRecording()
 {
+  PSafeLockReadWrite lock(*this);
+  if (!lock.IsLocked() || m_recordManager == NULL)
+    return;
+
   // tell each connection to stop sending data
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadWrite))
     connection->DisableRecording();
 
-  manager.GetRecordManager().Close(myToken);
+  m_recordManager->Close();
+  delete m_recordManager;
+  m_recordManager = NULL;
+}
+
+
+bool OpalCall::OnStartRecording(const PString & streamId, const OpalMediaFormat & format)
+{
+  return m_recordManager != NULL && m_recordManager->OpenStream(streamId, format);
+}
+
+
+void OpalCall::OnStopRecording(const PString & streamId)
+{
+  if (m_recordManager != NULL)
+    m_recordManager->CloseStream(streamId);
 }
 
 
 void OpalCall::OnRecordAudio(const PString & streamId, const RTP_DataFrame & frame)
 {
-  if (!manager.GetRecordManager().WriteAudio(myToken, streamId, frame))
-    StopRecording();
+  if (m_recordManager != NULL && !m_recordManager->WriteAudio(streamId, frame))
+    m_recordManager->CloseStream(streamId);
 }
 
 
-void OpalCall::OnStopRecordAudio(const PString & streamId)
+#if OPAL_VIDEO
+
+void OpalCall::OnRecordVideo(const PString & streamId, const RTP_DataFrame & frame)
 {
-  manager.GetRecordManager().CloseStream(myToken, streamId);
+  if (m_recordManager != NULL && !m_recordManager->WriteVideo(streamId, frame))
+    m_recordManager->CloseStream(streamId);
 }
 
+#endif
+
+#endif
+
 
 bool OpalCall::IsNetworkOriginated() const
 {
@@ -698,7 +903,7 @@ bool OpalCall::EnumerateConnections(PSafePtr<OpalConnection> & connection,
 
   while (connection != NULL) {
     if (connection != skipConnection &&
-        connection->GetPhase() < OpalConnection::ReleasingPhase &&
+        connection->GetPhase() <= OpalConnection::EstablishedPhase &&
         connection.SetSafetyMode(mode))
       return true;
     ++connection;
