@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26322 $
+ * $Revision: 26557 $
  * $Author: rjongbloed $
- * $Date: 2011-08-17 20:14:41 -0500 (Wed, 17 Aug 2011) $
+ * $Date: 2011-10-06 21:53:15 -0500 (Thu, 06 Oct 2011) $
  */
 
 #include <ptlib.h>
@@ -377,8 +377,10 @@ bool OpalManagerConsole::Initialise(PArgList & args, bool verbose)
       if (verbose)
         cout << "SIP registrar: " << flush;
       PString aor;
-      if (!sip->Register(params, aor, false)) {
-        cerr << "Could not do SIP registration to " << params.m_addressOfRecord << endl;
+      SIP_PDU::StatusCodes status;
+      if (!sip->Register(params, aor, &status)) {
+        cerr << "\nSIP registration to " << params.m_addressOfRecord
+             << " failed (" << status << ')' << endl;
         return false;
       }
       if (verbose)
