@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23601 $
+ * $Revision: 25448 $
  * $Author: rjongbloed $
- * $Date: 2009-10-07 02:53:21 -0500 (Wed, 07 Oct 2009) $
+ * $Date: 2011-03-29 19:56:36 -0500 (Tue, 29 Mar 2011) $
  */
 
 #include <ptlib.h>
@@ -50,39 +50,25 @@
 
 #define new PNEW
 
-namespace PWLibStupidLinkerHacks {
-extern int opalLoader;
-
-static class InstantiateMe
-{
-  public:
-    InstantiateMe()
-    {
-      opalLoader = 1;
-    }
-} instance;
-
-}; // namespace PWLibStupidLinkerHacks
 
 /////////////////////////////////////////////////////////////////////////////
 
 OpalEndPoint::OpalEndPoint(OpalManager & mgr,
                            const PCaselessString & prefix,
                            unsigned attributes)
-  : manager(mgr),
-    prefixName(prefix),
-    attributeBits(attributes),
-    productInfo(mgr.GetProductInfo()),
-    defaultLocalPartyName(manager.GetDefaultUserName()),
-    defaultDisplayName(manager.GetDefaultDisplayName())
+  : manager(mgr)
+  , prefixName(prefix)
+  , attributeBits(attributes)
+  , defaultSignalPort(0)
+  , m_maxSizeUDP(4096)
+  , productInfo(mgr.GetProductInfo())
+  , defaultLocalPartyName(manager.GetDefaultUserName())
+  , defaultDisplayName(manager.GetDefaultDisplayName())
+  , initialBandwidth(BANDWITH_DEFAULT_INITIAL)
+  , defaultSendUserInputMode(OpalConnection::SendUserInputAsProtocolDefault)
 {
   manager.AttachEndPoint(this);
 
-  defaultSignalPort = 0;
-
-  initialBandwidth = BANDWITH_DEFAULT_INITIAL;
-  defaultSendUserInputMode = OpalConnection::SendUserInputAsProtocolDefault;
-
   if (defaultLocalPartyName.IsEmpty())
     defaultLocalPartyName = PProcess::Current().GetName() & "User";
 
@@ -180,18 +166,22 @@ PBoolean OpalEndPoint::StartListener(const OpalTransportAddress & listenerAddres
 PBoolean OpalEndPoint::StartListener(OpalListener * listener)
 {
   if (listener == NULL)
-    return PFalse;
+    return false;
+
+  OpalListenerUDP * udpListener = dynamic_cast<OpalListenerUDP *>(listener);
+  if (udpListener != NULL)
+    udpListener->SetBufferSize(m_maxSizeUDP);
 
   // as the listener is not open, this will have the effect of immediately
   // stopping the listener thread. This is good - it means that the 
   // listener Close function will appear to have stopped the thread
   if (!listener->Open(PCREATE_NOTIFIER(ListenerCallback))) {
     delete listener;
-    return PFalse;
+    return false;
   }
 
   listeners.Append(listener);
-  return PTrue;
+  return true;
 }
 
 PString OpalEndPoint::GetDefaultTransport() const
@@ -332,6 +322,15 @@ OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLo
   OpalListenerList::iterator listener;
 
   if (!associatedLocalAddress.IsEmpty()) {
+    for (listener = listeners.begin(); listener != listeners.end(); ++listener) {
+      if (listener->GetLocalAddress().GetProto() == associatedLocalAddress.GetProto())
+        AddTransportAddresses(interfaceAddresses,
+                              excludeLocalHost,
+                              natInterfaceIP,
+                              natExternalIP,
+                              associatedLocalAddress,
+                              listener->GetLocalAddress(associatedRemoteAddress));
+    }
     for (listener = listeners.begin(); listener != listeners.end(); ++listener)
       AddTransportAddresses(interfaceAddresses,
                             excludeLocalHost,
@@ -341,6 +340,16 @@ OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLo
                             listener->GetLocalAddress(associatedRemoteAddress));
   }
 
+  for (listener = listeners.begin(); listener != listeners.end(); ++listener) {
+    if (listener->GetLocalAddress().GetProto() == associatedLocalAddress.GetProto())
+      AddTransportAddresses(interfaceAddresses,
+                            excludeLocalHost,
+                            natInterfaceIP,
+                            natExternalIP,
+                            OpalTransportAddress(),
+                            listener->GetLocalAddress());
+  }
+
   for (listener = listeners.begin(); listener != listeners.end(); ++listener)
     AddTransportAddresses(interfaceAddresses,
                           excludeLocalHost,
@@ -392,10 +401,10 @@ PBoolean OpalEndPoint::HasConnection(const PString & token)
 }
 
 
-PBoolean OpalEndPoint::AddConnection(OpalConnection * connection)
+OpalConnection * OpalEndPoint::AddConnection(OpalConnection * connection)
 {
   if (connection == NULL)
-    return PFalse;
+    return NULL;
 
   connection->SetStringOptions(m_defaultStringOptions, false);
 
@@ -403,7 +412,7 @@ PBoolean OpalEndPoint::AddConnection(OpalConnection * connection)
 
   connectionsActive.SetAt(connection->GetToken(), connection);
 
-  return PTrue;
+  return connection;
 }
 
 
@@ -420,23 +429,9 @@ PBoolean OpalEndPoint::OnSetUpConnection(OpalConnection & PTRACE_PARAM(connectio
 }
 
 
-PBoolean OpalEndPoint::OnIncomingConnection(OpalConnection & /*connection*/)
-{
-  return PTrue;
-}
-
-
-PBoolean OpalEndPoint::OnIncomingConnection(OpalConnection & /*connection*/, unsigned /*options*/)
-{
-  return PTrue;
-}
-
-
 PBoolean OpalEndPoint::OnIncomingConnection(OpalConnection & connection, unsigned options, OpalConnection::StringOptions * stringOptions)
 {
-  return OnIncomingConnection(connection) &&
-         OnIncomingConnection(connection, options) &&
-         manager.OnIncomingConnection(connection, options, stringOptions);
+  return manager.OnIncomingConnection(connection, options, stringOptions);
 }
 
 
@@ -497,6 +492,12 @@ PBoolean OpalEndPoint::OnForwarded(OpalConnection & connection,
 }
 
 
+bool OpalEndPoint::OnTransferNotify(OpalConnection & connection, const PStringToString & info)
+{
+  return manager.OnTransferNotify(connection, info);
+}
+
+
 void OpalEndPoint::ConnectionDict::DeleteObject(PObject * object) const
 {
   OpalConnection * connection = PDownCast(OpalConnection, object);
@@ -530,31 +531,11 @@ void OpalEndPoint::ClearAllCalls(OpalConnection::CallEndReason reason, PBoolean
 }
 
 
-OpalMediaFormatList OpalEndPoint::GetMediaFormats() const
-{
-  OpalMediaFormatList formats;
-
-  // Sound cards can only do 16 bit PCM, but at various sample rates
-  // The following will be in order of preference, so lets do wideband first
-  formats += OpalPCM16_48KHZ;
-  formats += OpalPCM16_32KHZ;
-  formats += OpalPCM16_16KHZ;
-  formats += OpalPCM16;
-
-#if OPAL_VIDEO
-  AddVideoMediaFormats(formats);
-#endif
-
-  AddIMMediaFormats(formats);
-
-  return formats;
-}
-
-
-void OpalEndPoint::AdjustMediaFormats(const OpalConnection & connection,
+void OpalEndPoint::AdjustMediaFormats(bool local,
+                                      const OpalConnection & connection,
                                       OpalMediaFormatList & mediaFormats) const
 {
-  manager.AdjustMediaFormats(connection, mediaFormats);
+  manager.AdjustMediaFormats(local, connection, mediaFormats);
 }
 
 
@@ -571,13 +552,6 @@ void OpalEndPoint::OnClosedMediaStream(const OpalMediaStream & stream)
 }
 
 #if OPAL_VIDEO
-void OpalEndPoint::AddVideoMediaFormats(OpalMediaFormatList & mediaFormats,
-                                        const OpalConnection * connection) const
-{
-  manager.AddVideoMediaFormats(mediaFormats, connection);
-}
-
-
 PBoolean OpalEndPoint::CreateVideoInputDevice(const OpalConnection & connection,
                                           const OpalMediaFormat & mediaFormat,
                                           PVideoInputDevice * & device,
@@ -652,9 +626,70 @@ PString OpalEndPoint::GetSSLCertificate() const
 }
 #endif
 
-void OpalEndPoint::AddIMMediaFormats(OpalMediaFormatList & mediaFormats, const OpalConnection * connection) const
+
+bool OpalEndPoint::Message(const PString & to, const PString & body)
 {
-  manager.AddIMMediaFormats(mediaFormats, connection);
+  OpalIM message;
+  message.m_to   = to;
+  message.m_body = body;
+  return Message(message);
+}
+
+
+PBoolean OpalEndPoint::Message(
+  const PURL & to, 
+  const PString & type,
+  const PString & body,
+  PURL & from, 
+  PString & conversationId
+)
+{
+  OpalIM message;
+  message.m_to             = to;
+  message.m_mimeType       = type;
+  message.m_body           = body;
+  message.m_from           = from;
+  message.m_conversationId = conversationId;
+
+  bool stat = Message(message);
+
+  from           = message.m_from;
+  conversationId = message.m_conversationId;
+
+  return stat;
+}
+
+
+PBoolean OpalEndPoint::Message(OpalIM & /*Message*/)
+{
+  return false;
+}
+
+
+void OpalEndPoint::OnMessageReceived(const OpalIM & message)
+{
+  manager.OnMessageReceived(message);
+}
+
+
+/////////////////////////////////////////////////////////////////////////////
+
+bool OpalIsE164(const PString & number, bool strict)
+{
+  if (number.IsEmpty())
+    return false;
+
+  PINDEX offset;
+
+  if (strict || number[0] != '+')
+    offset = 0;
+  else {
+    if (number.GetLength() < 2)
+      return false;
+    offset = 1;
+  }
+
+  return number.FindSpan("1234567890*#", offset) == P_MAX_INDEX;
 }
 
 
