@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26090 $
+ * $Revision: 26825 $
  * $Author: rjongbloed $
- * $Date: 2011-06-28 02:21:34 -0500 (Tue, 28 Jun 2011) $
+ * $Date: 2011-12-21 19:02:59 -0600 (Wed, 21 Dec 2011) $
  */
 
 #include <ptlib.h>
@@ -60,6 +60,9 @@
 
 
 static const char * const DefaultMediaFormatOrder[] = {
+  OPAL_G7222,
+  OPAL_G7221,
+  OPAL_G722,
   OPAL_GSMAMR,
   OPAL_G7231_6k3,
   OPAL_G729B,
@@ -69,10 +72,15 @@ static const char * const DefaultMediaFormatOrder[] = {
   OPAL_iLBC,
   OPAL_GSM0610,
   OPAL_G728,
+  OPAL_G726_40K,
+  OPAL_G726_32K,
+  OPAL_G726_24K,
+  OPAL_G726_16K,
   OPAL_G711_ULAW_64K,
   OPAL_G711_ALAW_64K,
-  OPAL_H264_MODE1,
-  OPAL_H264_MODE0,
+  OPAL_H264,        // H.323 version
+  OPAL_H264_MODE1,  // SIP version, packetisation mode 1
+  OPAL_H264_MODE0,  // SIP version, packetisation mode 0
   OPAL_MPEG4,
   OPAL_H263,
   OPAL_H261
@@ -213,6 +221,7 @@ OpalManager::OpalManager()
 #ifdef OPAL_ZRTP
   , zrtpEnabled(false)
 #endif
+  , garbageCollectSkip(false)
 #ifdef OPAL_HAS_IM
   , m_imManager(NULL)
 #endif
@@ -336,6 +345,12 @@ void OpalManager::AttachEndPoint(OpalEndPoint * endpoint, const PString & prefix
   if (endpointList.GetObjectsIndex(endpoint) == P_MAX_INDEX)
     endpointList.Append(endpoint);
   endpointMap[thePrefix] = endpoint;
+
+  /* Avoid strange race condition caused when garbage collection occurs
+     on the endpoint instqance which has not completed cosntruction. This
+     is an ulgly hack andrelies on the ctors taking less than one second. */
+  garbageCollectSkip = true;
+
   PTRACE(3, "OpalMan\tAttached endpoint with prefix " << thePrefix);
 }
 
@@ -443,7 +458,7 @@ PSafePtr<OpalCall> OpalManager::SetUpCall(const PString & partyA,
 
   PTRACE_IF(2, connection == NULL, "OpalMan\tCould not create connection for \"" << partyA << '"');
 
-  OpalConnection::CallEndReason endReason = endReason = call->GetCallEndReason();
+  OpalConnection::CallEndReason endReason = call->GetCallEndReason();
   if (endReason == OpalConnection::NumCallEndReasons)
     endReason = OpalConnection::EndedByTemporaryFailure;
   call->Clear(endReason);
@@ -1792,8 +1807,12 @@ void OpalManager::CallDict::DeleteObject(PObject * object) const
 
 void OpalManager::GarbageMain(PThread &, INT)
 {
-  while (!garbageCollectExit.Wait(1000))
-    GarbageCollection();
+  while (!garbageCollectExit.Wait(1000)) {
+    if (garbageCollectSkip)
+      garbageCollectSkip = false;
+    else
+      GarbageCollection();
+  }
 }
 
 void OpalManager::OnNewConnection(OpalConnection & /*conn*/)
