@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26320 $
+ * $Revision: 26923 $
  * $Author: rjongbloed $
- * $Date: 2011-08-17 06:58:09 -0500 (Wed, 17 Aug 2011) $
+ * $Date: 2012-02-01 00:25:19 -0600 (Wed, 01 Feb 2012) $
  */
 
 #include <ptlib.h>
@@ -774,6 +774,28 @@ OpalMediaFormat::OpalMediaFormat(const char * fullName,
 }
 
 
+OpalMediaFormat::OpalMediaFormat(const OpalMediaFormat & c)
+  : PContainer() // can't use PContainer copy c-tor as this c-tor must be synchronized
+  , m_info(NULL)
+{
+  PWaitAndSignal m(c.m_mutex); // here is no need to use mutex of the shared object
+  PContainer::AssignContents(c);
+  m_info = c.m_info;
+}
+
+
+OpalMediaFormat::~OpalMediaFormat()
+{
+  if (m_info != NULL)
+    m_info->media_format_mutex.Wait(); // don't use PWaitAndSignal as m_info can be removed
+
+  Destruct();
+
+  if (m_info != NULL)
+    m_info->media_format_mutex.Signal();
+}
+
+
 void OpalMediaFormat::Construct(OpalMediaFormatInternal * info)
 {
   if (info == NULL)
@@ -853,12 +875,21 @@ PBoolean OpalMediaFormat::MakeUnique()
 
 void OpalMediaFormat::AssignContents(const PContainer & c)
 {
-  m_mutex.Wait();
+  PWaitAndSignal m1(m_mutex);
 
-  PContainer::AssignContents(c);
-  m_info = ((const OpalMediaFormat &)c).m_info;
+  const OpalMediaFormat & other = (const OpalMediaFormat &)c;
+  PWaitAndSignal m2(other.m_mutex);
 
-  m_mutex.Signal();
+  if (m_info != NULL) {
+    m_info->media_format_mutex.Wait(); // don't use PWaitAndSignal as m_info can be removed
+    PContainer::AssignContents(c);
+    if (m_info != NULL)
+      m_info->media_format_mutex.Signal();
+  }
+  else // current object can't be removed
+    PContainer::AssignContents(c);
+
+  m_info = other.m_info;
 }
 
 
@@ -867,8 +898,8 @@ void OpalMediaFormat::DestroyContents()
   m_mutex.Wait();
 
   if (m_info != NULL) {
-    m_info->media_format_mutex.Wait();
     delete m_info;
+    m_info = NULL;
   }
 
   m_mutex.Signal();
@@ -1003,6 +1034,22 @@ bool OpalMediaFormat::SetRegisteredMediaFormat(const OpalMediaFormat & mediaForm
 }
 
 
+bool OpalMediaFormat::RemoveRegisteredMediaFormat(const OpalMediaFormat & mediaFormat)
+{
+  PWaitAndSignal mutex(GetMediaFormatsListMutex());
+  OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
+
+  for (OpalMediaFormatList::iterator format = registeredFormats.begin(); format != registeredFormats.end(); ++format) {
+    if (*format == mediaFormat) {
+      registeredFormats.erase(format);
+      return true;
+    }
+  }
+
+  return false;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
