@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 26163 $
+ * $Revision: 26818 $
  * $Author: rjongbloed $
- * $Date: 2011-07-11 00:36:48 -0500 (Mon, 11 Jul 2011) $
+ * $Date: 2011-12-20 17:29:16 -0600 (Tue, 20 Dec 2011) $
  */
 
 #include <ptlib.h>
@@ -107,16 +107,19 @@ OpalMediaFormat OpalMediaStream::GetMediaFormat() const
 }
 
 
-PBoolean OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaFormat, bool fromPatch)
+bool OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaFormat)
 {
   PSafeLockReadWrite safeLock(*this);
   if (!safeLock.IsLocked())
     return false;
 
   // If we are source, then update the sink side, and vice versa
-  if (!fromPatch && mediaPatch != NULL)
-    return mediaPatch->UpdateMediaFormat(newMediaFormat);
+  return mediaPatch != NULL && mediaPatch->UpdateMediaFormat(newMediaFormat);
+}
 
+
+bool OpalMediaStream::InternalUpdateMediaFormat(const OpalMediaFormat & newMediaFormat)
+{
   if (!mediaFormat.Update(newMediaFormat))
     return false;
 
@@ -137,18 +140,15 @@ PBoolean OpalMediaStream::ExecuteCommand(const OpalMediaCommand & command)
   if (mediaPatch == NULL)
     return false;
 
-  return mediaPatch->ExecuteCommand(command, IsSink());
-}
+  PTRACE(4, "Media\tExecute command \"" << command << "\" on " << *this << " for " << connection);
 
+  if (mediaPatch->ExecuteCommand(command, IsSink()))
+    return true;
 
-void OpalMediaStream::SetCommandNotifier(const PNotifier & notifier)
-{
-  if (!LockReadWrite())
-    return;
+  if (IsSink())
+    return false;
 
-  commandNotifier = notifier;
-
-  UnlockReadWrite();
+  return connection.OnMediaCommand(*this, command);
 }
 
 
@@ -742,7 +742,8 @@ PBoolean OpalRTPMediaStream::SetDataSize(PINDEX PTRACE_PARAM(dataSize), PINDEX /
 
 PBoolean OpalRTPMediaStream::IsSynchronous() const
 {
-  return false;
+  // Sinks never block, and source with jitter buffer never blocks
+  return IsSource() && maxAudioJitterDelay == 0;
 }
 
 
@@ -1048,8 +1049,6 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
   : OpalRawMediaStream(conn, mediaFormat, sessionID, isSource, channel, autoDel)
   , m_soundChannelBuffers(buffers)
   , m_soundChannelBufferTime(bufferTime)
-  , m_failSafeRate(mediaFormat.GetTimeUnits())
-  , m_failSafeDelay(50, 10)
 {
 }
 
@@ -1070,8 +1069,6 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
                        true)
   , m_soundChannelBuffers(buffers)
   , m_soundChannelBufferTime(bufferTime)
-  , m_failSafeRate(mediaFormat.GetTimeUnits())
-  , m_failSafeDelay(50, 10)
 {
 }
 
@@ -1113,18 +1110,6 @@ PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 }
 
 
-PBoolean OpalAudioMediaStream::WriteData(const BYTE * data, PINDEX length, PINDEX & written)
-{
-  if (!OpalRawMediaStream::WriteData(data, length, written))
-    return false;
-
-  /* Note if the audio device is working correctly, then this actually will have
-     no effect, it will always be delaying 0ms */
-  m_failSafeDelay.Delay(length/sizeof(short)/m_failSafeRate);
-  return true;
-}
-
-
 PBoolean OpalAudioMediaStream::IsSynchronous() const
 {
   return true;
@@ -1183,6 +1168,36 @@ PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameTime)
 }
 
 
+bool OpalVideoMediaStream::InternalUpdateMediaFormat(const OpalMediaFormat & newMediaFormat)
+{
+  if (!OpalMediaStream::InternalUpdateMediaFormat(newMediaFormat))
+    return false;
+
+  unsigned width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption(), PVideoFrameInfo::QCIFWidth);
+  unsigned height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption(), PVideoFrameInfo::QCIFHeight);
+
+  if (m_inputDevice != NULL) {
+    if (!m_inputDevice->SetFrameSizeConverter(width, height)) {
+      PTRACE(1, "Media\tCould not set frame size in grabber to " << width << 'x' << height << " in " << mediaFormat);
+      return false;
+    }
+    if (!m_inputDevice->SetFrameRate(mediaFormat.GetClockRate()/mediaFormat.GetFrameTime())) {
+      PTRACE(1, "Media\tCould not set frame rate in grabber to " << (mediaFormat.GetClockRate()/mediaFormat.GetFrameTime()));
+      return false;
+    }
+  }
+
+  if (m_outputDevice != NULL) {
+    if (!m_outputDevice->SetFrameSizeConverter(width, height)) {
+      PTRACE(1, "Media\tCould not set frame size in video display to " << width << 'x' << height << " in " << mediaFormat);
+      return false;
+    }
+  }
+
+  return true;
+}
+
+
 PBoolean OpalVideoMediaStream::Open()
 {
   if (isOpen)
@@ -1272,8 +1287,10 @@ PBoolean OpalVideoMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & lengt
 
   PINDEX bytesReturned = size - sizeof(OpalVideoTranscoder::FrameHeader);
   unsigned flags = 0;
-  if (!m_inputDevice->GetFrameData((BYTE *)OPAL_VIDEO_FRAME_DATA_PTR(frame), &bytesReturned, flags))
+  if (!m_inputDevice->GetFrameData((BYTE *)OPAL_VIDEO_FRAME_DATA_PTR(frame), &bytesReturned, flags)) {
+    PTRACE(2, "Media\tFailed to grab frame from " << m_inputDevice->GetDeviceName());
     return false;
+  }
 
   PTimeInterval currentGrabTime = PTimer::Tick();
   timestamp += (int)((currentGrabTime - m_lastGrabTime).GetMilliSeconds()*OpalMediaFormat::VideoClockRate/1000);
