@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26246 $
+ * $Revision: 26995 $
  * $Author: rjongbloed $
- * $Date: 2011-07-25 22:42:23 -0500 (Mon, 25 Jul 2011) $
+ * $Date: 2012-02-16 19:30:39 -0600 (Thu, 16 Feb 2012) $
  */
 
 #include <ptlib.h>
@@ -293,7 +293,7 @@ class OpalManager_C : public OpalManager
 };
 
 
-class PProcess_C : public PProcess
+class PProcess_C : public PLibraryProcess
 {
 public:
   PProcess_C(const PCaselessString & options)
@@ -335,11 +335,6 @@ public:
   {
     PTRACE(1, "OpalC\tShut Down.");
   }
-
-private:
-  virtual void Main()
-  {
-  }
 };
 
 struct OpalHandleStruct
@@ -680,7 +675,7 @@ void OpalIVREndPoint_C::OnEndDialog(OpalIVRConnection & connection)
   PTRACE(4, "OpalC API\tOnEndDialog for " << connection);
 
   // Do not call ancestor and start a long pause, as do not want it to hang up
-  connection.TransferConnection("silence=3600000"); 
+  connection.TransferConnection("<vxml><form><break time=\"3600s\"/></form></vxml>"); 
 
   // Send message to app, which may (or may not) start a new IVR script
   OpalMessageBuffer message(OpalIndCompletedIVR);
@@ -981,6 +976,7 @@ OpalMessage * OpalManager_C::GetMessage(unsigned timeout)
     m_messageMutex.Signal();
   }
 
+  PTRACE_IF(4, msg != NULL, "OpalC API\tGiving message " << msg->m_type << " to application");
   return msg;
 }
 
@@ -1408,7 +1404,7 @@ void OpalManager_C::HandleSetProtocol(const OpalMessage & command, OpalMessageBu
 {
   if (IsNullString(command.m_param.m_protocol.m_prefix)) {
     SET_MESSAGE_STRING(response, m_param.m_protocol.m_userName, GetDefaultUserName());
-    if (!IsNullString(command.m_param.m_protocol.m_userName))
+    if (command.m_param.m_protocol.m_userName != NULL)
       SetDefaultUserName(command.m_param.m_protocol.m_userName);
 
     SET_MESSAGE_STRING(response, m_param.m_protocol.m_displayName, GetDefaultUserName());
@@ -1441,7 +1437,7 @@ void OpalManager_C::HandleSetProtocol(const OpalMessage & command, OpalMessageBu
   }
 
   SET_MESSAGE_STRING(response, m_param.m_protocol.m_userName, ep->GetDefaultLocalPartyName());
-  if (!IsNullString(command.m_param.m_protocol.m_userName))
+  if (command.m_param.m_protocol.m_userName != NULL)
     ep->SetDefaultLocalPartyName(command.m_param.m_protocol.m_userName);
 
   SET_MESSAGE_STRING(response, m_param.m_protocol.m_displayName, ep->GetDefaultDisplayName());
@@ -1593,6 +1589,28 @@ void OpalManager_C::HandleRegistration(const OpalMessage & command, OpalMessageB
 }
 
 
+static void SetOptionOverrides(bool originating,
+                               OpalConnection::StringOptions & options,
+                               const OpalParamProtocol & params)
+{
+  if (!IsNullString(params.m_defaultOptions)) {
+    PStringStream strm(params.m_defaultOptions);
+    strm >> options;
+  }
+
+  if (!IsNullString(params.m_userName))
+    options.Set(originating ? OPAL_OPT_CALLING_PARTY_NAME : OPAL_OPT_CALLED_PARTY_NAME, params.m_userName);
+
+  if (!IsNullString(params.m_displayName))
+    options.Set(originating ? OPAL_OPT_CALLING_DISPLAY_NAME : OPAL_OPT_CALLED_DISPLAY_NAME, params.m_displayName);
+
+  if (params.m_userInputMode > OpalUserInputDefault && params.m_userInputMode <= OpalUserInputInBand) {
+    static char const * const ModeNames[] = { "Q.931", "String", "Tone", "RFC2833", "InBand" };
+    options.Set(OPAL_OPT_USER_INPUT_MODE, ModeNames[params.m_userInputMode-1]);
+  }
+}
+
+
 void OpalManager_C::HandleSetUpCall(const OpalMessage & command, OpalMessageBuffer & response)
 {
   if (IsNullString(command.m_param.m_callSetUp.m_partyB)) {
@@ -1620,6 +1638,8 @@ void OpalManager_C::HandleSetUpCall(const OpalMessage & command, OpalMessageBuff
   OpalConnection::StringOptions options;
   if (!IsNullString(command.m_param.m_callSetUp.m_alertingType))
     options.SetAt(OPAL_OPT_ALERTING_TYPE, command.m_param.m_callSetUp.m_alertingType);
+  if (m_apiVersion >= 26)
+    SetOptionOverrides(true, options, command.m_param.m_answerCall.m_overrides);
 
   PString token;
   if (SetUpCall(partyA, command.m_param.m_callSetUp.m_partyB, token, NULL, 0, &options)) {
@@ -1645,17 +1665,21 @@ void OpalManager_C::HandleAlerting(const OpalMessage & command, OpalMessageBuffe
     return;
   }
 
+  OpalConnection::StringOptions options;
+  if (m_apiVersion >= 26)
+    SetOptionOverrides(false, options, command.m_param.m_answerCall.m_overrides);
+
 #if OPAL_HAS_PCSS
-  if (m_pcssEP != NULL && m_pcssEP->AlertingIncomingCall(command.m_param.m_callToken))
+  if (m_pcssEP != NULL && m_pcssEP->AlertingIncomingCall(command.m_param.m_callToken, &options))
     return;
 #endif
 
 #if OPAL_IVR
-  if (m_ivrEP != NULL && m_ivrEP->AlertingIncomingCall(command.m_param.m_callToken))
+  if (m_ivrEP != NULL && m_ivrEP->AlertingIncomingCall(command.m_param.m_callToken, &options))
     return;
 #endif
 
-  if (m_localEP != NULL && m_localEP->AlertingIncomingCall(command.m_param.m_callToken))
+  if (m_localEP != NULL && m_localEP->AlertingIncomingCall(command.m_param.m_callToken, &options))
     return;
 
   response.SetError("No call found by the token provided.");
@@ -1669,17 +1693,21 @@ void OpalManager_C::HandleAnswerCall(const OpalMessage & command, OpalMessageBuf
     return;
   }
 
+  OpalConnection::StringOptions options;
+  if (m_apiVersion >= 26)
+    SetOptionOverrides(false, options, command.m_param.m_answerCall.m_overrides);
+
 #if OPAL_HAS_PCSS
-  if (m_pcssEP != NULL && m_pcssEP->AcceptIncomingCall(command.m_param.m_callToken))
+  if (m_pcssEP != NULL && m_pcssEP->AcceptIncomingCall(command.m_param.m_callToken, &options))
     return;
 #endif
 
 #if OPAL_IVR
-  if (m_ivrEP != NULL && m_ivrEP->AcceptIncomingCall(command.m_param.m_callToken))
+  if (m_ivrEP != NULL && m_ivrEP->AcceptIncomingCall(command.m_param.m_callToken, &options))
     return;
 #endif
 
-  if (m_localEP != NULL && m_localEP->AcceptIncomingCall(command.m_param.m_callToken))
+  if (m_localEP != NULL && m_localEP->AcceptIncomingCall(command.m_param.m_callToken, &options))
     return;
 
   response.SetError("No call found by the token provided.");
@@ -1904,11 +1932,11 @@ void OpalManager_C::HandleMediaStream(const OpalMessage & command, OpalMessageBu
 
 void OpalManager_C::HandleStartRecording(const OpalMessage & command, OpalMessageBuffer & response)
 {
-#if OPAL_HAS_MIXER
   PSafePtr<OpalCall> call;
   if (!FindCall(command.m_param.m_userInput.m_callToken, response, call))
     return;
 
+#if OPAL_HAS_MIXER
   if (IsNullString(command.m_param.m_recording.m_file)) {
     if (!call->IsRecording())
       response.SetError("No recording active for call.");
@@ -1929,6 +1957,8 @@ void OpalManager_C::HandleStartRecording(const OpalMessage & command, OpalMessag
   }
 
   if (!call->StartRecording(command.m_param.m_recording.m_file, options))
+#else
+  if (!IsNullString(command.m_param.m_recording.m_file))
 #endif
     response.SetError("Could not start recording for call.");
 }
@@ -1936,11 +1966,11 @@ void OpalManager_C::HandleStartRecording(const OpalMessage & command, OpalMessag
 
 void OpalManager_C::HandleStopRecording(const OpalMessage & command, OpalMessageBuffer & response)
 {
-#if OPAL_HAS_MIXER
   PSafePtr<OpalCall> call;
   if (!FindCall(command.m_param.m_userInput.m_callToken, response, call))
     return;
 
+#if OPAL_HAS_MIXER
   call->StopRecording();
 #endif
 }
@@ -2476,6 +2506,13 @@ OpalStatusIncomingCall * OpalMessagePtr::GetIncomingCall() const
 }
 
 
+OpalParamAnswerCall * OpalMessagePtr::GetAnswerCall() const
+{
+  return m_message->m_type == OpalCmdAlerting ||
+         m_message->m_type == OpalCmdAnswerCall ? &m_message->m_param.m_answerCall : NULL;
+}
+
+
 OpalStatusUserInput * OpalMessagePtr::GetUserInput() const
 {
   switch (m_message->m_type) {
