@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23148 $
+ * $Revision: 26944 $
  * $Author: rjongbloed $
- * $Date: 2009-07-28 07:42:40 -0500 (Tue, 28 Jul 2009) $
+ * $Date: 2012-02-07 17:00:36 -0600 (Tue, 07 Feb 2012) $
  */
 
 #include <ptlib.h>
@@ -52,11 +52,13 @@
 /////////////////////////////////////////////////////////////////////////////
 
 OpalMediaPatch::OpalMediaPatch(OpalMediaStream & src)
-: source(src)
+  : source(src)
+  , m_bypassToPatch(NULL)
+  , m_bypassFromPatch(NULL)
+  , patchThread(NULL)
 {
+  PTRACE(5, "Patch\tCreated media patch " << this << ", session " << src.GetSessionID());
   src.SetPatch(this);
-  patchThread = NULL;
-  PTRACE(5, "Patch\tCreated media patch " << this);
 }
 
 
@@ -75,7 +77,7 @@ OpalMediaPatch::~OpalMediaPatch()
 
 void OpalMediaPatch::PrintOn(ostream & strm) const
 {
-  strm << "Patch " << source;
+  strm << "Patch[" << this << "] " << source;
 
   inUse.StartRead();
 
@@ -115,8 +117,15 @@ void OpalMediaPatch::Close()
   PTRACE(3, "Patch\tClosing media patch " << *this);
 
   inUse.StartWrite();
+
+  if (m_bypassFromPatch != NULL)
+    m_bypassFromPatch->SetBypassPatch(NULL);
+  else
+    SetBypassPatch(NULL);
+
   filters.RemoveAll();
-  source.Close();
+  if (source.GetPatch() == this)
+    source.Close();
 
   while (sinks.GetSize() > 0) {
     OpalMediaStreamPtr stream = sinks.front().stream;
@@ -139,7 +148,7 @@ void OpalMediaPatch::Close()
       return;
     }
   }
-  
+
   inUse.EndWrite();
 }
 
@@ -177,6 +186,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
     source.SetDataSize(packetSize, packetTime);
     sinkStream->SetDataSize(packetSize, packetTime);
     PTRACE(3, "Patch\tAdded direct media stream sink " << *sinkStream);
+    EnableJitterBuffer();
     return true;
   }
 
@@ -191,11 +201,13 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
       return false;
     }
     sink->primaryCodec->SetMaxOutputSize(sinkStream->GetDataSize());
+    sink->primaryCodec->SetSessionID(source.GetSessionID());
 
     PTRACE(3, "Patch\tAdded media stream sink " << *sinkStream
            << " using transcoder " << *sink->primaryCodec << ", data size=" << sinkStream->GetDataSize());
   }
   else {
+    PTRACE(4, "Patch\tCreating two stage transcoders for " << sourceFormat << "->" << destinationFormat << " with ID " << id);
     OpalMediaFormat intermediateFormat;
     if (!OpalTranscoder::FindIntermediateFormat(sourceFormat, destinationFormat,
                                                 intermediateFormat)) {
@@ -203,10 +215,32 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
       return false;
     }
 
+    if (intermediateFormat.GetMediaType() == OpalMediaType::Audio()) {
+      // try prepare intermediateFormat for correct frames to frames transcoding
+      // so we need make sure that tx frames time of destinationFormat be equal 
+      // to tx frames time of intermediateFormat (all this does not produce during
+      // Merge phase in FindIntermediateFormat)
+      int destinationPacketTime = destinationFormat.GetFrameTime()*destinationFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1);
+      if ((destinationPacketTime % intermediateFormat.GetFrameTime()) != 0) {
+        PTRACE(1, "Patch\tCould produce without buffered media format converting (which not implemented yet) for " << *sinkStream);
+        return false;
+      }
+      intermediateFormat.AddOption(new OpalMediaOptionUnsigned(OpalAudioFormat::TxFramesPerPacketOption(),
+                                                               true,
+                                                               OpalMediaOption::NoMerge,
+                                                               destinationPacketTime/intermediateFormat.GetFrameTime()),
+                                   true);
+    }
+
     sink->primaryCodec = OpalTranscoder::Create(sourceFormat, intermediateFormat, (const BYTE *)id, id.GetLength());
     sink->secondaryCodec = OpalTranscoder::Create(intermediateFormat, destinationFormat, (const BYTE *)id, id.GetLength());
+    if (sink->primaryCodec == NULL || sink->secondaryCodec == NULL)
+      return false;
 
-    PTRACE(4, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
+    PTRACE(3, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
+
+    sink->primaryCodec->SetMaxOutputSize(sink->secondaryCodec->GetOptimalDataFrameSize(true));
+    sink->primaryCodec->SetSessionID(source.GetSessionID());
 
     if (!sinkStream->SetDataSize(sink->secondaryCodec->GetOptimalDataFrameSize(false), sourceFormat.GetFrameTime())) {
       PTRACE(1, "Patch\tSink stream " << *sinkStream << " cannot support data size "
@@ -214,13 +248,21 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
       return false;
     }
     sink->secondaryCodec->SetMaxOutputSize(sinkStream->GetDataSize());
+    sink->secondaryCodec->SetSessionID(source.GetSessionID());
 
     PTRACE(3, "Patch\tAdded media stream sink " << *sinkStream
            << " using transcoders " << *sink->primaryCodec
            << " and " << *sink->secondaryCodec << ", data size=" << sinkStream->GetDataSize());
   }
 
+  if (sink->secondaryCodec != NULL)
+    sink->secondaryCodec->SetCommandNotifier(PCREATE_NOTIFIER(OnMediaCommand));
+
+  if (sink->primaryCodec != NULL)
+    sink->primaryCodec->SetCommandNotifier(PCREATE_NOTIFIER(OnMediaCommand));
+
   source.SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(true), destinationFormat.GetFrameTime());
+  EnableJitterBuffer();
   return true;
 }
 
@@ -232,6 +274,8 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
 
   PTRACE(3, "Patch\tRemoving media stream sink " << *stream);
 
+  bool closeSource = false;
+
   inUse.StartWrite();
 
   for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
@@ -242,7 +286,16 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
     }
   }
 
+  if (sinks.IsEmpty()) {
+    closeSource = true;
+    if (m_bypassFromPatch != NULL)
+      m_bypassFromPatch->SetBypassPatch(NULL);
+  }
+
   inUse.EndWrite();
+
+  if (closeSource  && source.GetPatch() == this)
+    source.Close();
 }
 
 
@@ -330,6 +383,8 @@ OpalMediaPatch::Sink::Sink(OpalMediaPatch & p, const OpalMediaStreamPtr & s)
   , primaryCodec(NULL)
   , secondaryCodec(NULL)
   , writeSuccessful(true)
+  , m_lastPayloadType(RTP_DataFrame::IllegalPayloadType)
+  , m_consecutivePayloadTypeMismatches(0)
 #if OPAL_VIDEO
   , rateController(NULL)
 #endif
@@ -355,11 +410,16 @@ OpalMediaPatch::Sink::~Sink()
 void OpalMediaPatch::AddFilter(const PNotifier & filter, const OpalMediaFormat & stage)
 {
   PWriteWaitAndSignal mutex(inUse);
-  
+
+  if (source.GetMediaFormat().GetMediaType() != stage.GetMediaType())
+    return;
+
   // ensures that a filter is added only once
   for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
-    if (f->notifier == filter && f->stage == stage)
+    if (f->notifier == filter && f->stage == stage) {
+      PTRACE(3, "OpalCon\tFilter already added for stage " << stage);
       return;
+    }
   }
   filters.Append(new Filter(filter, stage));
 }
@@ -372,11 +432,12 @@ PBoolean OpalMediaPatch::RemoveFilter(const PNotifier & filter, const OpalMediaF
   for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
     if (f->notifier == filter && f->stage == stage) {
       filters.erase(f);
-      return PTrue;
+      return true;
     }
   }
 
-  return PFalse;
+  PTRACE(3, "OpalCon\tNo filter to remove for stage " << stage);
+  return false;
 }
 
 
@@ -396,9 +457,9 @@ void OpalMediaPatch::FilterFrame(RTP_DataFrame & frame,
 
 bool OpalMediaPatch::UpdateMediaFormat(const OpalMediaFormat & mediaFormat)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PWriteWaitAndSignal mutex(inUse);
 
-  bool atLeastOne = source.UpdateMediaFormat(mediaFormat, true);
+  bool atLeastOne = source.InternalUpdateMediaFormat(mediaFormat);
 
   for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
     atLeastOne = s->UpdateMediaFormat(mediaFormat) || atLeastOne;
@@ -412,49 +473,78 @@ PBoolean OpalMediaPatch::ExecuteCommand(const OpalMediaCommand & command, PBoole
 {
   PReadWaitAndSignal mutex(inUse);
 
-  if (fromSink)
-    return source.ExecuteCommand(command);
+  if (fromSink) {
+    OpalMediaPatch * patch = m_bypassToPatch != NULL ? m_bypassToPatch : this;
+    PTRACE(5, "Patch\tExecute command \"" << command << "\" "
+           << (m_bypassToPatch != NULL ? "bypassed" : "normally") << ' ' << *this);
+    return patch->source.ExecuteCommand(command);
+  }
 
-  PBoolean atLeastOne = PFalse;
-  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
-    atLeastOne = s->ExecuteCommand(command) || atLeastOne;
+  bool atLeastOne = false;
+
+  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
+    if (s->ExecuteCommand(command))
+      atLeastOne = true;
+  }
 
   return atLeastOne;
 }
 
 
-void OpalMediaPatch::SetCommandNotifier(const PNotifier & notifier, PBoolean fromSink)
+void OpalMediaPatch::OnMediaCommand(OpalMediaCommand & command, INT)
 {
-  PReadWaitAndSignal mutex(inUse);
-
-  if (fromSink)
-    source.SetCommandNotifier(notifier);
-  else {
-    for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
-      s->SetCommandNotifier(notifier);
-  }
+  source.ExecuteCommand(command);
 }
 
 
-bool OpalMediaPatch::OnPatchStart()
+bool OpalMediaPatch::SetPaused(bool pause)
 {
-  source.OnPatchStart();
+  PReadWaitAndSignal mutex(inUse);
+
+  bool atLeastOne = source.SetPaused(pause, true);
+
+  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
+    if (s->stream->SetPaused(pause, true))
+      atLeastOne = true;
+  }
+
+  if (!pause)
+    Start();
+
+  return atLeastOne;
+}
+
+
+bool OpalMediaPatch::OnStartMediaPatch()
+{
+  source.OnStartMediaPatch();
 
   if (source.IsSynchronous())
     return false;
 
-  bool isAudio = source.GetMediaFormat().GetMediaType() == OpalMediaType::Audio();
+  return EnableJitterBuffer();
+}
 
+
+bool OpalMediaPatch::EnableJitterBuffer()
+{
   PReadWaitAndSignal mutex(inUse);
 
-  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
-    if (s->stream->IsSynchronous()) {
-      if (isAudio) 
-        source.EnableJitterBuffer();
+  bool enab = m_bypassToPatch == NULL;
+
+  PList<Sink>::iterator s;
+  for (s = sinks.begin(); s != sinks.end(); ++s) {
+    if (s->stream->EnableJitterBuffer(enab)) {
+      source.EnableJitterBuffer(false);
       return false;
     }
   }
-	
+
+  for (s = sinks.begin(); s != sinks.end(); ++s) {
+    if (s->stream->IsSynchronous() && source.EnableJitterBuffer(enab))
+      return false;
+  }
+
   return true;
 }
 
@@ -463,7 +553,9 @@ void OpalMediaPatch::Main()
 {
   PTRACE(4, "Patch\tThread started for " << *this);
 	
-  bool asynchronous = OnPatchStart();
+  bool asynchronous = OnStartMediaPatch();
+  PAdaptiveDelay asynchPacing;
+  PThread::Times lastThreadTimes;
   PTimeInterval lastTick;
 
   /* Note the RTP frame is outside loop so that a) it is more efficient
@@ -476,6 +568,12 @@ void OpalMediaPatch::Main()
   RTP_DataFrame sourceFrame(0);
 
   while (source.IsOpen()) {
+    if (source.IsPaused()) {
+      PThread::Sleep(100);
+      continue;
+    }
+
+    sourceFrame.MakeUnique();
     sourceFrame.SetPayloadType(source.GetMediaFormat().GetPayloadType());
 
     // We do the following to make sure that the buffer size is large enough,
@@ -493,40 +591,121 @@ void OpalMediaPatch::Main()
     inUse.EndRead();
 
     if (!written) {
-      PTRACE(4, "Patch\tThread ended because all sink writes failed failed");
+      PTRACE(4, "Patch\tThread ended because all sink writes failed");
       break;
     }
  
+    if (asynchronous)
+      asynchPacing.Delay(10);
+
     /* Don't starve the CPU if we have idle frames and the no source or
        destination is synchronous. Note that performing a Yield is not good
        enough, as the media patch threads are high priority and will consume
        all available CPU if allowed. Also just doing a sleep each time around
        the loop slows down video where you get clusters of packets thrown at
        us, want to clear them as quickly as possible out of the UDP OS buffers
-       or we overflow and lose some. Best compromise is to every 100ms, sleep
-       for 10ms so can not use more than about 90% of CPU. */
-    if (asynchronous && PTimer::Tick() - lastTick > 100) {
-      PThread::Sleep(10);
+       or we overflow and lose some. Best compromise is to every X ms, sleep
+       for X/10 ms so can not use more than about 90% of CPU. */
+#if P_CONFIG_FILE
+    static const unsigned SampleTimeMS = PConfig(PConfig::Environment).GetInteger("OPAL_MEDIA_PATCH_CPU_CHECK", 1000);
+#else
+    static const unsigned SampleTimeMS = 1000;
+#endif
+    static const unsigned ThresholdPercent = 90;
+    if (PTimer::Tick() - lastTick > SampleTimeMS) {
+      PThread::Times threadTimes;
+      if (PThread::Current()->GetTimes(threadTimes)) {
+        PTRACE(5, "Patch\tCPU for " << *this << " is " << threadTimes);
+        if ((threadTimes.m_user - lastThreadTimes.m_user + threadTimes.m_kernel - lastThreadTimes.m_kernel)
+                                      > (threadTimes.m_real - lastThreadTimes.m_real)*ThresholdPercent/100) {
+          PTRACE(2, "Patch\tGreater that 90% CPU usage for " << *this);
+          PThread::Sleep(SampleTimeMS*(100-ThresholdPercent)/100);
+        }
+        lastThreadTimes = threadTimes;
+      }
       lastTick = PTimer::Tick();
     }
   }
 
-  source.OnPatchStop();
+  source.OnStopMediaPatch(*this);
 
   PTRACE(4, "Patch\tThread ended for " << *this);
 }
 
 
+bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
+{
+  PWriteWaitAndSignal mutex(inUse);
+
+  if (!PAssert(m_bypassFromPatch == NULL, PLogicError))
+    return false; // Can't be both!
+
+  if (m_bypassToPatch == patch)
+    return true; // Already set
+
+  PTRACE(4, "Patch\tSetting media patch bypass to " << patch << " on " << *this);
+
+  if (m_bypassToPatch != NULL) {
+    if (!PAssert(m_bypassToPatch->m_bypassFromPatch == this, PLogicError))
+      return false;
+
+    m_bypassToPatch->m_bypassFromPatch = NULL;
+    m_bypassToPatch->m_bypassEnded.Signal();
+  }
+
+  if (patch != NULL) {
+    if (!PAssert(patch->m_bypassFromPatch == NULL, PLogicError))
+      return false;
+
+    patch->m_bypassFromPatch = this;
+  }
+
+  m_bypassToPatch = patch;
+
+#if OPAL_VIDEO
+  OpalMediaFormat format = source.GetMediaFormat();
+  if (format.IsTransportable() && format.GetMediaType() == OpalMediaType::Video())
+    source.ExecuteCommand(OpalVideoUpdatePicture());
+  else
+#endif
+    EnableJitterBuffer();
+
+  return true;
+}
+
+
+PBoolean OpalMediaPatch::PushFrame(RTP_DataFrame & frame)
+{
+  PReadWaitAndSignal mutex(inUse);
+  return DispatchFrame(frame);
+}
+
+
 bool OpalMediaPatch::DispatchFrame(RTP_DataFrame & frame)
 {
-  FilterFrame(frame, source.GetMediaFormat());    
+  if (m_bypassFromPatch != NULL) {
+    PTRACE(3, "Patch\tMedia patch bypass started by " << *m_bypassFromPatch << " on " << *this);
+    inUse.EndRead();
+    m_bypassEnded.Wait();
+    PTRACE(4, "Patch\tMedia patch bypass ended on " << *this);
+    inUse.StartRead();
+    return true;
+  }
+
+  FilterFrame(frame, source.GetMediaFormat());
 
   bool written = false;
-  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
-    if (s->WriteFrame(frame))
-      written = true;
-    else {
-      PTRACE(2, "Patch\tWriteFrame failed");
+
+  if (m_bypassToPatch == NULL) {
+    for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
+      if (s->WriteFrame(frame))
+        written = true;
+    }
+  }
+  else {
+    for (PList<Sink>::iterator s = m_bypassToPatch->sinks.begin(); s != m_bypassToPatch->sinks.end(); ++s) {
+      if (s->stream->WritePacket(frame))
+        written = true;
     }
   }
 
@@ -539,16 +718,16 @@ bool OpalMediaPatch::Sink::UpdateMediaFormat(const OpalMediaFormat & mediaFormat
   bool ok;
 
   if (primaryCodec == NULL)
-    ok = stream->UpdateMediaFormat(mediaFormat, true);
+    ok = stream->InternalUpdateMediaFormat(mediaFormat);
   else if (secondaryCodec != NULL && secondaryCodec->GetOutputFormat() == mediaFormat)
     ok = secondaryCodec->UpdateMediaFormats(OpalMediaFormat(), mediaFormat) &&
-         stream->UpdateMediaFormat(secondaryCodec->GetOutputFormat(), true);
+         stream->InternalUpdateMediaFormat(secondaryCodec->GetOutputFormat());
   else if (primaryCodec->GetOutputFormat() == mediaFormat)
     ok = primaryCodec->UpdateMediaFormats(OpalMediaFormat(), mediaFormat) &&
-         stream->UpdateMediaFormat(primaryCodec->GetOutputFormat(), true);
+         stream->InternalUpdateMediaFormat(primaryCodec->GetOutputFormat());
   else
     ok = primaryCodec->UpdateMediaFormats(mediaFormat, OpalMediaFormat()) &&
-         stream->UpdateMediaFormat(primaryCodec->GetInputFormat(), true);
+         stream->InternalUpdateMediaFormat(primaryCodec->GetInputFormat());
 
 #if OPAL_VIDEO
     SetRateControlParameters(stream->GetMediaFormat());
@@ -573,20 +752,15 @@ bool OpalMediaPatch::Sink::ExecuteCommand(const OpalMediaCommand & command)
 }
 
 
-void OpalMediaPatch::Sink::SetCommandNotifier(const PNotifier & notifier)
-{
-  if (secondaryCodec != NULL)
-    secondaryCodec->SetCommandNotifier(notifier);
-
-  if (primaryCodec != NULL)
-    primaryCodec->SetCommandNotifier(notifier);
-}
-
-
-static bool CannotTranscodeFrame(const OpalTranscoder & codec, RTP_DataFrame & frame)
+bool OpalMediaPatch::Sink::CannotTranscodeFrame(OpalTranscoder & codec, RTP_DataFrame & frame)
 {
   RTP_DataFrame::PayloadTypes pt = frame.GetPayloadType();
 
+  if (!codec.AcceptEmptyPayload() && frame.GetPayloadSize() == 0) {
+    frame.SetPayloadType(codec.GetPayloadType(false));
+    return true;
+  }
+
   if (!codec.AcceptComfortNoise()) {
     if (pt == RTP_DataFrame::CN || pt == RTP_DataFrame::Cisco_CN) {
       PTRACE(4, "Patch\tRemoving comfort noise frame with payload type " << pt);
@@ -597,17 +771,24 @@ static bool CannotTranscodeFrame(const OpalTranscoder & codec, RTP_DataFrame & f
   }
 
   if ((pt != codec.GetPayloadType(true)) && !codec.AcceptOtherPayloads()) {
+    if (pt != m_lastPayloadType) {
+      m_consecutivePayloadTypeMismatches = 0;
+      m_lastPayloadType = pt;
+    }
+    else if (++m_consecutivePayloadTypeMismatches > 10) {
+      PTRACE(2, "Patch\tConsecutive mismatched payload type, was expecting " 
+             << codec.GetPayloadType(true) << ", now using " << pt);
+      OpalMediaFormat fmt = codec.GetInputFormat();
+      fmt.SetPayloadType(pt);
+      codec.UpdateMediaFormats(fmt, OpalMediaFormat());
+      return false;
+    }
     PTRACE(4, "Patch\tRemoving frame with mismatched payload type " << pt << " - should be " << codec.GetPayloadType(true));
     frame.SetPayloadSize(0);   // remove the payload because the transcoder has indicated it won't understand it
     frame.SetPayloadType(codec.GetPayloadType(true)); // Reset pt so if get silence frames from jitter buffer, they don't cause errors
     return true;
   }
 
-  if (!codec.AcceptEmptyPayload() && frame.GetPayloadSize() == 0) {
-    frame.SetPayloadType(codec.GetPayloadType(false));
-    return true;
-  }
-
   return false;
 }
 
@@ -653,6 +834,9 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
   if (!writeSuccessful)
     return false;
   
+  if (stream->IsPaused())
+    return true;
+
 #if OPAL_VIDEO
   if (rateController != NULL) {
     bool forceIFrame = false;
@@ -701,7 +885,7 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
         patch.FilterFrame(*interFrame, primaryCodec->GetOutputFormat());
         if (!stream->WritePacket(*interFrame))
           return (writeSuccessful = false);
-        sourceFrame.SetTimestamp(interFrame->GetTimestamp());
+        primaryCodec->CopyTimestamp(sourceFrame, *interFrame, false);
         continue;
       }
       intermediateFrames.RemoveAll();
@@ -715,7 +899,7 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
     if (secondaryCodec == NULL) {
       if (!stream->WritePacket(*interFrame))
         return (writeSuccessful = false);
-      sourceFrame.SetTimestamp(interFrame->GetTimestamp());
+      primaryCodec->CopyTimestamp(sourceFrame, *interFrame, false);
       continue;
     }
 
@@ -734,7 +918,7 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
       patch.FilterFrame(*finalFrame, secondaryCodec->GetOutputFormat());
       if (!stream->WritePacket(*finalFrame))
         return (writeSuccessful = false);
-      sourceFrame.SetTimestamp(finalFrame->GetTimestamp());
+      secondaryCodec->CopyTimestamp(sourceFrame, *finalFrame, false);
     }
   }
 
@@ -761,21 +945,13 @@ OpalMediaPatch::Thread::Thread(OpalMediaPatch & p)
 
 
 OpalPassiveMediaPatch::OpalPassiveMediaPatch(OpalMediaStream & source)
-: OpalMediaPatch(source)
+  : OpalMediaPatch(source)
 {
 }
 
 
 void OpalPassiveMediaPatch::Start()
 {
-  source.OnPatchStart();
+  OnStartMediaPatch();
 }
 
-
-PBoolean OpalPassiveMediaPatch::PushFrame(RTP_DataFrame & frame)
-{
-  PReadWaitAndSignal mutex(inUse);
-  return DispatchFrame(frame);
-}
-
-
