@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 25956 $
+ * $Revision: 26760 $
  * $Author: rjongbloed $
- * $Date: 2011-06-01 21:39:26 -0500 (Wed, 01 Jun 2011) $
+ * $Date: 2011-12-06 19:02:40 -0600 (Tue, 06 Dec 2011) $
  */
 
 #include <ptlib.h>
@@ -1081,7 +1081,7 @@ PBoolean OpalTransportTCP::Connect()
     PTRACE(4, "OpalTCP\tConnecting to "
            << remoteAddress.AsString(true) << ':' << remotePort
            << " (local port=" << localPort << ')');
-    if (socket->Connect(localPort, remoteAddress))
+    if (socket->Connect(localAddress, localPort, remoteAddress))
       break;
 
     int errnum = socket->GetErrorNumber();
@@ -1570,7 +1570,7 @@ PBoolean OpalTransportTCPS::Connect()
     PTRACE(4, "OpalTCPS\tConnecting to "
            << remoteAddress.AsString(true) << ':' << remotePort
            << " (local port=" << localPort << ')');
-    if (socket->Connect(localPort, remoteAddress))
+    if (socket->Connect(localAddress, localPort, remoteAddress))
       break;
 
     int errnum = socket->GetErrorNumber();
