@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26293 $
+ * $Revision: 26862 $
  * $Author: rjongbloed $
- * $Date: 2011-08-08 21:25:33 -0500 (Mon, 08 Aug 2011) $
+ * $Date: 2012-01-15 06:30:18 -0600 (Sun, 15 Jan 2012) $
  */
 
 #include <ptlib.h>
@@ -570,6 +570,16 @@ void RTP_UserData::OnTxIntraFrameRequest(const RTP_Session & /*session*/) const
 
 /////////////////////////////////////////////////////////////////////////////
 
+#if P_CONFIG_FILE
+static PTimeInterval GetDefaultOutOfOrderWaitTime()
+{
+  static PTimeInterval ooowt(PConfig(PConfig::Environment).GetInteger("OPAL_RTP_OUT_OF_ORDER_TIME", 100));
+  return ooowt;
+}
+#else
+#define GetDefaultOutOfOrderWaitTime() (100)
+#endif
+
 RTP_Session::RTP_Session(const Params & params)
   : m_timeUnits(params.isAudio ? 8 : 90)
   , canonicalName(PProcess::Current().GetUserName())
@@ -577,6 +587,7 @@ RTP_Session::RTP_Session(const Params & params)
   , reportTimeInterval(0, 12)  // Seconds
   , lastSRTimestamp(0)
   , lastSRReceiveTime(0)
+  , outOfOrderWaitTime(GetDefaultOutOfOrderWaitTime())
   , firstPacketSent(0)
   , firstPacketReceived(0)
   , reportTimer(reportTimeInterval)
@@ -627,13 +638,19 @@ RTP_Session::~RTP_Session()
 {
 #if PTRACING
   PTime now;
-#endif
+  int sentDuration = (now-firstPacketSent).GetSeconds();
+  if (sentDuration == 0)
+    sentDuration = 1;
+  int receiveDuration = (now-firstPacketReceived).GetSeconds();
+  if (receiveDuration == 0)
+    receiveDuration = 1;
+ #endif
   PTRACE_IF(3, packetsSent != 0 || packetsReceived != 0,
       "RTP\tSession " << sessionID << ", final statistics:\n"
       "    firstPacketSent    = " << firstPacketSent << "\n"
       "    packetsSent        = " << packetsSent << "\n"
       "    octetsSent         = " << octetsSent << "\n"
-      "    bitRateSent        = " << (8*octetsSent/(now-firstPacketSent).GetSeconds()) << "\n"
+      "    bitRateSent        = " << (8*octetsSent/sentDuration) << "\n"
       "    averageSendTime    = " << averageSendTime << "\n"
       "    maximumSendTime    = " << maximumSendTime << "\n"
       "    minimumSendTime    = " << minimumSendTime << "\n"
@@ -642,7 +659,7 @@ RTP_Session::~RTP_Session()
       "    firstPacketReceived= " << firstPacketReceived << "\n"
       "    packetsReceived    = " << packetsReceived << "\n"
       "    octetsReceived     = " << octetsReceived << "\n"
-      "    bitRateReceived    = " << (8*octetsReceived/(now-firstPacketReceived).GetSeconds()) << "\n"
+      "    bitRateReceived    = " << (8*octetsReceived/receiveDuration) << "\n"
       "    packetsLost        = " << packetsLost << "\n"
       "    packetsTooLate     = " << GetPacketsTooLate() << "\n"
       "    packetOverruns     = " << GetPacketOverruns() << "\n"
@@ -1322,7 +1339,7 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
       }
     }
     else if (resequenceOutOfOrderPackets &&
-                (m_outOfOrderPackets.empty() || (tick - outOfOrderPacketTime) < 200)) {
+                (m_outOfOrderPackets.empty() || (tick - outOfOrderPacketTime) < outOfOrderWaitTime)) {
       if (m_outOfOrderPackets.empty())
         outOfOrderPacketTime = tick;
       // Maybe packet lost, maybe out of order, save for now
@@ -1896,7 +1913,7 @@ static void SetMinBufferSize(PUDPSocket & sock, int buftype, int bufsz)
 {
   int sz = 0;
   if (!sock.GetOption(buftype, sz)) {
-    PTRACE(1, "RTP_UDP\tGetOption(" << buftype << ") failed: " << sock.GetErrorText());
+    PTRACE(1, "RTP_UDP\tGetOption(" << sock.GetHandle() << ',' << buftype << ") failed: " << sock.GetErrorText());
     return;
   }
 
@@ -1907,20 +1924,22 @@ static void SetMinBufferSize(PUDPSocket & sock, int buftype, int bufsz)
   for (; bufsz >= 1024; bufsz /= 2) {
     // Set to new size
     if (!sock.SetOption(buftype, bufsz)) {
-      PTRACE(1, "RTP_UDP\tSetOption(" << buftype << ',' << bufsz << ") failed: " << sock.GetErrorText());
+      PTRACE(1, "RTP_UDP\tSetOption(" << sock.GetHandle() << ',' << buftype << ',' << bufsz << ") failed: " << sock.GetErrorText());
       continue;
     }
 
     // As some stacks lie about setting the buffer size, we double check.
     if (!sock.GetOption(buftype, sz)) {
-      PTRACE(1, "RTP_UDP\tGetOption(" << buftype << ") failed: " << sock.GetErrorText());
+      PTRACE(1, "RTP_UDP\tGetOption(" << sock.GetHandle() << ',' << buftype << ") failed: " << sock.GetErrorText());
       return;
     }
 
-    if (sz >= bufsz)
+    if (sz >= bufsz) {
+      PTRACE(4, "RTP_UDP\tSetOption(" << sock.GetHandle() << ',' << buftype << ',' << bufsz << ") succeeded.");
       return;
+    }
 
-    PTRACE(1, "RTP_UDP\tSetOption(" << buftype << ',' << bufsz << ") failed, even though it said it succeeded!");
+    PTRACE(1, "RTP_UDP\tSetOption(" << sock.GetHandle() << ',' << buftype << ',' << bufsz << ") failed, even though it said it succeeded!");
   }
 }
 
@@ -2375,7 +2394,7 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(BYTE * framePtr,
     return RTP_Session::e_ProcessPacket;
   }
 
-  switch (socket.GetErrorNumber()) {
+  switch (socket.GetErrorNumber(PChannel::LastReadError)) {
     case ECONNRESET :
     case ECONNREFUSED :
       PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " port on remote not ready.");
@@ -2590,7 +2609,8 @@ bool RTP_UDP::WriteDataOrControlPDU(const BYTE * framePtr, PINDEX frameSize, boo
 
       default:
         PTRACE(1, "RTP_UDP\tSession " << sessionID
-               << ", write error on " << (toDataChannel ? "data" : "control") << " port ("
+               << ", write (" << frameSize << " bytes) error on "
+               << (toDataChannel ? "data" : "control") << " port ("
                << socket.GetErrorNumber(PChannel::LastWriteError) << "): "
                << socket.GetErrorText(PChannel::LastWriteError));
         return false;
