@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26327 $
+ * $Revision: 27008 $
  * $Author: rjongbloed $
- * $Date: 2011-08-18 19:13:00 -0500 (Thu, 18 Aug 2011) $
+ * $Date: 2012-02-19 20:47:08 -0600 (Sun, 19 Feb 2012) $
  *
  * This code implements all or part of the following RFCs
  *
@@ -83,19 +83,17 @@ SIPHandler::SIPHandler(SIP_PDU::Methods method, SIPEndPoint & ep, const SIPParam
   , m_method(method)
   , m_addressOfRecord(params.m_addressOfRecord)
   , m_remoteAddress(params.m_remoteAddress)
-  , callID(SIPTransaction::GenerateCallID())
-  , expire(params.m_expire)
-  , originalExpire(params.m_expire)
-  , offlineExpire(params.m_restoreTime)
-  , authenticationAttempts(0)
+  , m_callID(SIPTransaction::GenerateCallID())
+  , m_lastCseq(0)
+  , m_currentExpireTime(params.m_expire)
+  , m_originalExpireTime(params.m_expire)
+  , m_offlineExpireTime(params.m_restoreTime)
   , m_state(Unavailable)
   , m_receivedResponse(false)
-  , retryTimeoutMin(params.m_minRetryTime)
-  , retryTimeoutMax(params.m_maxRetryTime)
   , m_proxy(params.m_proxyAddress)
 {
   m_transactions.DisallowDeleteObjects();
-  expireTimer.SetNotifier(PCREATE_NOTIFIER(OnExpireTimeout));
+  m_expireTimer.SetNotifier(PCREATE_NOTIFIER(OnExpireTimeout));
 
   if (m_proxy.IsEmpty())
     m_proxy = ep.GetProxy();
@@ -106,7 +104,7 @@ SIPHandler::SIPHandler(SIP_PDU::Methods method, SIPEndPoint & ep, const SIPParam
 
 SIPHandler::~SIPHandler() 
 {
-  expireTimer.Stop();
+  m_expireTimer.Stop();
 
   if (m_transport) {
     m_transport->CloseWait();
@@ -123,7 +121,7 @@ PObject::Comparison SIPHandler::Compare(const PObject & obj) const
 {
   PAssert(PIsDescendant(&obj, SIPHandler), PInvalidCast);
   const SIPHandler * other = dynamic_cast<const SIPHandler *>(&obj);
-  return other != NULL ? callID.Compare(other->callID) : GreaterThan;
+  return other != NULL ? GetCallID().Compare(other->GetCallID()) : GreaterThan;
 }
 
 
@@ -134,10 +132,10 @@ bool SIPHandler::ShutDown()
   {
     PSafeLockReadWrite mutex(*this);
     if (!mutex.IsLocked())
-      return true;
+    return true;
 
-    while (!m_stateQueue.empty())
-      m_stateQueue.pop();
+  while (!m_stateQueue.empty())
+    m_stateQueue.pop();
 
     switch (GetState()) {
       case Subscribed :
@@ -192,14 +190,17 @@ void SIPHandler::SetState(SIPHandler::State newState)
 
 bool SIPHandler::ActivateState(SIPHandler::State newState)
 {
+  if (GetState() == Unsubscribed)
+    return false;
+
   // If subscribing with zero expiry time, is same as unsubscribe
-  if (newState == Subscribing && expire == 0)
+  if (newState == Subscribing && GetExpire() == 0)
     newState = Unsubscribing;
 
   // If unsubscribing and never got a response from server, rather than trying
   // to send more unsuccessful packets, abort transactions and mark Unsubscribed
   if (newState == Unsubscribing && !m_receivedResponse) {
-    ShutDown();
+    SetState(Unsubscribed); // Allow garbage collection thread to clean up
     return true;
   }
 
@@ -256,13 +257,15 @@ bool SIPHandler::ActivateState(SIPHandler::State newState)
 
 PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
 {
-  expireTimer.Stop(false); // Stop automatic retry
+  m_expireTimer.Stop(false); // Stop automatic retry
 
   SetState(newState);
 
   if (GetTransport() == NULL)
     OnFailed(SIP_PDU::Local_BadTransportAddress);
   else {
+    m_lastCseq = 0;
+
     // Restoring or first time, try every interface
     if (newState == Restoring || m_transport->GetInterface().IsEmpty()) {
       PWaitAndSignal mutex(m_transport->GetWriteMutex());
@@ -284,7 +287,7 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
     return true;
   }
 
-  RetryLater(offlineExpire);
+  RetryLater(m_offlineExpireTime);
   return true;
 }
 
@@ -325,21 +328,21 @@ OpalTransport * SIPHandler::GetTransport()
 
 void SIPHandler::SetExpire(int e)
 {
-  expire = e;
-  PTRACE(3, "SIP\tExpiry time for " << GetMethod() << " set to " << expire << " seconds.");
+  m_currentExpireTime = e;
+  PTRACE(3, "SIP\tExpiry time for " << GetMethod() << " set to " << e << " seconds.");
 
-  // Only modify the originalExpire for future requests if IntervalTooBrief gives
+  // Only modify the m_originalExpireTime for future requests if IntervalTooBrief gives
   // a bigger expire time. expire itself will always reflect the proxy decision
-  // (bigger or lower), but originalExpire determines what is used in future 
+  // (bigger or lower), but m_originalExpireTime determines what is used in future 
   // requests and is only modified if interval too brief
-  if (originalExpire < e)
-    originalExpire = e;
+  if (m_originalExpireTime < e)
+    m_originalExpireTime = e;
 
   // retry before the expire time.
   // if the expire time is more than 20 mins, retry 10mins before expiry
   // if the expire time is less than 20 mins, retry after half of the expiry time
-  if (expire > 0 && GetState() < Unsubscribing)
-    expireTimer.SetInterval(0, (unsigned)(expire < 20*60 ? expire/2 : expire-10*60));
+  if (GetExpire() > 0 && GetState() < Unsubscribing)
+    m_expireTimer.SetInterval(0, (unsigned)(GetExpire() < 20*60 ? GetExpire()/2 : GetExpire()-10*60));
 }
 
 
@@ -349,23 +352,37 @@ PBoolean SIPHandler::WriteSIPHandler(OpalTransport & transport, void * param)
 }
 
 
-bool SIPHandler::WriteSIPHandler(OpalTransport & transport, bool /*forked*/)
+bool SIPHandler::WriteSIPHandler(OpalTransport & transport, bool forked)
 {
   SIPTransaction * transaction = CreateTransaction(transport);
+  if (transaction == NULL) {
+    PTRACE(2, "SIP\tCould not create transaction on " << transport);
+    return false;
+  }
 
-  if (transaction != NULL) {
-    for (PINDEX i = 0; i < m_mime.GetSize(); ++i) 
-      transaction->GetMIME().SetAt(m_mime.GetKeyAt(i), PString(m_mime.GetDataAt(i)));
-    if (GetState() == Unsubscribing)
-      transaction->GetMIME().SetExpires(0);
-    if (authentication != NULL) {
-      SIPAuthenticator auth(*transaction);
-      authentication->Authorise(auth); // If already have info from last time, use it!
-    }
-    if (transaction->Start()) {
-      m_transactions.Append(transaction);
-      return true;
-    }
+  SIPMIMEInfo & mime = transaction->GetMIME();
+
+  if (forked) {
+    if (m_lastCseq == 0)
+      m_lastCseq = mime.GetCSeqIndex();
+    else
+      transaction->SetCSeq(m_lastCseq);
+  }
+
+  for (PINDEX i = 0; i < m_mime.GetSize(); ++i) 
+    mime.SetAt(m_mime.GetKeyAt(i), PString(m_mime.GetDataAt(i)));
+
+  if (GetState() == Unsubscribing)
+    mime.SetExpires(0);
+
+  if (authentication != NULL) {
+    SIPAuthenticator auth(*transaction);
+    authentication->Authorise(auth); // If already have info from last time, use it!
+  }
+
+  if (transaction->Start()) {
+    m_transactions.Append(transaction);
+    return true;
   }
 
   PTRACE(2, "SIP\tDid not start transaction on " << transport);
@@ -435,7 +452,7 @@ void SIPHandler::OnReceivedIntervalTooBrief(SIPTransaction & /*transaction*/, SI
 void SIPHandler::OnReceivedTemporarilyUnavailable(SIPTransaction & /*transaction*/, SIP_PDU & response)
 {
   OnFailed(SIP_PDU::Failure_TemporarilyUnavailable);
-  RetryLater(response.GetMIME().GetInteger("Retry-After", offlineExpire));
+  RetryLater(response.GetMIME().GetInteger("Retry-After", m_offlineExpireTime));
 }
 
 
@@ -448,16 +465,6 @@ void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction,
 #endif
   PTRACE(3, "SIP\tReceived " << proxyTrace << "Authentication Required response");
   
-  // Abort after some unsuccesful authentication attempts. This is required since
-  // some implementations return "401 Unauthorized" with a different nonce at every
-  // time.
-  if (authenticationAttempts >= 10) {
-    PTRACE(1, "SIP\tAborting after " << authenticationAttempts << " attempts to REGISTER/SUBSCRIBE");
-    OnFailed(SIP_PDU::Failure_UnAuthorised);
-    return;
-  }
-  ++authenticationAttempts;
-
   // authenticate 
   PString errorMsg;
   SIPAuthentication * newAuth = PHTTPClientAuthentication::ParseAuthenticationRequired(isProxy, response.GetMIME(), errorMsg);
@@ -483,7 +490,7 @@ void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction,
         PTRACE(2, "SIP\tAuthentication not possible yet, no credentials available.");
         OnFailed(SIP_PDU::Failure_TemporarilyUnavailable);
         if (!transaction.IsCanceled())
-          RetryLater(offlineExpire);
+          RetryLater(m_offlineExpireTime);
         return;
       }
 
@@ -536,7 +543,7 @@ void SIPHandler::OnReceivedOK(SIPTransaction & /*transaction*/, SIP_PDU & respon
     case Subscribing :
     case Refreshing :
     case Restoring :
-      if (expire == 0)
+      if (GetExpire() == 0)
         SetState(Unsubscribed);
       else
         SetState(Subscribed);
@@ -545,9 +552,6 @@ void SIPHandler::OnReceivedOK(SIPTransaction & /*transaction*/, SIP_PDU & respon
     default :
       PTRACE(2, "SIP\tUnexpected 200 OK in handler with state " << GetState());
   }
-
-  // reset the number of unsuccesful authentication attempts
-  authenticationAttempts = 0;
 }
 
 
@@ -556,18 +560,18 @@ void SIPHandler::OnTransactionFailed(SIPTransaction & transaction)
   if (m_transactions.Remove(&transaction)) {
     OnFailed(transaction.GetStatusCode());
     if (!transaction.IsCanceled())
-      RetryLater(offlineExpire);
+      RetryLater(m_offlineExpireTime);
   }
 }
 
 
 void SIPHandler::RetryLater(unsigned after)
 {
-  if (after == 0 || expire == 0)
+  if (after == 0 || GetExpire() == 0)
     return;
 
   PTRACE(3, "SIP\tRetrying " << GetMethod() << " after " << after << " seconds.");
-  expireTimer.SetInterval(0, after); // Keep trying to get it back
+  m_expireTimer.SetInterval(0, after); // Keep trying to get it back
 }
 
 
@@ -593,10 +597,9 @@ void SIPHandler::OnFailed(SIP_PDU::StatusCodes code)
 
     default :
       PTRACE(4, "SIP\tNot retrying " << GetMethod() << " due to error response " << code);
-      expire = 0; // OK, stop trying
-      expireTimer.Stop(false);
-      SetState(Unsubscribed);
-      ShutDown();
+      m_currentExpireTime = 0; // OK, stop trying
+      m_expireTimer.Stop(false);
+      SetState(Unsubscribed); // Allow garbage collection thread to clean up
   }
 }
 
@@ -655,42 +658,37 @@ SIPTransaction * SIPRegisterHandler::CreateTransaction(OpalTransport & trans)
       if (m_contactAddresses.empty())
         params.m_contactAddress = "*";
       else {
-        for (SIPURLList::iterator contact = m_contactAddresses.begin(); contact != m_contactAddresses.end(); ++contact) {
+        for (SIPURLList::iterator contact = m_contactAddresses.begin(); contact != m_contactAddresses.end(); ++contact)
           contact->GetFieldParameters().Remove("expires");
-          if (!params.m_contactAddress.IsEmpty())
-            params.m_contactAddress += ", ";
-          params.m_contactAddress += contact->AsQuotedString();
-        }
+        params.m_contactAddress = m_contactAddresses.ToString();
       }
     }
   }
   else {
-    params.m_expire = expire;
+    params.m_expire = GetExpire();
 
     if (params.m_contactAddress.IsEmpty()) {
-      if (m_contactAddresses.empty()) {
+      if (GetState() != Refreshing || m_contactAddresses.empty()) {
         PString userName = SIPURL(params.m_addressOfRecord).GetUserName();
         OpalTransportAddressArray interfaces = endpoint.GetInterfaceAddresses(true, &trans);
         if (params.m_compatibility == SIPRegister::e_CannotRegisterMultipleContacts) {
           // If translated by STUN then only the external address of the interface is used.
           SIPURL contact(userName, interfaces[0]);
           contact.Sanitise(SIPURL::RegContactURI);
-          m_contactAddresses.push_back(contact);
           params.m_contactAddress += contact.AsQuotedString();
         }
         else {
-          OpalTransportAddress localAddress = trans.GetLocalAddress();
+          OpalTransportAddress localAddress = trans.GetLocalAddress(params.m_compatibility != SIPRegister::e_HasApplicationLayerGateway);
           unsigned qvalue = 1000;
           for (PINDEX i = 0; i < interfaces.GetSize(); ++i) {
             /* If fully compliant, put into the contact field all the bound
                interfaces. If special then we only put into the contact
                listeners that are on the same interface. If translated by STUN
                then only the external address of the interface is used. */
-            if (params.m_compatibility != SIPRegister::e_CannotRegisterPrivateContacts || localAddress.IsEquivalent(interfaces[i], true)) {
+            if (params.m_compatibility == SIPRegister::e_FullyCompliant || localAddress.IsEquivalent(interfaces[i], true)) {
               SIPURL contact(userName, interfaces[i]);
               contact.Sanitise(SIPURL::RegContactURI);
               contact.GetFieldParameters().Set("q", qvalue < 1000 ? psprintf("0.%03u", qvalue) : "1");
-              m_contactAddresses.push_back(contact);
 
               if (!params.m_contactAddress.IsEmpty())
                 params.m_contactAddress += ", ";
@@ -701,18 +699,8 @@ SIPTransaction * SIPRegisterHandler::CreateTransaction(OpalTransport & trans)
           }
         }
       }
-      else {
-        for (SIPURLList::iterator contact = m_contactAddresses.begin(); contact != m_contactAddresses.end(); ++contact) {
-          if (!params.m_contactAddress.IsEmpty())
-            params.m_contactAddress += ", ";
-          params.m_contactAddress += contact->AsQuotedString();
-        }
-      }
-    }
-    else {
-      if (!m_contactAddresses.FromString(params.m_contactAddress)) {
-        PTRACE(1, "SIP\tUser defined contact address has no SIP addresses in it!");
-      }
+      else
+        params.m_contactAddress = m_contactAddresses.ToString();
     }
   }
 
@@ -722,115 +710,110 @@ SIPTransaction * SIPRegisterHandler::CreateTransaction(OpalTransport & trans)
 
 void SIPRegisterHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
 {
-  State oldState = GetState();
+  State previousState = GetState();
+  switch (previousState) {
+    case Unsubscribing :
+      SetState(Unsubscribed);
+      SendStatus(SIP_PDU::Successful_OK, Unsubscribing);
+      return;
 
-  SIPHandler::OnReceivedOK(transaction, response);
+    case Subscribing :
+    case Refreshing :
+    case Restoring :
+      break;
 
-  if (GetState() == Unsubscribed || m_contactAddresses.empty()) {
-    SendStatus(SIP_PDU::Successful_OK, oldState);
-    return;
+    default :
+      PTRACE(2, "SIP\tUnexpected 200 OK in handler with state " << previousState);
+      return;
   }
 
   SIPMIMEInfo & mime = response.GetMIME();
 
+  mime.GetProductInfo(m_productInfo);
+
   m_serviceRoute.FromString(mime("Service-Route"));
 
-  /* See if we are behind NAT and the Via header rport was present. This is
-     a STUN free mechanism for working behind firewalls. */
-  OpalTransportAddress externalAddress = mime.GetViaReceivedAddress();
-  bool useExternalAddress = !externalAddress.IsEmpty();
+  SIPURLList requestedContacts;
+  transaction.GetMIME().GetContacts(requestedContacts);
 
   SIPURLList replyContacts;
   mime.GetContacts(replyContacts);
 
+  /* See if we are behind NAT and the Via header rport was present. This is
+     a STUN free mechanism for working behind firewalls. Also, some servers
+     will refuse to work unless the Contact header agrees whith where the
+     packet is physically coming from. */
+  OpalTransportAddress externalAddress;
+  if (m_parameters.m_compatibility != SIPRegister::e_HasApplicationLayerGateway)
+    externalAddress = mime.GetViaReceivedAddress();
+
   /* RFC3261 does say the registrar must send back ALL registrations, however
      we only deal with replies from the registrar that we actually requested.
 
      Except! We look for special condition where registrar (e.g. OpenSIPS)
      tries to be smart and adjusts the Contact field to be the external
-     address as per the via rport. IMHO this is a bug as it does not follow
+     address as per the Via header rport. IMHO this is a bug as it does not follow
      what is implied by RFC3261/10.2.4 and 10.3 step 7. It should include the
-     original as well as the extra Contact. But it doesn't, so we have to deal. */
-  SIPURLList contacts;
-  for (SIPURLList::iterator reply = replyContacts.begin(); reply != replyContacts.end(); ++reply) {
-    SIPURLList::iterator request = std::find(m_contactAddresses.begin(), m_contactAddresses.end(), *reply);
-    if (request != m_contactAddresses.end() ||
-        (useExternalAddress && reply->GetHostAddress() == externalAddress))
-      contacts.push_back(*reply);
+     original as well as the extra Contact. But it doesn't, so we have to deal.
+     */
+  for (SIPURLList::iterator reply = replyContacts.begin(); reply != replyContacts.end(); ) {
+    if (reply->GetHostAddress() == externalAddress) {
+      externalAddress.MakeEmpty(); // Clear this so no further action taken
+      ++reply;
+    }
+    else if (std::find(requestedContacts.begin(), requestedContacts.end(), *reply) != requestedContacts.end())
+      ++reply;
+    else
+      replyContacts.erase(reply++);
   }
 
-  if (contacts.empty()) {
-    if (transaction.GetMIME().GetExpires() != 0) {
-      // Huh? Nothing we tried to register, registered! How is that possible?
-      PTRACE(2, "SIP\tREGISTER returned no Contacts we requested, unregistering.");
-      SendRequest(Unsubscribing);
-      return;
-    }
-
-    // We did a partial unsubscribe but registrar doesn't understand and
-    // unregisetered everthing, reregister
-    SIPURLList::iterator contact = m_contactAddresses.begin();
-    while (contact != m_contactAddresses.end()) {
-      if (contact->GetFieldParameters().GetInteger("expires") > 0)
-        ++contact;
-      else
-        m_contactAddresses.erase(contact++);
-    }
-
-    PTRACE(2, "SIP\tREGISTER only had some contacts to unregister but registrar to dumb, re-registering.");
-    SendRequest(Subscribing);
+  if (replyContacts.empty() && GetExpire() != 0) {
+    // Huh? Nothing we tried to register, registered! How is that possible?
+    PTRACE(2, "SIP\tREGISTER returned no Contact addresses we requested, not really registered.");
+    SendRequest(Unsubscribing);
+    SendStatus(SIP_PDU::GlobalFailure_NotAcceptable, previousState);
+    return;
   }
 
-  // See if we are behind NAT and the Via header rport was present, and different
-  // to our registered contact field. Some servers will refuse to work unless the
-  // Contact agrees whith where the packet is physically coming from.
-  if (useExternalAddress) {
-    for (SIPURLList::iterator contact = contacts.begin(); contact != contacts.end(); ++contact) {
-      if (contact->GetHostAddress() == externalAddress) {
-        useExternalAddress = false;
-        break;
-      }
-    }
-  }
-
-  if (useExternalAddress) {
-    SIPURLList newContacts;
-
-    for (SIPURLList::iterator contact = contacts.begin(); contact != contacts.end(); ++contact) {
-      if (contact->GetHostAddress().GetProto() == "udp") {
-        contact->GetFieldParameters().Remove("expires");
-
-        SIPURL newContact(contact->GetUserName(), externalAddress);
-        newContact.GetFieldParameters().SetInteger("expires", originalExpire);
-        newContacts.push_back(newContact);
-      }
-    }
-
-    contacts.insert(contacts.end(), newContacts.begin(), newContacts.end());
-    SetExpire(0);
-  }
-  else {
+  // If this is the final (possibly one and only) REGISTER, process it
+  if (m_externalAddress == externalAddress) {
     int minExpiry = INT_MAX;
-    for (SIPURLList::iterator contact = contacts.begin(); contact != contacts.end(); ++contact) {
+    for (SIPURLList::iterator contact = replyContacts.begin(); contact != replyContacts.end(); ++contact) {
       long expires = contact->GetFieldParameters().GetInteger("expires",
                           mime.GetExpires(endpoint.GetRegistrarTimeToLive().GetSeconds()));
-      if (minExpiry > expires)
+      if (expires > 0 && minExpiry > expires)
         minExpiry = expires;
     }
     SetExpire(minExpiry);
+
+    m_contactAddresses = replyContacts;
+    SetState(Subscribed);
+    SendStatus(SIP_PDU::Successful_OK, previousState);
+    return;
   }
 
-  m_contactAddresses = contacts;
+  // Remember (possibly new) NAT address
+  m_externalAddress == externalAddress;
 
-  mime.GetProductInfo(m_productInfo);
-
-  SendStatus(SIP_PDU::Successful_OK, oldState);
-
-  // If we have had a NAT port change to what we thought, need to initiate another REGISTER
-  if (useExternalAddress) {
+  if (GetExpire() == 0) {
+    // If we had discovered we are behind NAT and had unregistered, re-REGISTER with new addresses
+    PTRACE(2, "SIP\tRe-registering with NAT address " << externalAddress);
+    for (SIPURLList::iterator contact = m_contactAddresses.begin(); contact != m_contactAddresses.end(); ++contact)
+      contact->SetHostAddress(externalAddress);
+    SetExpire(m_originalExpireTime);
+  }
+  else {
+    /* If we got here then we have done a successful register, but registrar indicated
+       that we are behind firewall. Unregister what we just registered */
     PTRACE(2, "SIP\tRemote indicated change of REGISTER Contact header required due to NAT");
-    SendRequest(GetState());
+    for (SIPURLList::iterator contact = replyContacts.begin(); contact != replyContacts.end(); ++contact)
+      contact->GetFieldParameters().Remove("expires");
+    m_contactAddresses = replyContacts;
+    SetExpire(0);
   }
+
+  SendRequest(previousState);
+  SendStatus(SIP_PDU::Information_Trying, previousState);
 }
 
 
@@ -916,7 +899,7 @@ SIPSubscribeHandler::SIPSubscribeHandler(SIPEndPoint & endpoint, const SIPSubscr
   , m_packageHandler(SIPEventPackageFactory::CreateInstance(params.m_eventPackage))
   , m_previousResponse(NULL)
 {
-  callID = m_dialog.GetCallID();
+  m_callID = m_dialog.GetCallID();
 
   m_parameters.m_proxyAddress = m_proxy.AsString();
 
@@ -951,7 +934,7 @@ SIPTransaction * SIPSubscribeHandler::CreateTransaction(OpalTransport & trans)
     m_dialog.SetProxy(m_proxy, true);
   }
 
-  m_parameters.m_expire = GetState() != Unsubscribing ? expire : 0;
+  m_parameters.m_expire = GetState() != Unsubscribing ? GetExpire() : 0;
   return new SIPSubscribe(endpoint, trans, m_dialog, m_parameters);
 }
 
@@ -984,7 +967,7 @@ PBoolean SIPSubscribeHandler::SendRequest(SIPHandler::State s)
 bool SIPSubscribeHandler::WriteSIPHandler(OpalTransport & transport, bool forked)
 {
   m_dialog.SetForking(forked);
-  bool ok = SIPHandler::WriteSIPHandler(transport, forked);
+  bool ok = SIPHandler::WriteSIPHandler(transport, false);
   m_dialog.SetForking(false);
   return ok;
 }
@@ -1065,7 +1048,7 @@ void SIPSubscribeHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & r
    * for SUBSCRIBE. RFC3265, 3.1.1.
    * An answer can only shorten the expires time.
    */
-  SetExpire(response.GetMIME().GetExpires(originalExpire));
+  SetExpire(response.GetMIME().GetExpires(m_originalExpireTime));
   m_dialog.Update(*m_transport, response);
 
   if (GetState() != Unsubscribing)
@@ -1092,7 +1075,10 @@ PBoolean SIPSubscribeHandler::OnReceivedNOTIFY(SIP_PDU & request)
     if (m_previousResponse == NULL)
       return request.SendResponse(*m_transport, SIP_PDU::Failure_InternalServerError, &endpoint);
 
-    PTRACE(3, "SIP\tReceived duplicate NOTIFY");
+    /* Make sure our repeat response has the CSeq/Via etc of this request, due
+       to retries at various protocol layers we can have old and even older
+       NOTIFY packets still coming in requiring matching responses. */
+    m_previousResponse->InitialiseHeaders(request);
     return request.SendResponse(*m_transport, *m_previousResponse, &endpoint);
   }
 
@@ -1113,12 +1099,22 @@ PBoolean SIPSubscribeHandler::OnReceivedNOTIFY(SIP_PDU & request)
     PTRACE(3, "SIP\tSubscription is terminated, state=" << GetState());
     m_previousResponse->SetStatusCode(SIP_PDU::Successful_OK);
     request.SendResponse(*m_transport, *m_previousResponse, &endpoint);
-    if (GetState() != Unsubscribing)
-      ShutDown();
-    else {
-      SetState(Unsubscribed);
-      SendStatus(SIP_PDU::Successful_OK, Unsubscribing);
+
+    switch (GetState()) {
+      case Unsubscribed :
+        break;
+
+      case Subscribed :
+      case Unavailable :
+        SendRequest(Unsubscribing);
+        break;
+
+      default :
+        SetState(Unsubscribed); // Allow garbage collection thread to clean up
+        SendStatus(SIP_PDU::Successful_OK, Unsubscribing);
+        break;
     }
+
     return true;
   }
 
@@ -1303,26 +1299,24 @@ class SIPMwiEventPackageHandler : public SIPEventPackageHandler
       { "text-message",       OpalManager::TextMessageWaiting       },
       { "none",               OpalManager::NoMessageWaiting         }
     };
-    PString msgs;
-    PStringArray bodylines = body.Lines ();
+
+    PMIMEInfo info(request.GetEntityBody());
+
+    PString account = info.Get("Message-Account", handler.GetAddressOfRecord().AsString());
+
+    bool nothingSent = true;
     for (PINDEX z = 0 ; z < PARRAYSIZE(validMessageClasses); z++) {
-
-      for (int i = 0 ; i < bodylines.GetSize () ; i++) {
-
-        PCaselessString line (bodylines [i]);
-        PINDEX j = line.FindLast(validMessageClasses[z].name);
-        if (j != P_MAX_INDEX) {
-          line.Replace(validMessageClasses[z].name, "");
-          line.Replace (":", "");
-          msgs = line.Trim ();
-          handler.GetEndPoint().OnMWIReceived(handler.GetAddressOfRecord().AsString(), validMessageClasses[z].type, msgs);
-          return true;
-        }
+      if (info.Contains(validMessageClasses[z].name)) {
+        handler.GetEndPoint().OnMWIReceived(account, validMessageClasses[z].type, info[validMessageClasses[z].name]);
+        nothingSent = false;
       }
     }
 
-    // Received MWI, unknown messages number
-    handler.GetEndPoint().OnMWIReceived(handler.GetAddressOfRecord().AsString(), OpalManager::NumMessageWaitingTypes, "1/0");
+    // Received MWI, but not counts, must be old form. Also make sure we send a
+    // lower case yes/no to call back function, regardless of what comes down the wire.
+    if (nothingSent)
+      handler.GetEndPoint().OnMWIReceived(account, OpalManager::NumMessageWaitingTypes,
+                                 (info.Get("Message-Waiting") *= "yes") ? "yes" : "no");
 
     return true;
   }
@@ -1362,7 +1356,10 @@ class SIPPresenceEventPackageHandler : public SIPEventPackageHandler
       infoList.resize(1);
     else {
       PString error;
-      if (!SIPPresenceInfo::ParseXML(request.GetEntityBody(), infoList, error))
+      PString body = request.GetEntityBody();
+      if (handler.GetProductInfo().name.Find("Asterisk") != P_MAX_INDEX)
+        body.Replace(to.AsString(), from.AsString());
+      if (!SIPPresenceInfo::ParseXML(body, infoList, error))
         return false;
     }
 
@@ -1645,7 +1642,7 @@ SIPNotifyHandler::SIPNotifyHandler(SIPEndPoint & endpoint,
   , m_reason(Deactivated)
   , m_packageHandler(SIPEventPackageFactory::CreateInstance(eventPackage))
 {
-  callID = m_dialog.GetCallID();
+  m_callID = m_dialog.GetCallID();
 }
 
 
@@ -1658,8 +1655,8 @@ SIPNotifyHandler::~SIPNotifyHandler()
 SIPTransaction * SIPNotifyHandler::CreateTransaction(OpalTransport & trans)
 {
   PString state;
-  if (expire > 0 && GetState() != Unsubscribing)
-    state.sprintf("active;expires=%u", expire);
+  if (GetExpire() > 0 && GetState() != Unsubscribing)
+    state.sprintf("active;expires=%u", GetExpire());
   else {
     state = "terminated;reason=";
     static const char * const ReasonNames[] = {
@@ -1690,7 +1687,7 @@ PBoolean SIPNotifyHandler::SendRequest(SIPHandler::State state)
 bool SIPNotifyHandler::WriteSIPHandler(OpalTransport & transport, bool forked)
 {
   m_dialog.SetForking(forked);
-  bool ok = SIPHandler::WriteSIPHandler(transport, forked);
+  bool ok = SIPHandler::WriteSIPHandler(transport, false);
   m_dialog.SetForking(false);
   return ok;
 }
@@ -1735,7 +1732,7 @@ SIPTransaction * SIPPublishHandler::CreateTransaction(OpalTransport & transport)
   if (GetState() == Unsubscribing)
     return NULL;
 
-  m_parameters.m_expire = expire;
+  m_parameters.m_expire = m_originalExpireTime;
   return new SIPPublish(endpoint,
                         transport,
                         GetCallID(),
@@ -1752,7 +1749,7 @@ void SIPPublishHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & res
   if (!newETag.IsEmpty())
     m_sipETag = newETag;
 
-  SetExpire(response.GetMIME().GetExpires(originalExpire));
+  SetExpire(response.GetMIME().GetExpires(m_originalExpireTime));
 
   SIPHandler::OnReceivedOK(transaction, response);
 }
@@ -1976,6 +1973,8 @@ bool SIPPresenceInfo::ParseXML(const PString & body,
   SIPPresenceInfo info;
   info.m_tupleId.MakeEmpty();
 
+  PTime defaultTimestamp; // Start with "now"
+
   for (PINDEX idx = 0; idx < rootElement->GetSize(); ++idx) {
     PXMLElement * element = dynamic_cast<PXMLElement *>(rootElement->GetElement(idx));
     if (element == NULL)
@@ -1986,6 +1985,7 @@ bool SIPPresenceInfo::ParseXML(const PString & body,
 
       if (!info.m_tupleId.IsEmpty()) {
         infoList.push_back(info);
+        defaultTimestamp = info.m_when - PTimeInterval(0, 1); // One second older
         info = SIPPresenceInfo();
       }
 
@@ -2010,7 +2010,7 @@ bool SIPPresenceInfo::ParseXML(const PString & body,
         info.m_contact = element->GetData();
 
       if ((element = tupleElement->GetElement("timestamp")) == NULL || !info.m_when.Parse(element->GetData()))
-        info.m_when.SetTimestamp(0);
+        info.m_when = defaultTimestamp;
     }
     else if (element->GetName() == "urn:ietf:params:xml:ns:pidf:data-model|person") {
       static PConstCaselessString rpid("urn:ietf:params:xml:ns:pidf:rpid|");
@@ -2057,9 +2057,9 @@ SIPMessageHandler::SIPMessageHandler(SIPEndPoint & endpoint, const SIPMessage::P
   if (params.m_id.IsEmpty())
     m_parameters.m_id = GetCallID();
   else
-    callID = params.m_id;   // make sure the transation uses the conversation ID, so we can track it
+    m_callID = params.m_id;   // make sure the transation uses the conversation ID, so we can track it
 
-  offlineExpire = 0; // No retries for offline, just give up
+  m_offlineExpireTime = 0; // No retries for offline, just give up
 
   SetState(Subscribed);
 }
@@ -2076,7 +2076,7 @@ SIPTransaction * SIPMessageHandler::CreateTransaction(OpalTransport & transport)
     return NULL;
   }
 
-  SetExpire(originalExpire);
+  SetExpire(m_originalExpireTime);
 
   SIPMessage * msg = new SIPMessage(endpoint, transport, m_parameters);
   m_parameters.m_localAddress = msg->GetLocalAddress().AsString();
@@ -2123,7 +2123,7 @@ SIPOptionsHandler::SIPOptionsHandler(SIPEndPoint & endpoint, const SIPOptions::P
 {
   m_parameters.m_proxyAddress = m_proxy.AsString();
 
-  offlineExpire = 0; // No retries for offline, just give up
+  m_offlineExpireTime = 0; // No retries for offline, just give up
 
   SetState(Subscribed);
 }
@@ -2134,7 +2134,7 @@ SIPTransaction * SIPOptionsHandler::CreateTransaction(OpalTransport & transport)
   if (GetState() == Unsubscribing)
     return NULL;
 
-  return new SIPOptions(endpoint, transport, callID, m_parameters);
+  return new SIPOptions(endpoint, transport, GetCallID(), m_parameters);
 }
 
 
@@ -2361,27 +2361,8 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PURL & aor, SIP_
 }
 
 
-/**
- * Find the SIPHandler object with the specified authRealm
- */
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm(const PString & authRealm, const PString & userName, PSafetyMode mode)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm(const PString & authRealm, PSafetyMode mode)
 {
-  PSafePtr<SIPHandler> ptr;
-
-  if (!userName.IsEmpty()) {
-    // look for a match to exact user name and realm
-    if ((ptr = FindBy(m_byAuthIdAndRealm, userName + '\n' + authRealm, mode)) != NULL) {
-      PTRACE(4, "SIP\tLocated existing credentials for ID \"" << userName << "\" at realm \"" << authRealm << '"');
-      return ptr;
-    }
-
-    // look for a match to exact user name and realm
-    if ((ptr = FindBy(m_byAorUserAndRealm, userName + '\n' + authRealm, mode)) != NULL) {
-      PTRACE(4, "SIP\tLocated existing credentials for ID \"" << userName << "\" at realm \"" << authRealm << '"');
-      return ptr;
-    }
-  }
-
   // look for a match to realm without users
   for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
     if (handler->GetRealm() == authRealm && handler.SetSafetyMode(mode)) {
@@ -2394,6 +2375,26 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm(const PString &
 }
 
 
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm(const PString & authRealm, const PString & userName, PSafetyMode mode)
+{
+  PSafePtr<SIPHandler> ptr;
+
+  // look for a match to exact user name and realm
+  if ((ptr = FindBy(m_byAuthIdAndRealm, userName + '\n' + authRealm, mode)) != NULL) {
+    PTRACE(4, "SIP\tLocated existing credentials for ID \"" << userName << "\" at realm \"" << authRealm << '"');
+    return ptr;
+  }
+
+  // look for a match to exact user name and realm
+  if ((ptr = FindBy(m_byAorUserAndRealm, userName + '\n' + authRealm, mode)) != NULL) {
+    PTRACE(4, "SIP\tLocated existing credentials for ID \"" << userName << "\" at realm \"" << authRealm << '"');
+    return ptr;
+  }
+
+  return NULL;
+}
+
+
 /**
  * Find the SIPHandler object with the specified registration host.
  * For example, in the above case, the name parameter
