@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26330 $
+ * $Revision: 26951 $
  * $Author: rjongbloed $
- * $Date: 2011-08-19 07:16:29 -0500 (Fri, 19 Aug 2011) $
+ * $Date: 2012-02-07 21:15:47 -0600 (Tue, 07 Feb 2012) $
  */
 
 #include <ptlib.h>
@@ -67,6 +67,7 @@ SIPEndPoint::SIPEndPoint(OpalManager & mgr,
   , registrarTimeToLive(0, 0, 0, 1)  // 1 hour
   , notifierTimeToLive(0, 0, 0, 1)   // 1 hour
   , natBindingTimeout(0, 0, 1)       // 1 minute
+  , m_registeredUserMode(false)
   , m_shuttingDown(false)
   , m_defaultAppearanceCode(-1)
   , m_connectionThreadPool(maxConnectionThreads)
@@ -175,7 +176,7 @@ PBoolean SIPEndPoint::NewIncomingConnection(OpalTransport * transport)
     for (PSafePtr<OpalConnection> connection(connectionsActive, PSafeReference); connection != NULL; ++connection) {
       PSafePtr<SIPConnection> sipConnection = PSafePtrCast<OpalConnection, SIPConnection>(connection);
       if (&sipConnection->GetTransport() == transport && sipConnection->LockReadWrite()) {
-        sipConnection->SetTransport(NULL);
+        sipConnection->SetTransport(SIPURL());
         sipConnection->UnlockReadWrite();
       }
     }
@@ -862,7 +863,7 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   if (transport.IsReliable())
     newTransport = &transport;
   else {
-    newTransport = CreateTransport(SIPURL("", transport.GetRemoteAddress(), 0), transport.GetInterface());
+    newTransport = CreateTransport(SIPURL(PString::Empty(), transport.GetRemoteAddress(), 0), transport.GetInterface());
     if (newTransport == NULL) {
       PTRACE(1, "SIP\tFailed to create transport for SIPConnection for INVITE for " << request->GetURI() << " to " << toAddr);
       request->SendResponse(transport, SIP_PDU::Failure_NotFound, this);
@@ -883,7 +884,7 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   SIPConnection *connection = CreateConnection(*call,
                                                SIPURL::GenerateTag(),
                                                NULL,
-                                               PString::Empty(),
+                                               SIPURL(),
                                                newTransport,
                                                request);
   if (!AddConnection(connection)) {
@@ -952,7 +953,20 @@ bool SIPEndPoint::OnReceivedMESSAGE(OpalTransport & transport, SIP_PDU & pdu)
   if (!m_onConnectionlessMessage.IsNULL()) {
     ConnectionlessMessageInfo info(transport, pdu);
     m_onConnectionlessMessage(*this, info);
-    return info.m_status;
+    switch (info.m_status) {
+      case ConnectionlessMessageInfo::MethodNotAllowed :
+        return false;
+
+      case ConnectionlessMessageInfo::SendOK :
+        pdu.SendResponse(transport, SIP_PDU::Successful_OK, this);
+        // Do next case
+
+      case ConnectionlessMessageInfo::ResponseSent :
+        return true;
+
+      default :
+        break;
+    }
   }
 
   SIPMIMEInfo & mime  = pdu.GetMIME();
@@ -1047,6 +1061,13 @@ bool SIPEndPoint::Register(const PString & host,
 
 
 bool SIPEndPoint::Register(const SIPRegister::Params & newParams, PString & aor, bool asynchronous)
+{
+  SIP_PDU::StatusCodes reason;
+  return Register(newParams, aor, asynchronous ? NULL : &reason);
+}
+
+
+bool SIPEndPoint::Register(const SIPRegister::Params & newParams, PString & aor, SIP_PDU::StatusCodes * reason)
 {
   PTRACE(4, "SIP\tStart REGISTER\n" << newParams);
 
@@ -1070,10 +1091,11 @@ bool SIPEndPoint::Register(const SIPRegister::Params & newParams, PString & aor,
   if (!handler->ActivateState(SIPHandler::Subscribing))
     return false;
 
-  if (asynchronous)
+  if (reason == NULL)
     return true;
 
-  m_registrationComplete[aor].Wait();
+  m_registrationComplete[aor].m_sync.Wait();
+  *reason = m_registrationComplete[aor].m_reason;
   m_registrationComplete.erase(aor);
   return handler->GetState() == SIPHandler::Subscribed;
 }
@@ -1134,13 +1156,14 @@ void SIPEndPoint::OnRegistrationStatus(const RegistrationStatus & status)
 
   if (!status.m_wasRegistering ||
        status.m_reRegistering ||
-       status.m_reason == SIP_PDU::Information_Trying ||
-       status.m_reason == SIP_PDU::Failure_UnAuthorised)
+       status.m_reason == SIP_PDU::Information_Trying)
     return;
 
-  std::map<PString, PSyncPoint>::iterator it = m_registrationComplete.find(status.m_addressofRecord);
-  if (it != m_registrationComplete.end())
-    it->second.Signal();
+  std::map<PString, RegistrationCompletion>::iterator it = m_registrationComplete.find(status.m_addressofRecord);
+  if (it != m_registrationComplete.end()) {
+    it->second.m_reason = status.m_reason;
+    it->second.m_sync.Signal();
+  }
 }
 
 
@@ -1638,7 +1661,16 @@ unsigned SIPEndPoint::GetAllowedMethods() const
 bool SIPEndPoint::GetAuthentication(const PString & realm, PString & user, PString & password) 
 {
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(realm, user, PSafeReadOnly);
-  if (handler == NULL || handler->GetPassword().IsEmpty())
+  if (handler == NULL) {
+    if (m_registeredUserMode)
+      return false;
+
+    handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(realm, PSafeReadOnly);
+    if (handler == NULL)
+      return false;
+  }
+
+  if (handler->GetPassword().IsEmpty())
     return false;
 
   // really just after password, but username MAY change too.
@@ -1655,12 +1687,21 @@ SIPURL SIPEndPoint::GetRegisteredProxy(const SIPURL & url)
 
   if (url.GetScheme() != "tel") {
     // Look up by the full URL first in case of multiple registrations to the same domain.
-    if ((handler = activeSIPHandlers.FindSIPHandlerByUrl(url.AsString(), SIP_PDU::Method_REGISTER, PSafeReadOnly)) == NULL)
+    if ((handler = activeSIPHandlers.FindSIPHandlerByUrl(url.AsString(), SIP_PDU::Method_REGISTER, PSafeReadOnly)) == NULL) {
+      if (m_registeredUserMode)
+        return SIPURL();
+
       // Precise AOR not found, locate the name used for the domain.
-      handler = activeSIPHandlers.FindSIPHandlerByDomain(url.GetHostName(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
-    return handler != NULL ? handler->GetProxy() : SIPURL();
+      if ((handler = activeSIPHandlers.FindSIPHandlerByDomain(url.GetHostName(), SIP_PDU::Method_REGISTER, PSafeReadOnly)) == NULL)
+        return SIPURL();
+    }
+
+    return handler->GetProxy();
   }
 
+  // Can't do tel URI if in this mode
+  if (m_registeredUserMode)
+    return SIPURL();
 
   PString domain = url.GetHostName();
   if (domain.IsEmpty() || OpalIsE164(domain)) {
@@ -1687,6 +1728,9 @@ SIPURL SIPEndPoint::GetRegisteredPartyName(const SIPURL & url, const OpalTranspo
   // Look up by the full URL first in case of multiple registrations to the same domain.
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(url, SIP_PDU::Method_REGISTER, PSafeReadOnly);
   if (handler == NULL) {
+    if (m_registeredUserMode)
+      return GetDefaultRegisteredPartyName(transport);
+
     // Precise AOR not found, locate the name used for the domain.
     handler = activeSIPHandlers.FindSIPHandlerByDomain(url.GetHostName(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
     if (handler == NULL) 
@@ -1738,7 +1782,7 @@ SIPURL SIPEndPoint::GetDefaultRegisteredPartyName(const OpalTransport & transpor
     rpn = SIPURL(defPartyName, addr, myPort);
   else {
     rpn = SIPURL(defPartyName.Left(pos), addr, myPort);   // set transport from address
-    rpn.SetHostName(defPartyName.Right(pos+1));
+    rpn.SetHostName(defPartyName.Mid(pos+1));
   }
 
   rpn.SetDisplayName(GetDefaultDisplayName());
@@ -1747,7 +1791,9 @@ SIPURL SIPEndPoint::GetDefaultRegisteredPartyName(const OpalTransport & transpor
 }
 
 
-void SIPEndPoint::AdjustToRegistration(const OpalTransport &transport, SIP_PDU & pdu)
+void SIPEndPoint::AdjustToRegistration(SIP_PDU & pdu,
+                                       const OpalTransport & transport,
+                                       const SIPConnection * connection)
 {
   // When we start adding things like P-Asserted-Identity, this function will change
   // Right now it just does the Contact field
@@ -1761,6 +1807,10 @@ void SIPEndPoint::AdjustToRegistration(const OpalTransport &transport, SIP_PDU &
   if (pdu.GetMethod() == SIP_PDU::NumMethods) {
     user   = to.GetUserName();
     domain = from.GetHostName();
+    if (connection != NULL && to.GetDisplayName() != connection->GetDisplayName()) {
+      to.SetDisplayName(connection->GetDisplayName());
+      mime.SetTo(to.AsQuotedString());
+    }
   }
   else {
     user   = from.GetUserName();
@@ -1770,14 +1820,14 @@ void SIPEndPoint::AdjustToRegistration(const OpalTransport &transport, SIP_PDU &
   const SIPRegisterHandler * registrar = NULL;
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl("sip:"+user+'@'+domain, SIP_PDU::Method_REGISTER, PSafeReadOnly);
   // If precise AOR not found, locate the name used for the domain.
-  if (handler == NULL)
+  if (handler == NULL && !m_registeredUserMode)
     handler = activeSIPHandlers.FindSIPHandlerByDomain(domain, SIP_PDU::Method_REGISTER, PSafeReadOnly);
   if (handler != NULL) {
     registrar = dynamic_cast<const SIPRegisterHandler *>(&*handler);
     PAssertNULL(registrar);
   }
 
-  if (!mime.Has("Contact")) {
+  if (!mime.Has("Contact") && pdu.GetStatusCode() != SIP_PDU::Information_Trying) {
     SIPURL contact;
     if (registrar != NULL) {
       PIPSocket::Address ip;
@@ -1796,10 +1846,14 @@ void SIPEndPoint::AdjustToRegistration(const OpalTransport &transport, SIP_PDU &
     }
 
     if (contact.IsEmpty())
-      contact = GetLocalURL(transport, user);
+      contact = GetLocalURL(transport, connection != NULL ? connection->GetLocalPartyName() : user);
 
     contact.Sanitise(SIPURL::ContactURI);
-    pdu.GetMIME().SetContact(contact.AsQuotedString());
+
+    if (connection != NULL)
+      contact.SetDisplayName(connection->GetDisplayName());
+
+    mime.SetContact(contact.AsQuotedString());
   }
 
   if (!mime.Has("Route") && registrar != NULL)
