@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 24162 $
+ * $Revision: 26159 $
  * $Author: rjongbloed $
- * $Date: 2010-03-29 02:46:26 -0500 (Mon, 29 Mar 2010) $
+ * $Date: 2011-07-10 22:02:39 -0500 (Sun, 10 Jul 2011) $
  */
 
 #include <ptlib.h>
@@ -52,7 +52,7 @@ OPAL_DEFINE_MEDIA_COMMAND(OpalFaxTerminate, PLUGINCODEC_CONTROL_TERMINATE_CODEC)
 #define new PNEW
 
 
-static const char TIFF_File_FormatName[] = "TIFF-File";
+static const char TIFF_File_FormatName[] = OPAL_FAX_TIFF_FILE;
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -70,8 +70,22 @@ class OpalFaxMediaStream : public OpalNullMediaStream
       m_isAudio = true; // Even though we are not REALLY audio, act like we are
     }
 
+    bool Close()
+    {
+      if (isOpen &&
+          m_connection.m_state == OpalFaxConnection::e_CompletedSwitch &&
+          m_connection.m_finalStatistics.m_fax.m_result < 0) {
+        if (mediaPatch != NULL)
+          mediaPatch->ExecuteCommand(OpalFaxTerminate(), false);
+        GetStatistics(m_connection.m_finalStatistics);
+        PTRACE(4, "FAX\tGot final statistics: result=" << m_connection.m_finalStatistics.m_fax.m_result);
+      }
+
+      return OpalNullMediaStream::Close();
+    }
+
   private:
-    OpalFaxConnection      & m_connection;
+    OpalFaxConnection & m_connection;
 };
 
 
@@ -138,21 +152,19 @@ class T38PseudoRTP_Handler : public RTP_Encoding
               }
             }
 
-            PTRACE(2, "T38_UDPTL\tIgnored redundancy \"" << value[i] << "\"");
+            PTRACE(2, "T38_UDPTL\tIgnored redundancy \"" << value[i] << '"');
           }
 
 #if PTRACING
           if (PTrace::CanTrace(3)) {
-            PStringStream s;
-
-            for (std::map<int, int>::iterator i = m_redundancy.begin() ; i != m_redundancy.end() ; i++) {
-              if (!s.IsEmpty())
-                s << ",";
-
-              s << i->first << ":" << i->second;
+            ostream & trace = PTrace::Begin(3, __FILE__, __LINE__);
+            trace << "T38_UDPTL\tUse redundancy \"";
+            for (std::map<int, int>::iterator it = m_redundancy.begin() ; it != m_redundancy.end() ; it++) {
+              if (it != m_redundancy.begin())
+                trace << ",";
+              trace << it->first << ':' << it->second;
             }
-
-            PTRACE(3, "T38_UDPTL\tUse redundancy \"" << s << "\"");
+            trace << '"' << PTrace::End;
           }
 #endif
         }
@@ -179,7 +191,7 @@ class T38PseudoRTP_Handler : public RTP_Encoding
           PTRACE(3, "T38_UDPTL\tUse optimise on retransmit - " << (m_optimiseOnRetransmit ? "true" : "false"));
         }
         else {
-          PTRACE(4, "T38_UDPTL\tIgnored option " << key << " = \"" << stringOptions.GetDataAt(i) << "\"");
+          PTRACE(4, "T38_UDPTL\tIgnored option " << key << " = \"" << stringOptions.GetDataAt(i) << '"');
         }
       }
     }
@@ -242,10 +254,10 @@ class T38PseudoRTP_Handler : public RTP_Encoding
 
       int redundancy = 0;
 
-      for (std::map<int, int>::iterator i = m_redundancy.begin() ; i != m_redundancy.end() ; i++) {
-        if (plLen <= i->first) {
-          if (redundancy < i->second)
-            redundancy = i->second;
+      for (std::map<int, int>::iterator it = m_redundancy.begin() ; it != m_redundancy.end() ; it++) {
+        if (plLen <= it->first) {
+          if (redundancy < it->second)
+            redundancy = it->second;
 
           break;
         }
@@ -390,18 +402,18 @@ class T38PseudoRTP_Handler : public RTP_Encoding
       if (!m_receivedPacket.Decode(rawData)) {
   #if PTRACING
         if (m_oneGoodPacket)
-          PTRACE(2, "RTP_T38\tRaw data decode failure:\n  "
+          PTRACE(2, "T38_UDPTL\tRaw data decode failure:\n  "
                  << setprecision(2) << rawData << "\n  UDPTL = "
                  << setprecision(2) << m_receivedPacket);
         else
-          PTRACE(2, "RTP_T38\tRaw data decode failure: " << rawData.GetSize() << " bytes.");
+          PTRACE(2, "T38_UDPTL\tRaw data decode failure: " << rawData.GetSize() << " bytes.");
   #endif
 
         m_consecutiveBadPackets++;
-        if (m_consecutiveBadPackets < 100)
+        if (m_consecutiveBadPackets < 1000)
           return RTP_Session::e_IgnorePacket;
 
-        PTRACE(1, "RTP_T38\tRaw data decode failed 100 times, remote probably not switched from audio, aborting!");
+        PTRACE(1, "T38_UDPTL\tRaw data decode failed 1000 times, remote probably not switched from audio, aborting!");
         return RTP_Session::e_AbortTransport;
       }
 
@@ -456,7 +468,7 @@ PFACTORY_CREATE(PFactory<RTP_Encoding>, T38PseudoRTP_Handler, "udptl", false);
 /////////////////////////////////////////////////////////////////////////////
 
 OpalFaxEndPoint::OpalFaxEndPoint(OpalManager & mgr, const char * g711Prefix, const char * t38Prefix)
-  : OpalEndPoint(mgr, g711Prefix, CanTerminateCall)
+  : OpalLocalEndPoint(mgr, g711Prefix)
   , m_t38Prefix(t38Prefix)
   , m_defaultDirectory(".")
 {
@@ -481,7 +493,7 @@ PSafePtr<OpalConnection> OpalFaxEndPoint::MakeConnection(OpalCall & call,
 {
   if (!OpalMediaFormat(TIFF_File_FormatName).IsValid()) {
     PTRACE(1, "TIFF File format not valid! Missing plugin?");
-    return false;
+    return NULL;
   }
 
   PINDEX prefixLength = remoteParty.Find(':');
@@ -524,6 +536,12 @@ PSafePtr<OpalConnection> OpalFaxEndPoint::MakeConnection(OpalCall & call,
 }
 
 
+bool OpalFaxEndPoint::IsAvailable() const
+{
+  return OpalMediaFormat(OPAL_FAX_TIFF_FILE).IsValid();
+}
+
+
 OpalFaxConnection * OpalFaxEndPoint::CreateConnection(OpalCall & call,
                                                       void * /*userData*/,
                                                       OpalConnection::StringOptions * stringOptions,
@@ -544,17 +562,10 @@ OpalMediaFormatList OpalFaxEndPoint::GetMediaFormats() const
 }
 
 
-void OpalFaxEndPoint::AcceptIncomingConnection(const PString & token)
-{
-  PSafePtr<OpalFaxConnection> connection = PSafePtrCast<OpalConnection, OpalFaxConnection>(GetConnectionWithLock(token, PSafeReadOnly));
-  if (connection != NULL)
-    connection->AcceptIncoming();
-}
-
-
-void OpalFaxEndPoint::OnFaxCompleted(OpalFaxConnection & connection, bool failed)
+void OpalFaxEndPoint::OnFaxCompleted(OpalFaxConnection & connection, bool PTRACE_PARAM(failed))
 {
   PTRACE(3, "FAX\tFax " << (failed ? "failed" : "completed") << " on connection: " << connection);
+  connection.Release();
 }
 
 
@@ -566,19 +577,17 @@ OpalFaxConnection::OpalFaxConnection(OpalCall        & call,
                                      bool              receiving,
                                      bool              disableT38,
                                      OpalConnection::StringOptions * stringOptions)
-  : OpalConnection(call, ep, ep.GetManager().GetNextToken('F'), 0, stringOptions)
+  : OpalLocalConnection(call, ep, NULL, 0, stringOptions, 'F')
   , m_endpoint(ep)
   , m_filename(filename)
   , m_receiving(receiving)
   , m_disableT38(disableT38)
-  , m_releaseTimeout(0, 0, 1)
   , m_tiffFileFormat(TIFF_File_FormatName)
-  , m_awaitingSwitchToT38(!disableT38)
+  , m_state(disableT38 ? e_CompletedSwitch : e_AwaitingSwitchToT38)
 {
-  m_tiffFileFormat.SetOptionString("TIFF-File-Name", filename);
-  m_tiffFileFormat.SetOptionBoolean("Receiving", receiving);
+  SetFaxMediaFormatOptions(m_tiffFileFormat);
 
-  m_faxTimer.SetNotifier(PCREATE_NOTIFIER(OnSendCNGCED));
+  m_switchTimer.SetNotifier(PCREATE_NOTIFIER(OnSwitchTimeout));
 
   PTRACE(3, "FAX\tCreated FAX connection with token \"" << callToken << "\","
             " receiving=" << receiving << ","
@@ -599,22 +608,15 @@ PString OpalFaxConnection::GetPrefixName() const
 }
 
 
-void OpalFaxConnection::ApplyStringOptions(OpalConnection::StringOptions & stringOptions)
-{
-  m_stationId = stringOptions("stationid");
-  OpalConnection::ApplyStringOptions(stringOptions);
-}
-
-
 OpalMediaFormatList OpalFaxConnection::GetMediaFormats() const
 {
   OpalMediaFormatList formats;
 
-  if (!m_filename.IsEmpty())
+  if (m_filename.IsEmpty())
+    formats += OpalPCM16;
+  else
     formats += m_tiffFileFormat;
 
-  formats += OpalPCM16;
-
   if (!m_disableT38) {
     formats += OpalRFC2833;
     formats += OpalCiscoNSE;
@@ -624,78 +626,42 @@ OpalMediaFormatList OpalFaxConnection::GetMediaFormats() const
 }
 
 
-void OpalFaxConnection::AdjustMediaFormats(bool local, OpalMediaFormatList & mediaFormats, OpalConnection * otherConnection) const
+void OpalFaxConnection::AdjustMediaFormats(bool   local,
+                           const OpalConnection * otherConnection,
+                            OpalMediaFormatList & mediaFormats) const
 {
   // Remove everything but G.711 or fax stuff
-  OpalMediaFormatList::iterator i = mediaFormats.begin();
-  while (i != mediaFormats.end()) {
-    if ((m_awaitingSwitchToT38 && i->GetMediaType() == OpalMediaType::Audio()) ||
-        (*i == OpalG711_ULAW_64K || *i == OpalG711_ALAW_64K || *i == OpalRFC2833 || *i == OpalCiscoNSE))
-      ++i;
-    else if (i->GetMediaType() != OpalMediaType::Fax() || (m_disableT38 && *i == OpalT38))
-      mediaFormats -= *i++;
-    else {
-      i->SetOptionString("TIFF-File-Name", m_filename);
-      i->SetOptionBoolean("Receiving", m_receiving);
-      ++i;
-    }
+  OpalMediaFormatList::iterator it = mediaFormats.begin();
+  while (it != mediaFormats.end()) {
+    if ((m_state != e_SwitchingToT38 && it->GetMediaType() == OpalMediaType::Audio()) ||
+        (*it == OpalG711_ULAW_64K || *it == OpalG711_ALAW_64K || *it == OpalRFC2833 || *it == OpalCiscoNSE))
+      ++it;
+    else if (it->GetMediaType() != OpalMediaType::Fax() || (m_disableT38 && *it == OpalT38))
+      mediaFormats -= *it++;
+    else
+      SetFaxMediaFormatOptions(*it++);
   }
 
-  OpalConnection::AdjustMediaFormats(local, mediaFormats, otherConnection);
+  OpalConnection::AdjustMediaFormats(local, otherConnection, mediaFormats);
 }
 
 
-PBoolean OpalFaxConnection::SetUpConnection()
+void OpalFaxConnection::SetFaxMediaFormatOptions(OpalMediaFormat & mediaFormat) const
 {
-  // Check if we are A-Party in this call, so need to do things differently
-  if (ownerCall.GetConnection(0) == this) {
-    SetPhase(SetUpPhase);
+  mediaFormat.SetOptionString("TIFF-File-Name", m_filename);
+  mediaFormat.SetOptionBoolean("Receiving", m_receiving);
 
-    if (!OnIncomingConnection(0, NULL)) {
-      Release(EndedByCallerAbort);
-      return false;
-    }
-
-    PTRACE(2, "FAX\tOutgoing call routed to " << ownerCall.GetPartyB() << " for " << *this);
-    if (!ownerCall.OnSetUp(*this)) {
-      Release(EndedByNoAccept);
-      return false;
-    }
-
-    return true;
+  PString str = m_stringOptions(OPAL_OPT_STATION_ID);
+  if (!str.IsEmpty()) {
+    mediaFormat.SetOptionString("Station-Identifier", str);
+    PTRACE(4, "FAX\tSet Station-Identifier: \"" << str << '"');
   }
 
-  PTRACE(3, "FAX\tSetUpConnection(" << remotePartyName << ')');
-  SetPhase(AlertingPhase);
-  OnAlerting();
-
-  OnConnectedInternal();
-
-  // Do not need to do audio mode initially if not going to T.38, already there!
-  if (m_disableT38)
-    return true;
-
-  if (GetMediaStream(PString::Empty(), true) == NULL)
-    ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
-
-  return true;
-}
-
-
-PBoolean OpalFaxConnection::SetAlerting(const PString & calleeName, PBoolean)
-{
-  PTRACE(3, "Fax\tSetAlerting(" << calleeName << ')');
-  SetPhase(AlertingPhase);
-  remotePartyName = calleeName;
-  return true;
-}
-
-
-PBoolean OpalFaxConnection::SetConnected()
-{
-  if (GetMediaStream(PString::Empty(), true) == NULL)
-    ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
-  return OpalConnection::SetConnected();
+  str = m_stringOptions(OPAL_OPT_HEADER_INFO);
+  if (!str.IsEmpty()) {
+    mediaFormat.SetOptionString("Header-Info", str);
+    PTRACE(4, "FAX\tSet Header-Info: \"" << str << '"');
+  }
 }
 
 
@@ -705,16 +671,19 @@ void OpalFaxConnection::OnEstablished()
 
   // If switched and we don't need to do CNG/CED any more, or T.38 is disabled
   // in which case the SpanDSP will deal with CNG/CED stuff.
-  if (m_awaitingSwitchToT38) {
-    m_faxTimer.SetInterval(1000);
-    PTRACE(3, "T38\tStarting timer for CNG/CED tone");
+  if (m_state == e_AwaitingSwitchToT38) {
+    PString str = m_stringOptions(OPAL_T38_SWITCH_TIME);
+    if (!str.IsEmpty()) {
+      m_switchTimer.SetInterval(0, str.AsUnsigned());
+      PTRACE(3, "FAX\tStarting timer for auto-switch to T.38");
+    }
   }
 }
 
 
 void OpalFaxConnection::OnReleased()
 {
-  m_faxTimer.Stop(false);
+  m_switchTimer.Stop(false);
   OpalConnection::OnReleased();
 }
 
@@ -729,8 +698,11 @@ void OpalFaxConnection::OnStartMediaPatch(OpalMediaPatch & patch)
 {
   // Have switched to T.38 mode
   if (patch.GetSink()->GetMediaFormat() == OpalT38) {
-    m_faxTimer.Stop(false);
-    m_awaitingSwitchToT38 = false;
+    m_switchTimer.Stop(false);
+    m_state = e_CompletedSwitch;
+    m_finalStatistics.m_fax.m_result = OpalMediaStatistics::FaxNotStarted;
+    PTRACE(4, "FAX\tStarted fax media stream for " << m_tiffFileFormat
+           << " state=" << m_state << " switch=" << m_faxMediaStreamsSwitchState);
   }
 
   OpalConnection::OnStartMediaPatch(patch);
@@ -742,19 +714,21 @@ void OpalFaxConnection::OnStopMediaPatch(OpalMediaPatch & patch)
   // Finished the fax transmission, look for TIFF
   OpalMediaStream & source = patch.GetSource();
   if (source.GetMediaFormat() == m_tiffFileFormat) {
-    m_faxTimer.Stop();
+    m_switchTimer.Stop(false);
 
-    // Look for other sink, not the one from this patch
-    OpalMediaStreamPtr sink = GetMediaStream(source.GetID(), !source.IsSource());
-    if (sink != NULL)
-      sink->ExecuteCommand(OpalFaxTerminate());
-    else
-      source.ExecuteCommand(OpalFaxTerminate());
+    PTRACE(4, "FAX\tStopped fax media stream for " << m_tiffFileFormat
+           << " state=" << m_state << " switch=" << m_faxMediaStreamsSwitchState);
 
     // Not an explicit switch, so fax plug in indicated end of fax
-    if (m_faxMediaStreamsSwitchState == e_NotSwitchingFaxMediaStreams) {
-      OnFaxCompleted(false);
-      PThread::Create(PCREATE_NOTIFIER(ReleaseConnection));
+    if (m_state == e_CompletedSwitch && m_faxMediaStreamsSwitchState == e_NotSwitchingFaxMediaStreams) {
+      synchronousOnRelease = false;
+#if OPAL_STATISTICS
+      InternalGetStatistics(m_finalStatistics, true);
+      PTRACE(3, "FAX\tGot final statistics: result=" << m_finalStatistics.m_fax.m_result);
+      OnFaxCompleted(m_finalStatistics.m_fax.m_result != OpalMediaStatistics::FaxSuccessful);
+#else
+      OnFaxCompleted(true);
+#endif
     }
   }
 
@@ -771,23 +745,16 @@ PBoolean OpalFaxConnection::SendUserInputTone(char tone, unsigned duration)
 
 void OpalFaxConnection::OnUserInputTone(char tone, unsigned /*duration*/)
 {
-  // Not yet switched and got a CED from the remote system, start switch
-  if (m_awaitingSwitchToT38 && !m_receiving && toupper(tone) == 'Y') {
-    PTRACE(3, "T38\tRequesting mode change in response to CED");
+  // Not yet switched and got a CNG/CED from the remote system, start switch
+  if (m_state == e_AwaitingSwitchToT38 &&
+         (m_receiving ? (tone == 'X')
+                      : (tone == 'Y' && m_stringOptions.GetBoolean(OPAL_SWITCH_ON_CED)))) {
+    PTRACE(3, "FAX\tRequesting mode change in response to " << (tone == 'X' ? "CNG" : "CED"));
     PThread::Create(PCREATE_NOTIFIER(OpenFaxStreams));
   }
 }
 
 
-void OpalFaxConnection::AcceptIncoming()
-{
-  if (LockReadWrite()) {
-    OnConnectedInternal();
-    UnlockReadWrite();
-  }
-}
-
-
 void OpalFaxConnection::OnFaxCompleted(bool failed)
 {
   m_endpoint.OnFaxCompleted(*this, failed);
@@ -797,48 +764,52 @@ void OpalFaxConnection::OnFaxCompleted(bool failed)
 }
 
 
+#if OPAL_STATISTICS
+
 void OpalFaxConnection::GetStatistics(OpalMediaStatistics & statistics) const
 {
+  InternalGetStatistics(statistics, false);
+}
+
+
+void OpalFaxConnection::InternalGetStatistics(OpalMediaStatistics & statistics, bool terminate) const
+{
+  if (m_finalStatistics.m_fax.m_result >= 0) {
+    statistics = m_finalStatistics;
+    return;
+  }
+
   OpalMediaStreamPtr stream;
   if ((stream = GetMediaStream(OpalMediaType::Fax(), false)) == NULL &&
       (stream = GetMediaStream(OpalMediaType::Fax(), true )) == NULL) {
 
     PSafePtr<OpalConnection> other = GetOtherPartyConnection();
-    if (other == NULL)
+    if (other == NULL) {
+      PTRACE(2, "FAX\tNo connection to get statistics.");
       return;
+    }
 
     if ((stream = other->GetMediaStream(OpalMediaType::Fax(), false)) == NULL &&
-        (stream = other->GetMediaStream(OpalMediaType::Fax(), true )) == NULL)
+        (stream = other->GetMediaStream(OpalMediaType::Fax(), true )) == NULL) {
+      PTRACE(2, "FAX\tNo stream to get statistics.");
       return;
+    }
   }
 
+  if (terminate)
+    stream->ExecuteCommand(OpalFaxTerminate());
+
   stream->GetStatistics(statistics);
 }
 
+#endif
 
-void OpalFaxConnection::OnSendCNGCED(PTimer &, INT)
+
+void OpalFaxConnection::OnSwitchTimeout(PTimer &, INT)
 {
-  if (m_awaitingSwitchToT38 && LockReadOnly()) {
-    PTimeInterval elapsed = PTime() - connectedTime;
-    if (m_releaseTimeout > 0 && elapsed > m_releaseTimeout) {
-      PTRACE(2, "T38\tDid not switch to T.38 mode, releasing connection");
-      Release(OpalConnection::EndedByCapabilityExchange);
-    }
-    else if (m_receiving) {
-      PTRACE(2, "T38\tSwitching to T.38 mode");
-      PThread::Create(PCREATE_NOTIFIER(OpenFaxStreams));
-    }
-    else
-    if (m_switchTimeout > 0 && elapsed > m_switchTimeout) {
-      PTRACE(2, "T38\tDid not switch to T.38 mode, forcing switch");
-      PThread::Create(PCREATE_NOTIFIER(OpenFaxStreams));
-    }
-    else {
-      // Cadence for CNG is 500ms on 3 seconds off
-      OpalConnection::OnUserInputTone('X', 500);
-      m_faxTimer = 3500;
-    }
-    UnlockReadOnly();
+  if (m_state == e_AwaitingSwitchToT38) {
+    PTRACE(2, "FAX\tDid not switch to T.38 mode, forcing switch");
+    PThread::Create(PCREATE_NOTIFIER(OpenFaxStreams));
   }
 }
 
@@ -849,43 +820,36 @@ bool OpalFaxConnection::SwitchFaxMediaStreams(bool enableFax)
   if (other != NULL && other->SwitchFaxMediaStreams(enableFax))
     return true;
 
-  PTRACE(1, "T38\tMode change request to " << (enableFax ? "fax" : "audio") << " failed");
+  PTRACE(1, "FAX\tMode change request to " << (enableFax ? "fax" : "audio") << " failed");
   return false;
 }
 
 
 void OpalFaxConnection::OnSwitchedFaxMediaStreams(bool enabledFax)
 {
-  m_awaitingSwitchToT38 = false;
-
   if (enabledFax) {
-    PTRACE(3, "T38\tMode change request to fax succeeded");
+    PTRACE(3, "FAX\tMode change request to fax succeeded");
   }
   else {
-    PTRACE(4, "T38\tMode change request to fax failed, falling back to G.711");
-    m_disableT38 = true;
-    SwitchFaxMediaStreams(false);
+    PTRACE(4, "FAX\tMode change request to fax failed, falling back to G.711");
+    if (m_stringOptions.GetBoolean(OPAL_NO_G111_FAX))
+      OnFaxCompleted(true);
+    else {
+      m_disableT38 = true;
+      SwitchFaxMediaStreams(false);
+    }
   }
+
+  m_state = e_CompletedSwitch;
 }
 
 
 void OpalFaxConnection::OpenFaxStreams(PThread &, INT)
 {
   if (LockReadWrite()) {
-    m_awaitingSwitchToT38 = false;
-    if (!SwitchFaxMediaStreams(true)) {
-      OnFaxCompleted(true);
-      Release(OpalConnection::EndedByCapabilityExchange);
-    }
-    UnlockReadWrite();
-  }
-}
-
-
-void OpalFaxConnection::ReleaseConnection(PThread &, INT)
-{
-  if (LockReadWrite()) {
-    Release(OpalConnection::EndedByLocalUser);
+    m_state = e_SwitchingToT38;
+    if (!SwitchFaxMediaStreams(true))
+      m_state = e_CompletedSwitch; // Couldn't, don't try again.
     UnlockReadWrite();
   }
 }
